#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   Issue #350 is still open: on Windows, the code here causes it to link
   with PYTHON36.DLL (for example) instead of PYTHON3.DLL.  A fix was
   attempted in 164e526a5515 and 14ce6985e1c3, but reverted: virtualenv
   does not make PYTHON3.DLL available, and so the "correctly" compiled
   version would not run inside a virtualenv.  We will re-apply the fix
   after virtualenv has been fixed for some time.  For explanation, see
   issue #355.  For a workaround if you want PYTHON3.DLL and don't worry
   about virtualenv, see issue #350.  See also 'py_limited_api' in
   setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  include <pyconfig.h>
#  if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG)
#    define Py_LIMITED_API
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/


    #include <notcurses/direct.h>
    #include <notcurses/notcurses.h>
    

/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // _Bool()(struct ncdirect const *)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 822), // struct ncdirect const *
/*  2 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  3 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // _Bool()(struct ncmenu *, ncinput const *)
/*  4 */ _CFFI_OP(_CFFI_OP_POINTER, 825), // struct ncmenu *
/*  5 */ _CFFI_OP(_CFFI_OP_POINTER, 809), // ncinput const *
/*  6 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // _Bool()(struct ncmultiselector *, ncinput const *)
/*  8 */ _CFFI_OP(_CFFI_OP_POINTER, 832), // struct ncmultiselector *
/*  9 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 10 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 11 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // _Bool()(struct ncplane *, _Bool)
/* 12 */ _CFFI_OP(_CFFI_OP_POINTER, 833), // struct ncplane *
/* 13 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 1), // _Bool
/* 14 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 15 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // _Bool()(struct ncplane const *, int *, int *)
/* 16 */ _CFFI_OP(_CFFI_OP_POINTER, 833), // struct ncplane const *
/* 17 */ _CFFI_OP(_CFFI_OP_POINTER, 38), // int *
/* 18 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 19 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 20 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // _Bool()(struct ncreader *, ncinput const *)
/* 21 */ _CFFI_OP(_CFFI_OP_POINTER, 834), // struct ncreader *
/* 22 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 23 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 24 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // _Bool()(struct ncselector *, ncinput const *)
/* 25 */ _CFFI_OP(_CFFI_OP_POINTER, 836), // struct ncselector *
/* 26 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 27 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 28 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // _Bool()(struct notcurses const *)
/* 29 */ _CFFI_OP(_CFFI_OP_POINTER, 844), // struct notcurses const *
/* 30 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 31 */ _CFFI_OP(_CFFI_OP_FUNCTION, 86), // char *()(struct ncplane *, uint16_t *, uint64_t *)
/* 32 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 33 */ _CFFI_OP(_CFFI_OP_POINTER, 846), // uint16_t *
/* 34 */ _CFFI_OP(_CFFI_OP_POINTER, 113), // uint64_t *
/* 35 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 36 */ _CFFI_OP(_CFFI_OP_FUNCTION, 86), // char *()(struct ncplane const *, int, int, int, int)
/* 37 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 38 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/* 39 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 40 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 41 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 42 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 43 */ _CFFI_OP(_CFFI_OP_FUNCTION, 86), // char *()(struct ncplane const *, int, int, uint16_t *, uint64_t *)
/* 44 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 45 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 46 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 47 */ _CFFI_OP(_CFFI_OP_NOOP, 33),
/* 48 */ _CFFI_OP(_CFFI_OP_NOOP, 34),
/* 49 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 50 */ _CFFI_OP(_CFFI_OP_FUNCTION, 86), // char *()(struct ncreader const *)
/* 51 */ _CFFI_OP(_CFFI_OP_POINTER, 834), // struct ncreader const *
/* 52 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 53 */ _CFFI_OP(_CFFI_OP_FUNCTION, 86), // char *()(struct ncvisual const *)
/* 54 */ _CFFI_OP(_CFFI_OP_POINTER, 842), // struct ncvisual const *
/* 55 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 56 */ _CFFI_OP(_CFFI_OP_FUNCTION, 86), // char *()(struct notcurses *, int, int, uint16_t *, uint64_t *)
/* 57 */ _CFFI_OP(_CFFI_OP_POINTER, 844), // struct notcurses *
/* 58 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 59 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 60 */ _CFFI_OP(_CFFI_OP_NOOP, 33),
/* 61 */ _CFFI_OP(_CFFI_OP_NOOP, 34),
/* 62 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 63 */ _CFFI_OP(_CFFI_OP_FUNCTION, 92), // char const *()(ncblitter_e)
/* 64 */ _CFFI_OP(_CFFI_OP_ENUM, 1), // ncblitter_e
/* 65 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 66 */ _CFFI_OP(_CFFI_OP_FUNCTION, 92), // char const *()(ncscale_e)
/* 67 */ _CFFI_OP(_CFFI_OP_ENUM, 3), // ncscale_e
/* 68 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 69 */ _CFFI_OP(_CFFI_OP_FUNCTION, 92), // char const *()(struct ncmenu const *, ncinput *)
/* 70 */ _CFFI_OP(_CFFI_OP_POINTER, 825), // struct ncmenu const *
/* 71 */ _CFFI_OP(_CFFI_OP_POINTER, 809), // ncinput *
/* 72 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 73 */ _CFFI_OP(_CFFI_OP_FUNCTION, 92), // char const *()(struct ncplane const *, cell const *)
/* 74 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 75 */ _CFFI_OP(_CFFI_OP_POINTER, 804), // cell const *
/* 76 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 77 */ _CFFI_OP(_CFFI_OP_FUNCTION, 92), // char const *()(struct ncselector *)
/* 78 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 79 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 80 */ _CFFI_OP(_CFFI_OP_FUNCTION, 92), // char const *()(struct ncselector const *)
/* 81 */ _CFFI_OP(_CFFI_OP_POINTER, 836), // struct ncselector const *
/* 82 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 83 */ _CFFI_OP(_CFFI_OP_FUNCTION, 92), // char const *()(uintmax_t, unsigned int, char *, int, unsigned int, int)
/* 84 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 47), // uintmax_t
/* 85 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8), // unsigned int
/* 86 */ _CFFI_OP(_CFFI_OP_POINTER, 275), // char *
/* 87 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 88 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 89 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 90 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 91 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(char const *)
/* 92 */ _CFFI_OP(_CFFI_OP_POINTER, 275), // char const *
/* 93 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 94 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(char const *, ncblitter_e *)
/* 95 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 96 */ _CFFI_OP(_CFFI_OP_POINTER, 64), // ncblitter_e *
/* 97 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 98 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(char const *, ncscale_e *)
/* 99 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 100 */ _CFFI_OP(_CFFI_OP_POINTER, 67), // ncscale_e *
/* 101 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 102 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(char const *, notcurses_options *)
/* 103 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 104 */ _CFFI_OP(_CFFI_OP_POINTER, 820), // notcurses_options *
/* 105 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 106 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdirect *)
/* 107 */ _CFFI_OP(_CFFI_OP_POINTER, 822), // struct ncdirect *
/* 108 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 109 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdirect *, char const *, int, uint64_t, uint64_t)
/* 110 */ _CFFI_OP(_CFFI_OP_NOOP, 107),
/* 111 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 112 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 113 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24), // uint64_t
/* 114 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 115 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 116 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdirect *, char const *, ncalign_e, ncblitter_e, ncscale_e)
/* 117 */ _CFFI_OP(_CFFI_OP_NOOP, 107),
/* 118 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 119 */ _CFFI_OP(_CFFI_OP_ENUM, 0), // ncalign_e
/* 120 */ _CFFI_OP(_CFFI_OP_NOOP, 64),
/* 121 */ _CFFI_OP(_CFFI_OP_NOOP, 67),
/* 122 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 123 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdirect *, int)
/* 124 */ _CFFI_OP(_CFFI_OP_NOOP, 107),
/* 125 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 126 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 127 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdirect *, int, int)
/* 128 */ _CFFI_OP(_CFFI_OP_NOOP, 107),
/* 129 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 130 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 131 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 132 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdirect *, uint64_t, char const *)
/* 133 */ _CFFI_OP(_CFFI_OP_NOOP, 107),
/* 134 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 135 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 136 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 137 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdirect *, uint64_t, uint64_t, uint64_t, uint64_t, int, int, unsigned int)
/* 138 */ _CFFI_OP(_CFFI_OP_NOOP, 107),
/* 139 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 140 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 141 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 142 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 143 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 144 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 145 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 146 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 147 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdirect *, uint64_t, uint64_t, uint64_t, uint64_t, wchar_t const *, int, int, unsigned int)
/* 148 */ _CFFI_OP(_CFFI_OP_NOOP, 107),
/* 149 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 150 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 151 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 152 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 153 */ _CFFI_OP(_CFFI_OP_POINTER, 852), // wchar_t const *
/* 154 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 155 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 156 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 157 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 158 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdirect *, unsigned int)
/* 159 */ _CFFI_OP(_CFFI_OP_NOOP, 107),
/* 160 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 161 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 162 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdirect *, unsigned int, unsigned int, unsigned int)
/* 163 */ _CFFI_OP(_CFFI_OP_NOOP, 107),
/* 164 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 165 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 166 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 167 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 168 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdirect const *)
/* 169 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 170 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 171 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdplot *, uint64_t, double)
/* 172 */ _CFFI_OP(_CFFI_OP_POINTER, 823), // struct ncdplot *
/* 173 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 174 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/* 175 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 176 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncdplot const *, uint64_t, double *)
/* 177 */ _CFFI_OP(_CFFI_OP_POINTER, 823), // struct ncdplot const *
/* 178 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 179 */ _CFFI_OP(_CFFI_OP_POINTER, 174), // double *
/* 180 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 181 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncfdplane *)
/* 182 */ _CFFI_OP(_CFFI_OP_POINTER, 824), // struct ncfdplane *
/* 183 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 184 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncfdplane *, int, void *)
/* 185 */ _CFFI_OP(_CFFI_OP_NOOP, 182),
/* 186 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 187 */ _CFFI_OP(_CFFI_OP_POINTER, 851), // void *
/* 188 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 189 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncfdplane *, void const *, size_t, void *)
/* 190 */ _CFFI_OP(_CFFI_OP_NOOP, 182),
/* 191 */ _CFFI_OP(_CFFI_OP_POINTER, 851), // void const *
/* 192 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28), // size_t
/* 193 */ _CFFI_OP(_CFFI_OP_NOOP, 187),
/* 194 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 195 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncmenu *)
/* 196 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 197 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 198 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncmenu *, int)
/* 199 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 200 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 201 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 202 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncmultiselector *, _Bool *, unsigned int)
/* 203 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 204 */ _CFFI_OP(_CFFI_OP_POINTER, 13), // _Bool *
/* 205 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 206 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 207 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *)
/* 208 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 209 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 210 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, cell *)
/* 211 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 212 */ _CFFI_OP(_CFFI_OP_POINTER, 804), // cell *
/* 213 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 214 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, cell *, cell const *)
/* 215 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 216 */ _CFFI_OP(_CFFI_OP_NOOP, 212),
/* 217 */ _CFFI_OP(_CFFI_OP_NOOP, 75),
/* 218 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 219 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, cell *, char const *)
/* 220 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 221 */ _CFFI_OP(_CFFI_OP_NOOP, 212),
/* 222 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 223 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 224 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, cell const *)
/* 225 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 226 */ _CFFI_OP(_CFFI_OP_NOOP, 75),
/* 227 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 228 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, cell const *, cell const *, cell const *, cell const *, cell const *, cell const *, int, int, unsigned int)
/* 229 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 230 */ _CFFI_OP(_CFFI_OP_NOOP, 75),
/* 231 */ _CFFI_OP(_CFFI_OP_NOOP, 75),
/* 232 */ _CFFI_OP(_CFFI_OP_NOOP, 75),
/* 233 */ _CFFI_OP(_CFFI_OP_NOOP, 75),
/* 234 */ _CFFI_OP(_CFFI_OP_NOOP, 75),
/* 235 */ _CFFI_OP(_CFFI_OP_NOOP, 75),
/* 236 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 237 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 238 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 239 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 240 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, cell const *, int, uint64_t, uint64_t)
/* 241 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 242 */ _CFFI_OP(_CFFI_OP_NOOP, 75),
/* 243 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 244 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 245 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 246 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 247 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, char const *)
/* 248 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 249 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 250 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 251 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, char const *, int *)
/* 252 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 253 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 254 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 255 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 256 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, char const *, uint32_t, uint64_t)
/* 257 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 258 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 259 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22), // uint32_t
/* 260 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 261 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 262 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, char const *, uint32_t, uint64_t, uint64_t, uint64_t, uint64_t, int, int)
/* 263 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 264 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 265 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 266 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 267 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 268 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 269 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 270 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 271 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 272 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 273 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, char)
/* 274 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 275 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 276 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 277 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, int)
/* 278 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 279 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 280 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 281 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, int, int)
/* 282 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 283 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 284 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 285 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 286 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, int, int, cell const *)
/* 287 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 288 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 289 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 290 */ _CFFI_OP(_CFFI_OP_NOOP, 75),
/* 291 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 292 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, int, int, char const *, int *)
/* 293 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 294 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 295 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 296 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 297 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 298 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 299 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, int, int, int)
/* 300 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 301 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 302 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 303 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 304 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 305 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, int, int, int, int, int, int, int, int)
/* 306 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 307 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 308 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 309 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 310 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 311 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 312 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 313 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 314 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 315 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 316 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, int, int, size_t, char const *)
/* 317 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 318 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 319 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 320 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 321 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 322 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 323 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, int, int, uint32_t)
/* 324 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 325 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 326 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 327 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 328 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 329 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, int, int, uint64_t, uint64_t, uint64_t, uint64_t)
/* 330 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 331 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 332 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 333 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 334 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 335 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 336 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 337 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 338 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, int, ncalign_e, char const *)
/* 339 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 340 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 341 */ _CFFI_OP(_CFFI_OP_NOOP, 119),
/* 342 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 343 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 344 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, int, ncalign_e, char const *, size_t *)
/* 345 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 346 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 347 */ _CFFI_OP(_CFFI_OP_NOOP, 119),
/* 348 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 349 */ _CFFI_OP(_CFFI_OP_POINTER, 192), // size_t *
/* 350 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 351 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, int, ncalign_e, size_t, char const *)
/* 352 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 353 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 354 */ _CFFI_OP(_CFFI_OP_NOOP, 119),
/* 355 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 356 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 357 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 358 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, ncblitter_e, int *, int *, void const *, size_t)
/* 359 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 360 */ _CFFI_OP(_CFFI_OP_NOOP, 64),
/* 361 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 362 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 363 */ _CFFI_OP(_CFFI_OP_NOOP, 191),
/* 364 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 365 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 366 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, struct ncplane *)
/* 367 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 368 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 369 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 370 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, struct timespec const *, int(*)(struct notcurses *, struct ncplane *, struct timespec const *, void *), void *)
/* 371 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 372 */ _CFFI_OP(_CFFI_OP_POINTER, 845), // struct timespec const *
/* 373 */ _CFFI_OP(_CFFI_OP_POINTER, 514), // int(*)(struct notcurses *, struct ncplane *, struct timespec const *, void *)
/* 374 */ _CFFI_OP(_CFFI_OP_NOOP, 187),
/* 375 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 376 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, uint32_t, uint32_t, uint32_t, uint32_t, int, int)
/* 377 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 378 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 379 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 380 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 381 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 382 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 383 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 384 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 385 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, unsigned int)
/* 386 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 387 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 388 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 389 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, wchar_t const *)
/* 390 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 391 */ _CFFI_OP(_CFFI_OP_NOOP, 153),
/* 392 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 393 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane *, wchar_t const *, int *)
/* 394 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 395 */ _CFFI_OP(_CFFI_OP_NOOP, 153),
/* 396 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 397 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 398 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane const *)
/* 399 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 400 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 401 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane const *, struct ncplane *)
/* 402 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 403 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 404 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 405 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncplane const *, struct ncplane *, int, int, int, int, int, int)
/* 406 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 407 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 408 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 409 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 410 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 411 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 412 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 413 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 414 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 415 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncreader *)
/* 416 */ _CFFI_OP(_CFFI_OP_NOOP, 21),
/* 417 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 418 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncreader *, char const *)
/* 419 */ _CFFI_OP(_CFFI_OP_NOOP, 21),
/* 420 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 421 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 422 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncreel *)
/* 423 */ _CFFI_OP(_CFFI_OP_POINTER, 835), // struct ncreel *
/* 424 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 425 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncreel *, struct nctablet *)
/* 426 */ _CFFI_OP(_CFFI_OP_NOOP, 423),
/* 427 */ _CFFI_OP(_CFFI_OP_POINTER, 840), // struct nctablet *
/* 428 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 429 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncreel const *)
/* 430 */ _CFFI_OP(_CFFI_OP_POINTER, 835), // struct ncreel const *
/* 431 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 432 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncselector *, char const *)
/* 433 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 434 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 435 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 436 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncselector *, struct ncselector_item const *)
/* 437 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 438 */ _CFFI_OP(_CFFI_OP_POINTER, 838), // struct ncselector_item const *
/* 439 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 440 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncsubproc *)
/* 441 */ _CFFI_OP(_CFFI_OP_POINTER, 839), // struct ncsubproc *
/* 442 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 443 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct nctablet *, _Bool)
/* 444 */ _CFFI_OP(_CFFI_OP_NOOP, 427),
/* 445 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 1),
/* 446 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 447 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncuplot *, uint64_t, uint64_t)
/* 448 */ _CFFI_OP(_CFFI_OP_POINTER, 841), // struct ncuplot *
/* 449 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 450 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 451 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 452 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncuplot const *, uint64_t, uint64_t *)
/* 453 */ _CFFI_OP(_CFFI_OP_POINTER, 841), // struct ncuplot const *
/* 454 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 455 */ _CFFI_OP(_CFFI_OP_NOOP, 34),
/* 456 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 457 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncvisual *)
/* 458 */ _CFFI_OP(_CFFI_OP_POINTER, 842), // struct ncvisual *
/* 459 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 460 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncvisual *, double)
/* 461 */ _CFFI_OP(_CFFI_OP_NOOP, 458),
/* 462 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 463 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 464 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncvisual *, int, int)
/* 465 */ _CFFI_OP(_CFFI_OP_NOOP, 458),
/* 466 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 467 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 468 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 469 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncvisual *, int, int, uint32_t)
/* 470 */ _CFFI_OP(_CFFI_OP_NOOP, 458),
/* 471 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 472 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 473 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 474 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 475 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncvisual *, struct ncvisual_options *, struct timespec const *, void *)
/* 476 */ _CFFI_OP(_CFFI_OP_NOOP, 458),
/* 477 */ _CFFI_OP(_CFFI_OP_POINTER, 843), // struct ncvisual_options *
/* 478 */ _CFFI_OP(_CFFI_OP_NOOP, 372),
/* 479 */ _CFFI_OP(_CFFI_OP_NOOP, 187),
/* 480 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 481 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncvisual const *, int, int, uint32_t *)
/* 482 */ _CFFI_OP(_CFFI_OP_NOOP, 54),
/* 483 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 484 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 485 */ _CFFI_OP(_CFFI_OP_POINTER, 259), // uint32_t *
/* 486 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 487 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct ncvisual const *, int, int, uint32_t)
/* 488 */ _CFFI_OP(_CFFI_OP_NOOP, 54),
/* 489 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 490 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 491 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 492 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 493 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct notcurses *)
/* 494 */ _CFFI_OP(_CFFI_OP_NOOP, 57),
/* 495 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 496 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct notcurses *, FILE *)
/* 497 */ _CFFI_OP(_CFFI_OP_NOOP, 57),
/* 498 */ _CFFI_OP(_CFFI_OP_POINTER, 803), // FILE *
/* 499 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 500 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct notcurses *, int *, int *)
/* 501 */ _CFFI_OP(_CFFI_OP_NOOP, 57),
/* 502 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 503 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 504 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 505 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct notcurses *, int, int)
/* 506 */ _CFFI_OP(_CFFI_OP_NOOP, 57),
/* 507 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 508 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 509 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 510 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct notcurses *, palette256 const *)
/* 511 */ _CFFI_OP(_CFFI_OP_NOOP, 57),
/* 512 */ _CFFI_OP(_CFFI_OP_POINTER, 821), // palette256 const *
/* 513 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 514 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct notcurses *, struct ncplane *, struct timespec const *, void *)
/* 515 */ _CFFI_OP(_CFFI_OP_NOOP, 57),
/* 516 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 517 */ _CFFI_OP(_CFFI_OP_NOOP, 372),
/* 518 */ _CFFI_OP(_CFFI_OP_NOOP, 187),
/* 519 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 520 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct notcurses *, struct ncvisual *, float, int(*)(struct ncvisual *, struct ncvisual_options *, struct timespec const *, void *), struct ncvisual_options const *, void *)
/* 521 */ _CFFI_OP(_CFFI_OP_NOOP, 57),
/* 522 */ _CFFI_OP(_CFFI_OP_NOOP, 458),
/* 523 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13), // float
/* 524 */ _CFFI_OP(_CFFI_OP_POINTER, 475), // int(*)(struct ncvisual *, struct ncvisual_options *, struct timespec const *, void *)
/* 525 */ _CFFI_OP(_CFFI_OP_POINTER, 843), // struct ncvisual_options const *
/* 526 */ _CFFI_OP(_CFFI_OP_NOOP, 187),
/* 527 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 528 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(struct notcurses const *, struct ncvisual const *, struct ncvisual_options const *, int *, int *, int *, int *)
/* 529 */ _CFFI_OP(_CFFI_OP_NOOP, 29),
/* 530 */ _CFFI_OP(_CFFI_OP_NOOP, 54),
/* 531 */ _CFFI_OP(_CFFI_OP_NOOP, 525),
/* 532 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 533 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 534 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 535 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 536 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 537 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // int()(void const *, int, struct ncvisual_options const *)
/* 538 */ _CFFI_OP(_CFFI_OP_NOOP, 191),
/* 539 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 540 */ _CFFI_OP(_CFFI_OP_NOOP, 525),
/* 541 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 542 */ _CFFI_OP(_CFFI_OP_FUNCTION, 737), // palette256 *()(struct notcurses *)
/* 543 */ _CFFI_OP(_CFFI_OP_NOOP, 57),
/* 544 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 545 */ _CFFI_OP(_CFFI_OP_FUNCTION, 107), // struct ncdirect *()(char const *, FILE *, uint64_t)
/* 546 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 547 */ _CFFI_OP(_CFFI_OP_NOOP, 498),
/* 548 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 549 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 550 */ _CFFI_OP(_CFFI_OP_FUNCTION, 172), // struct ncdplot *()(struct ncplane *, ncplot_options const *, double, double)
/* 551 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 552 */ _CFFI_OP(_CFFI_OP_POINTER, 814), // ncplot_options const *
/* 553 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 554 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 555 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 556 */ _CFFI_OP(_CFFI_OP_FUNCTION, 182), // struct ncfdplane *()(struct ncplane *, ncfdplane_options const *, int, int(*)(struct ncfdplane *, void const *, size_t, void *), int(*)(struct ncfdplane *, int, void *))
/* 557 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 558 */ _CFFI_OP(_CFFI_OP_POINTER, 808), // ncfdplane_options const *
/* 559 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 560 */ _CFFI_OP(_CFFI_OP_POINTER, 189), // int(*)(struct ncfdplane *, void const *, size_t, void *)
/* 561 */ _CFFI_OP(_CFFI_OP_POINTER, 184), // int(*)(struct ncfdplane *, int, void *)
/* 562 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 563 */ _CFFI_OP(_CFFI_OP_FUNCTION, 4), // struct ncmenu *()(struct ncplane *, ncmenu_options const *)
/* 564 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 565 */ _CFFI_OP(_CFFI_OP_POINTER, 811), // ncmenu_options const *
/* 566 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 567 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // struct ncmultiselector *()(struct ncplane *, ncmultiselector_options const *)
/* 568 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 569 */ _CFFI_OP(_CFFI_OP_POINTER, 812), // ncmultiselector_options const *
/* 570 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 571 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct ncdplot *)
/* 572 */ _CFFI_OP(_CFFI_OP_NOOP, 172),
/* 573 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 574 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct ncfdplane *)
/* 575 */ _CFFI_OP(_CFFI_OP_NOOP, 182),
/* 576 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 577 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct ncmultiselector *)
/* 578 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 579 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 580 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct ncplane *)
/* 581 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 582 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 583 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct ncplane *, ncplane_options const *)
/* 584 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 585 */ _CFFI_OP(_CFFI_OP_POINTER, 813), // ncplane_options const *
/* 586 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 587 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct ncplane *, struct ncplane *)
/* 588 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 589 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 590 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 591 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct ncplane const *, void *)
/* 592 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 593 */ _CFFI_OP(_CFFI_OP_NOOP, 187),
/* 594 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 595 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct ncreader *)
/* 596 */ _CFFI_OP(_CFFI_OP_NOOP, 21),
/* 597 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 598 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct ncreel *)
/* 599 */ _CFFI_OP(_CFFI_OP_NOOP, 423),
/* 600 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 601 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct ncselector *)
/* 602 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 603 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 604 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct ncsubproc *)
/* 605 */ _CFFI_OP(_CFFI_OP_NOOP, 441),
/* 606 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 607 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct nctablet *)
/* 608 */ _CFFI_OP(_CFFI_OP_NOOP, 427),
/* 609 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 610 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct ncuplot *)
/* 611 */ _CFFI_OP(_CFFI_OP_NOOP, 448),
/* 612 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 613 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct notcurses *)
/* 614 */ _CFFI_OP(_CFFI_OP_NOOP, 57),
/* 615 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 616 */ _CFFI_OP(_CFFI_OP_FUNCTION, 12), // struct ncplane *()(struct notcurses *, struct ncvisual *, struct ncvisual_options const *)
/* 617 */ _CFFI_OP(_CFFI_OP_NOOP, 57),
/* 618 */ _CFFI_OP(_CFFI_OP_NOOP, 458),
/* 619 */ _CFFI_OP(_CFFI_OP_NOOP, 525),
/* 620 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 621 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // struct ncplane const *()(struct ncplane const *)
/* 622 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 623 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 624 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // struct ncplane const *()(struct notcurses const *)
/* 625 */ _CFFI_OP(_CFFI_OP_NOOP, 29),
/* 626 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 627 */ _CFFI_OP(_CFFI_OP_FUNCTION, 21), // struct ncreader *()(struct ncplane *, ncreader_options const *)
/* 628 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 629 */ _CFFI_OP(_CFFI_OP_POINTER, 815), // ncreader_options const *
/* 630 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 631 */ _CFFI_OP(_CFFI_OP_FUNCTION, 423), // struct ncreel *()(struct ncplane *, ncreel_options const *)
/* 632 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 633 */ _CFFI_OP(_CFFI_OP_POINTER, 816), // ncreel_options const *
/* 634 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 635 */ _CFFI_OP(_CFFI_OP_FUNCTION, 25), // struct ncselector *()(struct ncplane *, ncselector_options const *)
/* 636 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 637 */ _CFFI_OP(_CFFI_OP_POINTER, 817), // ncselector_options const *
/* 638 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 639 */ _CFFI_OP(_CFFI_OP_FUNCTION, 441), // struct ncsubproc *()(struct ncplane *, ncsubproc_options const *, char const *, char * const *, char * const *, int(*)(struct ncfdplane *, void const *, size_t, void *), int(*)(struct ncfdplane *, int, void *))
/* 640 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 641 */ _CFFI_OP(_CFFI_OP_POINTER, 819), // ncsubproc_options const *
/* 642 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 643 */ _CFFI_OP(_CFFI_OP_POINTER, 86), // char * const *
/* 644 */ _CFFI_OP(_CFFI_OP_NOOP, 643),
/* 645 */ _CFFI_OP(_CFFI_OP_NOOP, 560),
/* 646 */ _CFFI_OP(_CFFI_OP_NOOP, 561),
/* 647 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 648 */ _CFFI_OP(_CFFI_OP_FUNCTION, 441), // struct ncsubproc *()(struct ncplane *, ncsubproc_options const *, char const *, char * const *, int(*)(struct ncfdplane *, void const *, size_t, void *), int(*)(struct ncfdplane *, int, void *))
/* 649 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 650 */ _CFFI_OP(_CFFI_OP_NOOP, 641),
/* 651 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 652 */ _CFFI_OP(_CFFI_OP_NOOP, 643),
/* 653 */ _CFFI_OP(_CFFI_OP_NOOP, 560),
/* 654 */ _CFFI_OP(_CFFI_OP_NOOP, 561),
/* 655 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 656 */ _CFFI_OP(_CFFI_OP_FUNCTION, 427), // struct nctablet *()(struct ncreel *)
/* 657 */ _CFFI_OP(_CFFI_OP_NOOP, 423),
/* 658 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 659 */ _CFFI_OP(_CFFI_OP_FUNCTION, 427), // struct nctablet *()(struct ncreel *, struct nctablet *, struct nctablet *, int(*)(struct nctablet *, _Bool), void *)
/* 660 */ _CFFI_OP(_CFFI_OP_NOOP, 423),
/* 661 */ _CFFI_OP(_CFFI_OP_NOOP, 427),
/* 662 */ _CFFI_OP(_CFFI_OP_NOOP, 427),
/* 663 */ _CFFI_OP(_CFFI_OP_POINTER, 443), // int(*)(struct nctablet *, _Bool)
/* 664 */ _CFFI_OP(_CFFI_OP_NOOP, 187),
/* 665 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 666 */ _CFFI_OP(_CFFI_OP_FUNCTION, 448), // struct ncuplot *()(struct ncplane *, ncplot_options const *, uint64_t, uint64_t)
/* 667 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 668 */ _CFFI_OP(_CFFI_OP_NOOP, 552),
/* 669 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 670 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 671 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 672 */ _CFFI_OP(_CFFI_OP_FUNCTION, 458), // struct ncvisual *()(char const *)
/* 673 */ _CFFI_OP(_CFFI_OP_NOOP, 92),
/* 674 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 675 */ _CFFI_OP(_CFFI_OP_FUNCTION, 458), // struct ncvisual *()(struct ncplane const *, ncblitter_e, int, int, int, int)
/* 676 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 677 */ _CFFI_OP(_CFFI_OP_NOOP, 64),
/* 678 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 679 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 680 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 681 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 682 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 683 */ _CFFI_OP(_CFFI_OP_FUNCTION, 458), // struct ncvisual *()(void const *, int, int, int)
/* 684 */ _CFFI_OP(_CFFI_OP_NOOP, 191),
/* 685 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 686 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 687 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 688 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 689 */ _CFFI_OP(_CFFI_OP_FUNCTION, 57), // struct notcurses *()(notcurses_options const *, FILE *)
/* 690 */ _CFFI_OP(_CFFI_OP_POINTER, 820), // notcurses_options const *
/* 691 */ _CFFI_OP(_CFFI_OP_NOOP, 498),
/* 692 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 693 */ _CFFI_OP(_CFFI_OP_FUNCTION, 846), // uint16_t()(struct ncplane const *)
/* 694 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 695 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 696 */ _CFFI_OP(_CFFI_OP_FUNCTION, 485), // uint32_t *()(struct ncplane const *, ncblitter_e, int, int, int, int)
/* 697 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 698 */ _CFFI_OP(_CFFI_OP_NOOP, 64),
/* 699 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 700 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 701 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 702 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 703 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 704 */ _CFFI_OP(_CFFI_OP_FUNCTION, 113), // uint64_t()(struct ncplane *, uint32_t)
/* 705 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 706 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 707 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 708 */ _CFFI_OP(_CFFI_OP_FUNCTION, 113), // uint64_t()(struct ncplane const *)
/* 709 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 710 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 711 */ _CFFI_OP(_CFFI_OP_FUNCTION, 85), // unsigned int()(struct ncdirect const *)
/* 712 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 713 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 714 */ _CFFI_OP(_CFFI_OP_FUNCTION, 85), // unsigned int()(struct ncplane const *)
/* 715 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 716 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 717 */ _CFFI_OP(_CFFI_OP_FUNCTION, 85), // unsigned int()(struct notcurses const *)
/* 718 */ _CFFI_OP(_CFFI_OP_NOOP, 29),
/* 719 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 720 */ _CFFI_OP(_CFFI_OP_FUNCTION, 187), // void *()(struct ncplane *)
/* 721 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 722 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 723 */ _CFFI_OP(_CFFI_OP_FUNCTION, 187), // void *()(struct ncplane *, void *)
/* 724 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 725 */ _CFFI_OP(_CFFI_OP_NOOP, 187),
/* 726 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 727 */ _CFFI_OP(_CFFI_OP_FUNCTION, 187), // void *()(struct nctablet *)
/* 728 */ _CFFI_OP(_CFFI_OP_NOOP, 427),
/* 729 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 730 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(int *, int *, int *, int *)
/* 731 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 732 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 733 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 734 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 735 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 736 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(palette256 *)
/* 737 */ _CFFI_OP(_CFFI_OP_POINTER, 821), // palette256 *
/* 738 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 739 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncdplot *)
/* 740 */ _CFFI_OP(_CFFI_OP_NOOP, 172),
/* 741 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 742 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncmultiselector *)
/* 743 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 744 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 745 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncplane *)
/* 746 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 747 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 748 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncplane *, cell *)
/* 749 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 750 */ _CFFI_OP(_CFFI_OP_NOOP, 212),
/* 751 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 752 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncplane *, int *, int *)
/* 753 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 754 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 755 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 756 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 757 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncplane *, int, int, int)
/* 758 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 759 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 760 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 761 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 762 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 763 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncplane *, uint64_t)
/* 764 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 765 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 766 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 767 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncplane *, unsigned int)
/* 768 */ _CFFI_OP(_CFFI_OP_NOOP, 12),
/* 769 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 770 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 771 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncplane const *, int *, int *)
/* 772 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 773 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 774 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 775 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 776 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncplane const *, struct ncplane const *, int *, int *)
/* 777 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 778 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 779 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 780 */ _CFFI_OP(_CFFI_OP_NOOP, 17),
/* 781 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 782 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncreader *, char * *)
/* 783 */ _CFFI_OP(_CFFI_OP_NOOP, 21),
/* 784 */ _CFFI_OP(_CFFI_OP_POINTER, 86), // char * *
/* 785 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 786 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncselector *, char * *)
/* 787 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 788 */ _CFFI_OP(_CFFI_OP_NOOP, 784),
/* 789 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 790 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncuplot *)
/* 791 */ _CFFI_OP(_CFFI_OP_NOOP, 448),
/* 792 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 793 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct ncvisual *)
/* 794 */ _CFFI_OP(_CFFI_OP_NOOP, 458),
/* 795 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 796 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct notcurses *)
/* 797 */ _CFFI_OP(_CFFI_OP_NOOP, 57),
/* 798 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 799 */ _CFFI_OP(_CFFI_OP_FUNCTION, 851), // void()(struct notcurses *, ncstats *)
/* 800 */ _CFFI_OP(_CFFI_OP_NOOP, 57),
/* 801 */ _CFFI_OP(_CFFI_OP_POINTER, 818), // ncstats *
/* 802 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 803 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // FILE
/* 804 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // cell
/* 805 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 51), // char32_t
/* 806 */ _CFFI_OP(_CFFI_OP_POINTER, 207), // int(*)(struct ncplane *)
/* 807 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 23), // int64_t
/* 808 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 6), // ncfdplane_options
/* 809 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 7), // ncinput
/* 810 */ _CFFI_OP(_CFFI_OP_ENUM, 2), // ncloglevel_e
/* 811 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 10), // ncmenu_options
/* 812 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 14), // ncmultiselector_options
/* 813 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 16), // ncplane_options
/* 814 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 17), // ncplot_options
/* 815 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 19), // ncreader_options
/* 816 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 21), // ncreel_options
/* 817 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 24), // ncselector_options
/* 818 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 25), // ncstats
/* 819 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 27), // ncsubproc_options
/* 820 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 33), // notcurses_options
/* 821 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 34), // palette256
/* 822 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // struct ncdirect
/* 823 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 4), // struct ncdplot
/* 824 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 5), // struct ncfdplane
/* 825 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 8), // struct ncmenu
/* 826 */ _CFFI_OP(_CFFI_OP_POINTER, 827), // struct ncmenu_item *
/* 827 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 9), // struct ncmenu_item
/* 828 */ _CFFI_OP(_CFFI_OP_POINTER, 829), // struct ncmenu_section *
/* 829 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 11), // struct ncmenu_section
/* 830 */ _CFFI_OP(_CFFI_OP_POINTER, 831), // struct ncmselector_item *
/* 831 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 12), // struct ncmselector_item
/* 832 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 13), // struct ncmultiselector
/* 833 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 15), // struct ncplane
/* 834 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 18), // struct ncreader
/* 835 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 20), // struct ncreel
/* 836 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 22), // struct ncselector
/* 837 */ _CFFI_OP(_CFFI_OP_POINTER, 838), // struct ncselector_item *
/* 838 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 23), // struct ncselector_item
/* 839 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 26), // struct ncsubproc
/* 840 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 28), // struct nctablet
/* 841 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 29), // struct ncuplot
/* 842 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 30), // struct ncvisual
/* 843 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 31), // struct ncvisual_options
/* 844 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 32), // struct notcurses
/* 845 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 35), // struct timespec
/* 846 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 20), // uint16_t
/* 847 */ _CFFI_OP(_CFFI_OP_ARRAY, 259), // uint32_t[256]
/* 848 */ (_cffi_opcode_t)(256),
/* 849 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 18), // uint8_t
/* 850 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // union $1
/* 851 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
/* 852 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 16), // wchar_t
};

static int _cffi_const_NCALIGN_LEFT(unsigned long long *o)
{
  int n = (NCALIGN_LEFT) <= 0;
  *o = (unsigned long long)((NCALIGN_LEFT) | 0);  /* check that NCALIGN_LEFT is an integer */
  return n;
}

static int _cffi_const_NCALIGN_CENTER(unsigned long long *o)
{
  int n = (NCALIGN_CENTER) <= 0;
  *o = (unsigned long long)((NCALIGN_CENTER) | 0);  /* check that NCALIGN_CENTER is an integer */
  return n;
}

static int _cffi_const_NCALIGN_RIGHT(unsigned long long *o)
{
  int n = (NCALIGN_RIGHT) <= 0;
  *o = (unsigned long long)((NCALIGN_RIGHT) | 0);  /* check that NCALIGN_RIGHT is an integer */
  return n;
}

static int _cffi_const_NCBLIT_1x1(unsigned long long *o)
{
  int n = (NCBLIT_1x1) <= 0;
  *o = (unsigned long long)((NCBLIT_1x1) | 0);  /* check that NCBLIT_1x1 is an integer */
  return n;
}

static int _cffi_const_NCBLIT_2x1(unsigned long long *o)
{
  int n = (NCBLIT_2x1) <= 0;
  *o = (unsigned long long)((NCBLIT_2x1) | 0);  /* check that NCBLIT_2x1 is an integer */
  return n;
}

static int _cffi_const_NCBLIT_1x1x4(unsigned long long *o)
{
  int n = (NCBLIT_1x1x4) <= 0;
  *o = (unsigned long long)((NCBLIT_1x1x4) | 0);  /* check that NCBLIT_1x1x4 is an integer */
  return n;
}

static int _cffi_const_NCBLIT_2x2(unsigned long long *o)
{
  int n = (NCBLIT_2x2) <= 0;
  *o = (unsigned long long)((NCBLIT_2x2) | 0);  /* check that NCBLIT_2x2 is an integer */
  return n;
}

static int _cffi_const_NCBLIT_4x1(unsigned long long *o)
{
  int n = (NCBLIT_4x1) <= 0;
  *o = (unsigned long long)((NCBLIT_4x1) | 0);  /* check that NCBLIT_4x1 is an integer */
  return n;
}

static int _cffi_const_NCBLIT_BRAILLE(unsigned long long *o)
{
  int n = (NCBLIT_BRAILLE) <= 0;
  *o = (unsigned long long)((NCBLIT_BRAILLE) | 0);  /* check that NCBLIT_BRAILLE is an integer */
  return n;
}

static int _cffi_const_NCBLIT_8x1(unsigned long long *o)
{
  int n = (NCBLIT_8x1) <= 0;
  *o = (unsigned long long)((NCBLIT_8x1) | 0);  /* check that NCBLIT_8x1 is an integer */
  return n;
}

static int _cffi_const_NCBLIT_SIXEL(unsigned long long *o)
{
  int n = (NCBLIT_SIXEL) <= 0;
  *o = (unsigned long long)((NCBLIT_SIXEL) | 0);  /* check that NCBLIT_SIXEL is an integer */
  return n;
}

static int _cffi_const_NCLOGLEVEL_SILENT(unsigned long long *o)
{
  int n = (NCLOGLEVEL_SILENT) <= 0;
  *o = (unsigned long long)((NCLOGLEVEL_SILENT) | 0);  /* check that NCLOGLEVEL_SILENT is an integer */
  return n;
}

static int _cffi_const_NCLOGLEVEL_PANIC(unsigned long long *o)
{
  int n = (NCLOGLEVEL_PANIC) <= 0;
  *o = (unsigned long long)((NCLOGLEVEL_PANIC) | 0);  /* check that NCLOGLEVEL_PANIC is an integer */
  return n;
}

static int _cffi_const_NCLOGLEVEL_FATAL(unsigned long long *o)
{
  int n = (NCLOGLEVEL_FATAL) <= 0;
  *o = (unsigned long long)((NCLOGLEVEL_FATAL) | 0);  /* check that NCLOGLEVEL_FATAL is an integer */
  return n;
}

static int _cffi_const_NCLOGLEVEL_ERROR(unsigned long long *o)
{
  int n = (NCLOGLEVEL_ERROR) <= 0;
  *o = (unsigned long long)((NCLOGLEVEL_ERROR) | 0);  /* check that NCLOGLEVEL_ERROR is an integer */
  return n;
}

static int _cffi_const_NCLOGLEVEL_WARNING(unsigned long long *o)
{
  int n = (NCLOGLEVEL_WARNING) <= 0;
  *o = (unsigned long long)((NCLOGLEVEL_WARNING) | 0);  /* check that NCLOGLEVEL_WARNING is an integer */
  return n;
}

static int _cffi_const_NCLOGLEVEL_INFO(unsigned long long *o)
{
  int n = (NCLOGLEVEL_INFO) <= 0;
  *o = (unsigned long long)((NCLOGLEVEL_INFO) | 0);  /* check that NCLOGLEVEL_INFO is an integer */
  return n;
}

static int _cffi_const_NCLOGLEVEL_VERBOSE(unsigned long long *o)
{
  int n = (NCLOGLEVEL_VERBOSE) <= 0;
  *o = (unsigned long long)((NCLOGLEVEL_VERBOSE) | 0);  /* check that NCLOGLEVEL_VERBOSE is an integer */
  return n;
}

static int _cffi_const_NCLOGLEVEL_DEBUG(unsigned long long *o)
{
  int n = (NCLOGLEVEL_DEBUG) <= 0;
  *o = (unsigned long long)((NCLOGLEVEL_DEBUG) | 0);  /* check that NCLOGLEVEL_DEBUG is an integer */
  return n;
}

static int _cffi_const_NCLOGLEVEL_TRACE(unsigned long long *o)
{
  int n = (NCLOGLEVEL_TRACE) <= 0;
  *o = (unsigned long long)((NCLOGLEVEL_TRACE) | 0);  /* check that NCLOGLEVEL_TRACE is an integer */
  return n;
}

static int _cffi_const_NCSCALE_NONE(unsigned long long *o)
{
  int n = (NCSCALE_NONE) <= 0;
  *o = (unsigned long long)((NCSCALE_NONE) | 0);  /* check that NCSCALE_NONE is an integer */
  return n;
}

static int _cffi_const_NCSCALE_SCALE(unsigned long long *o)
{
  int n = (NCSCALE_SCALE) <= 0;
  *o = (unsigned long long)((NCSCALE_SCALE) | 0);  /* check that NCSCALE_SCALE is an integer */
  return n;
}

static int _cffi_const_NCSCALE_STRETCH(unsigned long long *o)
{
  int n = (NCSCALE_STRETCH) <= 0;
  *o = (unsigned long long)((NCSCALE_STRETCH) | 0);  /* check that NCSCALE_STRETCH is an integer */
  return n;
}

static int _cffi_d_cell_duplicate(struct ncplane * x0, cell * x1, cell const * x2)
{
  return cell_duplicate(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_cell_duplicate(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  cell * x1;
  cell const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "cell_duplicate", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(212), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (cell *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(212), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = cell_duplicate(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_cell_duplicate _cffi_d_cell_duplicate
#endif

static char const * _cffi_d_cell_extended_gcluster(struct ncplane const * x0, cell const * x1)
{
  return cell_extended_gcluster(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_cell_extended_gcluster(PyObject *self, PyObject *args)
{
  struct ncplane const * x0;
  cell const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "cell_extended_gcluster", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = cell_extended_gcluster(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(92));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_cell_extended_gcluster _cffi_d_cell_extended_gcluster
#endif

static int _cffi_d_cell_load(struct ncplane * x0, cell * x1, char const * x2)
{
  return cell_load(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_cell_load(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  cell * x1;
  char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "cell_load", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(212), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (cell *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(212), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = cell_load(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_cell_load _cffi_d_cell_load
#endif

static void _cffi_d_cell_release(struct ncplane * x0, cell * x1)
{
  cell_release(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_cell_release(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  cell * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "cell_release", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(212), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (cell *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(212), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { cell_release(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_cell_release _cffi_d_cell_release
#endif

static int _cffi_d_ncblit_bgrx(void const * x0, int x1, struct ncvisual_options const * x2)
{
  return ncblit_bgrx(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncblit_bgrx(PyObject *self, PyObject *args)
{
  void const * x0;
  int x1;
  struct ncvisual_options const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncblit_bgrx", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(191), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(191), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(525), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (struct ncvisual_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(525), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncblit_bgrx(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncblit_bgrx _cffi_d_ncblit_bgrx
#endif

static int _cffi_d_ncblit_rgba(void const * x0, int x1, struct ncvisual_options const * x2)
{
  return ncblit_rgba(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncblit_rgba(PyObject *self, PyObject *args)
{
  void const * x0;
  int x1;
  struct ncvisual_options const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncblit_rgba", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(191), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(191), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(525), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (struct ncvisual_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(525), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncblit_rgba(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncblit_rgba _cffi_d_ncblit_rgba
#endif

static int _cffi_d_ncdirect_bg_rgb(struct ncdirect * x0, unsigned int x1)
{
  return ncdirect_bg_rgb(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_bg_rgb(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncdirect_bg_rgb", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_bg_rgb(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_bg_rgb _cffi_d_ncdirect_bg_rgb
#endif

static int _cffi_d_ncdirect_bg_rgb8(struct ncdirect * x0, unsigned int x1, unsigned int x2, unsigned int x3)
{
  return ncdirect_bg_rgb8(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_bg_rgb8(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  unsigned int x1;
  unsigned int x2;
  unsigned int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncdirect_bg_rgb8", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, unsigned int);
  if (x2 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, unsigned int);
  if (x3 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_bg_rgb8(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_bg_rgb8 _cffi_d_ncdirect_bg_rgb8
#endif

static int _cffi_d_ncdirect_box(struct ncdirect * x0, uint64_t x1, uint64_t x2, uint64_t x3, uint64_t x4, wchar_t const * x5, int x6, int x7, unsigned int x8)
{
  return ncdirect_box(x0, x1, x2, x3, x4, x5, x6, x7, x8);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_box(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  uint64_t x1;
  uint64_t x2;
  uint64_t x3;
  uint64_t x4;
  wchar_t const * x5;
  int x6;
  int x7;
  unsigned int x8;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;
  PyObject *arg8;

  if (!PyArg_UnpackTuple(args, "ncdirect_box", 9, 9, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint64_t);
  if (x1 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, uint64_t);
  if (x2 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint64_t);
  if (x3 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, uint64_t);
  if (x4 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(153), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (wchar_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(153), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x6 = _cffi_to_c_int(arg6, int);
  if (x6 == (int)-1 && PyErr_Occurred())
    return NULL;

  x7 = _cffi_to_c_int(arg7, int);
  if (x7 == (int)-1 && PyErr_Occurred())
    return NULL;

  x8 = _cffi_to_c_int(arg8, unsigned int);
  if (x8 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_box(x0, x1, x2, x3, x4, x5, x6, x7, x8); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_box _cffi_d_ncdirect_box
#endif

static _Bool _cffi_d_ncdirect_canopen_images(struct ncdirect const * x0)
{
  return ncdirect_canopen_images(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_canopen_images(PyObject *self, PyObject *arg0)
{
  struct ncdirect const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_canopen_images(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_canopen_images _cffi_d_ncdirect_canopen_images
#endif

static _Bool _cffi_d_ncdirect_canutf8(struct ncdirect const * x0)
{
  return ncdirect_canutf8(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_canutf8(PyObject *self, PyObject *arg0)
{
  struct ncdirect const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_canutf8(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_canutf8 _cffi_d_ncdirect_canutf8
#endif

static int _cffi_d_ncdirect_clear(struct ncdirect * x0)
{
  return ncdirect_clear(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_clear(PyObject *self, PyObject *arg0)
{
  struct ncdirect * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_clear(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_clear _cffi_d_ncdirect_clear
#endif

static int _cffi_d_ncdirect_cursor_disable(struct ncdirect * x0)
{
  return ncdirect_cursor_disable(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_cursor_disable(PyObject *self, PyObject *arg0)
{
  struct ncdirect * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_cursor_disable(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_cursor_disable _cffi_d_ncdirect_cursor_disable
#endif

static int _cffi_d_ncdirect_cursor_down(struct ncdirect * x0, int x1)
{
  return ncdirect_cursor_down(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_cursor_down(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncdirect_cursor_down", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_cursor_down(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_cursor_down _cffi_d_ncdirect_cursor_down
#endif

static int _cffi_d_ncdirect_cursor_enable(struct ncdirect * x0)
{
  return ncdirect_cursor_enable(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_cursor_enable(PyObject *self, PyObject *arg0)
{
  struct ncdirect * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_cursor_enable(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_cursor_enable _cffi_d_ncdirect_cursor_enable
#endif

static int _cffi_d_ncdirect_cursor_left(struct ncdirect * x0, int x1)
{
  return ncdirect_cursor_left(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_cursor_left(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncdirect_cursor_left", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_cursor_left(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_cursor_left _cffi_d_ncdirect_cursor_left
#endif

static int _cffi_d_ncdirect_cursor_move_yx(struct ncdirect * x0, int x1, int x2)
{
  return ncdirect_cursor_move_yx(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_cursor_move_yx(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncdirect_cursor_move_yx", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_cursor_move_yx(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_cursor_move_yx _cffi_d_ncdirect_cursor_move_yx
#endif

static int _cffi_d_ncdirect_cursor_right(struct ncdirect * x0, int x1)
{
  return ncdirect_cursor_right(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_cursor_right(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncdirect_cursor_right", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_cursor_right(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_cursor_right _cffi_d_ncdirect_cursor_right
#endif

static int _cffi_d_ncdirect_cursor_up(struct ncdirect * x0, int x1)
{
  return ncdirect_cursor_up(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_cursor_up(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncdirect_cursor_up", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_cursor_up(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_cursor_up _cffi_d_ncdirect_cursor_up
#endif

static int _cffi_d_ncdirect_dim_x(struct ncdirect const * x0)
{
  return ncdirect_dim_x(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_dim_x(PyObject *self, PyObject *arg0)
{
  struct ncdirect const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_dim_x(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_dim_x _cffi_d_ncdirect_dim_x
#endif

static int _cffi_d_ncdirect_dim_y(struct ncdirect const * x0)
{
  return ncdirect_dim_y(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_dim_y(PyObject *self, PyObject *arg0)
{
  struct ncdirect const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_dim_y(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_dim_y _cffi_d_ncdirect_dim_y
#endif

static int _cffi_d_ncdirect_double_box(struct ncdirect * x0, uint64_t x1, uint64_t x2, uint64_t x3, uint64_t x4, int x5, int x6, unsigned int x7)
{
  return ncdirect_double_box(x0, x1, x2, x3, x4, x5, x6, x7);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_double_box(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  uint64_t x1;
  uint64_t x2;
  uint64_t x3;
  uint64_t x4;
  int x5;
  int x6;
  unsigned int x7;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;

  if (!PyArg_UnpackTuple(args, "ncdirect_double_box", 8, 8, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint64_t);
  if (x1 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, uint64_t);
  if (x2 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint64_t);
  if (x3 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, uint64_t);
  if (x4 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  x6 = _cffi_to_c_int(arg6, int);
  if (x6 == (int)-1 && PyErr_Occurred())
    return NULL;

  x7 = _cffi_to_c_int(arg7, unsigned int);
  if (x7 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_double_box(x0, x1, x2, x3, x4, x5, x6, x7); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_double_box _cffi_d_ncdirect_double_box
#endif

static int _cffi_d_ncdirect_fg_rgb(struct ncdirect * x0, unsigned int x1)
{
  return ncdirect_fg_rgb(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_fg_rgb(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncdirect_fg_rgb", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_fg_rgb(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_fg_rgb _cffi_d_ncdirect_fg_rgb
#endif

static int _cffi_d_ncdirect_fg_rgb8(struct ncdirect * x0, unsigned int x1, unsigned int x2, unsigned int x3)
{
  return ncdirect_fg_rgb8(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_fg_rgb8(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  unsigned int x1;
  unsigned int x2;
  unsigned int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncdirect_fg_rgb8", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, unsigned int);
  if (x2 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, unsigned int);
  if (x3 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_fg_rgb8(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_fg_rgb8 _cffi_d_ncdirect_fg_rgb8
#endif

static int _cffi_d_ncdirect_flush(struct ncdirect const * x0)
{
  return ncdirect_flush(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_flush(PyObject *self, PyObject *arg0)
{
  struct ncdirect const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_flush(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_flush _cffi_d_ncdirect_flush
#endif

static int _cffi_d_ncdirect_hline_interp(struct ncdirect * x0, char const * x1, int x2, uint64_t x3, uint64_t x4)
{
  return ncdirect_hline_interp(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_hline_interp(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  char const * x1;
  int x2;
  uint64_t x3;
  uint64_t x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncdirect_hline_interp", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint64_t);
  if (x3 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, uint64_t);
  if (x4 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_hline_interp(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_hline_interp _cffi_d_ncdirect_hline_interp
#endif

static struct ncdirect * _cffi_d_ncdirect_init(char const * x0, FILE * x1, uint64_t x2)
{
  return ncdirect_init(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_init(PyObject *self, PyObject *args)
{
  char const * x0;
  FILE * x1;
  uint64_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncdirect * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncdirect_init", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(498), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (FILE *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(498), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, uint64_t);
  if (x2 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_init(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(107));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_init _cffi_d_ncdirect_init
#endif

static unsigned int _cffi_d_ncdirect_palette_size(struct ncdirect const * x0)
{
  return ncdirect_palette_size(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_palette_size(PyObject *self, PyObject *arg0)
{
  struct ncdirect const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_palette_size(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, unsigned int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_palette_size _cffi_d_ncdirect_palette_size
#endif

static int _cffi_d_ncdirect_putstr(struct ncdirect * x0, uint64_t x1, char const * x2)
{
  return ncdirect_putstr(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_putstr(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  uint64_t x1;
  char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncdirect_putstr", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint64_t);
  if (x1 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_putstr(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_putstr _cffi_d_ncdirect_putstr
#endif

static int _cffi_d_ncdirect_render_image(struct ncdirect * x0, char const * x1, ncalign_e x2, ncblitter_e x3, ncscale_e x4)
{
  return ncdirect_render_image(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_render_image(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  char const * x1;
  ncalign_e x2;
  ncblitter_e x3;
  ncscale_e x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncdirect_render_image", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x2, _cffi_type(119), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(64), arg3) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(67), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_render_image(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_render_image _cffi_d_ncdirect_render_image
#endif

static int _cffi_d_ncdirect_rounded_box(struct ncdirect * x0, uint64_t x1, uint64_t x2, uint64_t x3, uint64_t x4, int x5, int x6, unsigned int x7)
{
  return ncdirect_rounded_box(x0, x1, x2, x3, x4, x5, x6, x7);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_rounded_box(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  uint64_t x1;
  uint64_t x2;
  uint64_t x3;
  uint64_t x4;
  int x5;
  int x6;
  unsigned int x7;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;

  if (!PyArg_UnpackTuple(args, "ncdirect_rounded_box", 8, 8, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint64_t);
  if (x1 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, uint64_t);
  if (x2 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint64_t);
  if (x3 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, uint64_t);
  if (x4 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  x6 = _cffi_to_c_int(arg6, int);
  if (x6 == (int)-1 && PyErr_Occurred())
    return NULL;

  x7 = _cffi_to_c_int(arg7, unsigned int);
  if (x7 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_rounded_box(x0, x1, x2, x3, x4, x5, x6, x7); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_rounded_box _cffi_d_ncdirect_rounded_box
#endif

static int _cffi_d_ncdirect_stop(struct ncdirect * x0)
{
  return ncdirect_stop(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_stop(PyObject *self, PyObject *arg0)
{
  struct ncdirect * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_stop(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_stop _cffi_d_ncdirect_stop
#endif

static int _cffi_d_ncdirect_styles_off(struct ncdirect * x0, unsigned int x1)
{
  return ncdirect_styles_off(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_styles_off(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncdirect_styles_off", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_styles_off(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_styles_off _cffi_d_ncdirect_styles_off
#endif

static int _cffi_d_ncdirect_styles_on(struct ncdirect * x0, unsigned int x1)
{
  return ncdirect_styles_on(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_styles_on(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncdirect_styles_on", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_styles_on(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_styles_on _cffi_d_ncdirect_styles_on
#endif

static int _cffi_d_ncdirect_styles_set(struct ncdirect * x0, unsigned int x1)
{
  return ncdirect_styles_set(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_styles_set(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncdirect_styles_set", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_styles_set(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_styles_set _cffi_d_ncdirect_styles_set
#endif

static int _cffi_d_ncdirect_vline_interp(struct ncdirect * x0, char const * x1, int x2, uint64_t x3, uint64_t x4)
{
  return ncdirect_vline_interp(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdirect_vline_interp(PyObject *self, PyObject *args)
{
  struct ncdirect * x0;
  char const * x1;
  int x2;
  uint64_t x3;
  uint64_t x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncdirect_vline_interp", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(107), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdirect *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(107), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint64_t);
  if (x3 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, uint64_t);
  if (x4 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdirect_vline_interp(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdirect_vline_interp _cffi_d_ncdirect_vline_interp
#endif

static int _cffi_d_ncdplot_add_sample(struct ncdplot * x0, uint64_t x1, double x2)
{
  return ncdplot_add_sample(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdplot_add_sample(PyObject *self, PyObject *args)
{
  struct ncdplot * x0;
  uint64_t x1;
  double x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncdplot_add_sample", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(172), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdplot *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(172), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint64_t);
  if (x1 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = (double)_cffi_to_c_double(arg2);
  if (x2 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdplot_add_sample(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdplot_add_sample _cffi_d_ncdplot_add_sample
#endif

static struct ncdplot * _cffi_d_ncdplot_create(struct ncplane * x0, ncplot_options const * x1, double x2, double x3)
{
  return ncdplot_create(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdplot_create(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncplot_options const * x1;
  double x2;
  double x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncdplot * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncdplot_create", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(552), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncplot_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(552), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = (double)_cffi_to_c_double(arg2);
  if (x2 == (double)-1 && PyErr_Occurred())
    return NULL;

  x3 = (double)_cffi_to_c_double(arg3);
  if (x3 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdplot_create(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(172));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdplot_create _cffi_d_ncdplot_create
#endif

static void _cffi_d_ncdplot_destroy(struct ncdplot * x0)
{
  ncdplot_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdplot_destroy(PyObject *self, PyObject *arg0)
{
  struct ncdplot * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(172), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdplot *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(172), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncdplot_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncdplot_destroy _cffi_d_ncdplot_destroy
#endif

static struct ncplane * _cffi_d_ncdplot_plane(struct ncdplot * x0)
{
  return ncdplot_plane(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdplot_plane(PyObject *self, PyObject *arg0)
{
  struct ncdplot * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(172), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdplot *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(172), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdplot_plane(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdplot_plane _cffi_d_ncdplot_plane
#endif

static int _cffi_d_ncdplot_sample(struct ncdplot const * x0, uint64_t x1, double * x2)
{
  return ncdplot_sample(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdplot_sample(PyObject *self, PyObject *args)
{
  struct ncdplot const * x0;
  uint64_t x1;
  double * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncdplot_sample", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(177), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdplot const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(177), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint64_t);
  if (x1 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(179), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(179), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdplot_sample(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdplot_sample _cffi_d_ncdplot_sample
#endif

static int _cffi_d_ncdplot_set_sample(struct ncdplot * x0, uint64_t x1, double x2)
{
  return ncdplot_set_sample(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncdplot_set_sample(PyObject *self, PyObject *args)
{
  struct ncdplot * x0;
  uint64_t x1;
  double x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncdplot_set_sample", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(172), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncdplot *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(172), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint64_t);
  if (x1 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = (double)_cffi_to_c_double(arg2);
  if (x2 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncdplot_set_sample(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncdplot_set_sample _cffi_d_ncdplot_set_sample
#endif

static struct ncfdplane * _cffi_d_ncfdplane_create(struct ncplane * x0, ncfdplane_options const * x1, int x2, int(* x3)(struct ncfdplane *, void const *, size_t, void *), int(* x4)(struct ncfdplane *, int, void *))
{
  return ncfdplane_create(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncfdplane_create(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncfdplane_options const * x1;
  int x2;
  int(* x3)(struct ncfdplane *, void const *, size_t, void *);
  int(* x4)(struct ncfdplane *, int, void *);
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncfdplane * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncfdplane_create", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(558), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncfdplane_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(558), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = (int(*)(struct ncfdplane *, void const *, size_t, void *))_cffi_to_c_pointer(arg3, _cffi_type(560));
  if (x3 == (int(*)(struct ncfdplane *, void const *, size_t, void *))NULL && PyErr_Occurred())
    return NULL;

  x4 = (int(*)(struct ncfdplane *, int, void *))_cffi_to_c_pointer(arg4, _cffi_type(561));
  if (x4 == (int(*)(struct ncfdplane *, int, void *))NULL && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncfdplane_create(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(182));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncfdplane_create _cffi_d_ncfdplane_create
#endif

static int _cffi_d_ncfdplane_destroy(struct ncfdplane * x0)
{
  return ncfdplane_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncfdplane_destroy(PyObject *self, PyObject *arg0)
{
  struct ncfdplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(182), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncfdplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(182), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncfdplane_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncfdplane_destroy _cffi_d_ncfdplane_destroy
#endif

static struct ncplane * _cffi_d_ncfdplane_plane(struct ncfdplane * x0)
{
  return ncfdplane_plane(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncfdplane_plane(PyObject *self, PyObject *arg0)
{
  struct ncfdplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(182), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncfdplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(182), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncfdplane_plane(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncfdplane_plane _cffi_d_ncfdplane_plane
#endif

static struct ncmenu * _cffi_d_ncmenu_create(struct ncplane * x0, ncmenu_options const * x1)
{
  return ncmenu_create(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmenu_create(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncmenu_options const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncmenu * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncmenu_create", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(565), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncmenu_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(565), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmenu_create(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(4));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmenu_create _cffi_d_ncmenu_create
#endif

static int _cffi_d_ncmenu_destroy(struct ncmenu * x0)
{
  return ncmenu_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmenu_destroy(PyObject *self, PyObject *arg0)
{
  struct ncmenu * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmenu *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmenu_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmenu_destroy _cffi_d_ncmenu_destroy
#endif

static int _cffi_d_ncmenu_nextitem(struct ncmenu * x0)
{
  return ncmenu_nextitem(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmenu_nextitem(PyObject *self, PyObject *arg0)
{
  struct ncmenu * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmenu *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmenu_nextitem(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmenu_nextitem _cffi_d_ncmenu_nextitem
#endif

static int _cffi_d_ncmenu_nextsection(struct ncmenu * x0)
{
  return ncmenu_nextsection(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmenu_nextsection(PyObject *self, PyObject *arg0)
{
  struct ncmenu * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmenu *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmenu_nextsection(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmenu_nextsection _cffi_d_ncmenu_nextsection
#endif

static _Bool _cffi_d_ncmenu_offer_input(struct ncmenu * x0, ncinput const * x1)
{
  return ncmenu_offer_input(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmenu_offer_input(PyObject *self, PyObject *args)
{
  struct ncmenu * x0;
  ncinput const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncmenu_offer_input", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmenu *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncinput const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmenu_offer_input(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmenu_offer_input _cffi_d_ncmenu_offer_input
#endif

static int _cffi_d_ncmenu_previtem(struct ncmenu * x0)
{
  return ncmenu_previtem(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmenu_previtem(PyObject *self, PyObject *arg0)
{
  struct ncmenu * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmenu *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmenu_previtem(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmenu_previtem _cffi_d_ncmenu_previtem
#endif

static int _cffi_d_ncmenu_prevsection(struct ncmenu * x0)
{
  return ncmenu_prevsection(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmenu_prevsection(PyObject *self, PyObject *arg0)
{
  struct ncmenu * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmenu *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmenu_prevsection(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmenu_prevsection _cffi_d_ncmenu_prevsection
#endif

static int _cffi_d_ncmenu_rollup(struct ncmenu * x0)
{
  return ncmenu_rollup(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmenu_rollup(PyObject *self, PyObject *arg0)
{
  struct ncmenu * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmenu *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmenu_rollup(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmenu_rollup _cffi_d_ncmenu_rollup
#endif

static char const * _cffi_d_ncmenu_selected(struct ncmenu const * x0, ncinput * x1)
{
  return ncmenu_selected(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmenu_selected(PyObject *self, PyObject *args)
{
  struct ncmenu const * x0;
  ncinput * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncmenu_selected", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(70), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmenu const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(70), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(71), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncinput *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(71), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmenu_selected(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(92));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmenu_selected _cffi_d_ncmenu_selected
#endif

static int _cffi_d_ncmenu_unroll(struct ncmenu * x0, int x1)
{
  return ncmenu_unroll(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmenu_unroll(PyObject *self, PyObject *args)
{
  struct ncmenu * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncmenu_unroll", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmenu *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmenu_unroll(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmenu_unroll _cffi_d_ncmenu_unroll
#endif

static char const * _cffi_d_ncmetric(uintmax_t x0, unsigned int x1, char * x2, int x3, unsigned int x4, int x5)
{
  return ncmetric(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmetric(PyObject *self, PyObject *args)
{
  uintmax_t x0;
  unsigned int x1;
  char * x2;
  int x3;
  unsigned int x4;
  int x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "ncmetric", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  x0 = _cffi_to_c_int(arg0, uintmax_t);
  if (x0 == (uintmax_t)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(86), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(86), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, unsigned int);
  if (x4 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmetric(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(92));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmetric _cffi_d_ncmetric
#endif

static struct ncmultiselector * _cffi_d_ncmultiselector_create(struct ncplane * x0, ncmultiselector_options const * x1)
{
  return ncmultiselector_create(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmultiselector_create(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncmultiselector_options const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncmultiselector * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncmultiselector_create", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(569), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncmultiselector_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(569), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmultiselector_create(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(8));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmultiselector_create _cffi_d_ncmultiselector_create
#endif

static void _cffi_d_ncmultiselector_destroy(struct ncmultiselector * x0)
{
  ncmultiselector_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmultiselector_destroy(PyObject *self, PyObject *arg0)
{
  struct ncmultiselector * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmultiselector *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncmultiselector_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncmultiselector_destroy _cffi_d_ncmultiselector_destroy
#endif

static _Bool _cffi_d_ncmultiselector_offer_input(struct ncmultiselector * x0, ncinput const * x1)
{
  return ncmultiselector_offer_input(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmultiselector_offer_input(PyObject *self, PyObject *args)
{
  struct ncmultiselector * x0;
  ncinput const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncmultiselector_offer_input", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmultiselector *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncinput const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmultiselector_offer_input(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmultiselector_offer_input _cffi_d_ncmultiselector_offer_input
#endif

static struct ncplane * _cffi_d_ncmultiselector_plane(struct ncmultiselector * x0)
{
  return ncmultiselector_plane(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmultiselector_plane(PyObject *self, PyObject *arg0)
{
  struct ncmultiselector * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmultiselector *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmultiselector_plane(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmultiselector_plane _cffi_d_ncmultiselector_plane
#endif

static int _cffi_d_ncmultiselector_selected(struct ncmultiselector * x0, _Bool * x1, unsigned int x2)
{
  return ncmultiselector_selected(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncmultiselector_selected(PyObject *self, PyObject *args)
{
  struct ncmultiselector * x0;
  _Bool * x1;
  unsigned int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncmultiselector_selected", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncmultiselector *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(204), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (_Bool *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(204), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, unsigned int);
  if (x2 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncmultiselector_selected(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncmultiselector_selected _cffi_d_ncmultiselector_selected
#endif

static struct ncplane * _cffi_d_ncplane_above(struct ncplane * x0)
{
  return ncplane_above(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_above(PyObject *self, PyObject *arg0)
{
  struct ncplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_above(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_above _cffi_d_ncplane_above
#endif

static char * _cffi_d_ncplane_at_cursor(struct ncplane * x0, uint16_t * x1, uint64_t * x2)
{
  return ncplane_at_cursor(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_at_cursor(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  uint16_t * x1;
  uint64_t * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncplane_at_cursor", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(33), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (uint16_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(33), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(34), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (uint64_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(34), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_at_cursor(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(86));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_at_cursor _cffi_d_ncplane_at_cursor
#endif

static char * _cffi_d_ncplane_at_yx(struct ncplane const * x0, int x1, int x2, uint16_t * x3, uint64_t * x4)
{
  return ncplane_at_yx(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_at_yx(PyObject *self, PyObject *args)
{
  struct ncplane const * x0;
  int x1;
  int x2;
  uint16_t * x3;
  uint64_t * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncplane_at_yx", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(33), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (uint16_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(33), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(34), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (uint64_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(34), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_at_yx(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(86));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_at_yx _cffi_d_ncplane_at_yx
#endif

static uint16_t _cffi_d_ncplane_attr(struct ncplane const * x0)
{
  return ncplane_attr(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_attr(PyObject *self, PyObject *arg0)
{
  struct ncplane const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint16_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_attr(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, uint16_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_attr _cffi_d_ncplane_attr
#endif

static int _cffi_d_ncplane_base(struct ncplane * x0, cell * x1)
{
  return ncplane_base(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_base(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  cell * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_base", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(212), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (cell *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(212), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_base(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_base _cffi_d_ncplane_base
#endif

static struct ncplane * _cffi_d_ncplane_below(struct ncplane * x0)
{
  return ncplane_below(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_below(PyObject *self, PyObject *arg0)
{
  struct ncplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_below(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_below _cffi_d_ncplane_below
#endif

static int _cffi_d_ncplane_box(struct ncplane * x0, cell const * x1, cell const * x2, cell const * x3, cell const * x4, cell const * x5, cell const * x6, int x7, int x8, unsigned int x9)
{
  return ncplane_box(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_box(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  cell const * x1;
  cell const * x2;
  cell const * x3;
  cell const * x4;
  cell const * x5;
  cell const * x6;
  int x7;
  int x8;
  unsigned int x9;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;
  PyObject *arg8;
  PyObject *arg9;

  if (!PyArg_UnpackTuple(args, "ncplane_box", 10, 10, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8, &arg9))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg6, (char **)&x6);
  if (datasize != 0) {
    x6 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg6, (char **)&x6,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x7 = _cffi_to_c_int(arg7, int);
  if (x7 == (int)-1 && PyErr_Occurred())
    return NULL;

  x8 = _cffi_to_c_int(arg8, int);
  if (x8 == (int)-1 && PyErr_Occurred())
    return NULL;

  x9 = _cffi_to_c_int(arg9, unsigned int);
  if (x9 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_box(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_box _cffi_d_ncplane_box
#endif

static uint64_t _cffi_d_ncplane_channels(struct ncplane const * x0)
{
  return ncplane_channels(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_channels(PyObject *self, PyObject *arg0)
{
  struct ncplane const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint64_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_channels(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, uint64_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_channels _cffi_d_ncplane_channels
#endif

static char * _cffi_d_ncplane_contents(struct ncplane const * x0, int x1, int x2, int x3, int x4)
{
  return ncplane_contents(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_contents(PyObject *self, PyObject *args)
{
  struct ncplane const * x0;
  int x1;
  int x2;
  int x3;
  int x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncplane_contents", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_contents(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(86));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_contents _cffi_d_ncplane_contents
#endif

static struct ncplane * _cffi_d_ncplane_create(struct ncplane * x0, ncplane_options const * x1)
{
  return ncplane_create(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_create(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncplane_options const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_create", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(585), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncplane_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(585), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_create(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_create _cffi_d_ncplane_create
#endif

static int _cffi_d_ncplane_cursor_move_yx(struct ncplane * x0, int x1, int x2)
{
  return ncplane_cursor_move_yx(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_cursor_move_yx(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncplane_cursor_move_yx", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_cursor_move_yx(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_cursor_move_yx _cffi_d_ncplane_cursor_move_yx
#endif

static void _cffi_d_ncplane_cursor_yx(struct ncplane * x0, int * x1, int * x2)
{
  ncplane_cursor_yx(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_cursor_yx(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncplane_cursor_yx", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_cursor_yx(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_cursor_yx _cffi_d_ncplane_cursor_yx
#endif

static int _cffi_d_ncplane_destroy(struct ncplane * x0)
{
  return ncplane_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_destroy(PyObject *self, PyObject *arg0)
{
  struct ncplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_destroy _cffi_d_ncplane_destroy
#endif

static void _cffi_d_ncplane_dim_yx(struct ncplane const * x0, int * x1, int * x2)
{
  ncplane_dim_yx(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_dim_yx(PyObject *self, PyObject *args)
{
  struct ncplane const * x0;
  int * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncplane_dim_yx", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_dim_yx(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_dim_yx _cffi_d_ncplane_dim_yx
#endif

static struct ncplane * _cffi_d_ncplane_dup(struct ncplane const * x0, void * x1)
{
  return ncplane_dup(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_dup(PyObject *self, PyObject *args)
{
  struct ncplane const * x0;
  void * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_dup", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(187), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(187), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_dup(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_dup _cffi_d_ncplane_dup
#endif

static void _cffi_d_ncplane_erase(struct ncplane * x0)
{
  ncplane_erase(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_erase(PyObject *self, PyObject *arg0)
{
  struct ncplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_erase(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_erase _cffi_d_ncplane_erase
#endif

static int _cffi_d_ncplane_fadein(struct ncplane * x0, struct timespec const * x1, int(* x2)(struct notcurses *, struct ncplane *, struct timespec const *, void *), void * x3)
{
  return ncplane_fadein(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_fadein(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  struct timespec const * x1;
  int(* x2)(struct notcurses *, struct ncplane *, struct timespec const *, void *);
  void * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_fadein", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(372), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct timespec const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(372), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = (int(*)(struct notcurses *, struct ncplane *, struct timespec const *, void *))_cffi_to_c_pointer(arg2, _cffi_type(373));
  if (x2 == (int(*)(struct notcurses *, struct ncplane *, struct timespec const *, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(187), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(187), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_fadein(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_fadein _cffi_d_ncplane_fadein
#endif

static int _cffi_d_ncplane_fadeout(struct ncplane * x0, struct timespec const * x1, int(* x2)(struct notcurses *, struct ncplane *, struct timespec const *, void *), void * x3)
{
  return ncplane_fadeout(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_fadeout(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  struct timespec const * x1;
  int(* x2)(struct notcurses *, struct ncplane *, struct timespec const *, void *);
  void * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_fadeout", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(372), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct timespec const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(372), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = (int(*)(struct notcurses *, struct ncplane *, struct timespec const *, void *))_cffi_to_c_pointer(arg2, _cffi_type(373));
  if (x2 == (int(*)(struct notcurses *, struct ncplane *, struct timespec const *, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(187), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(187), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_fadeout(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_fadeout _cffi_d_ncplane_fadeout
#endif

static int _cffi_d_ncplane_format(struct ncplane * x0, int x1, int x2, uint32_t x3)
{
  return ncplane_format(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_format(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  uint32_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_format", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint32_t);
  if (x3 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_format(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_format _cffi_d_ncplane_format
#endif

static int _cffi_d_ncplane_gradient(struct ncplane * x0, char const * x1, uint32_t x2, uint64_t x3, uint64_t x4, uint64_t x5, uint64_t x6, int x7, int x8)
{
  return ncplane_gradient(x0, x1, x2, x3, x4, x5, x6, x7, x8);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_gradient(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  char const * x1;
  uint32_t x2;
  uint64_t x3;
  uint64_t x4;
  uint64_t x5;
  uint64_t x6;
  int x7;
  int x8;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;
  PyObject *arg8;

  if (!PyArg_UnpackTuple(args, "ncplane_gradient", 9, 9, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, uint32_t);
  if (x2 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint64_t);
  if (x3 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, uint64_t);
  if (x4 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, uint64_t);
  if (x5 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x6 = _cffi_to_c_int(arg6, uint64_t);
  if (x6 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x7 = _cffi_to_c_int(arg7, int);
  if (x7 == (int)-1 && PyErr_Occurred())
    return NULL;

  x8 = _cffi_to_c_int(arg8, int);
  if (x8 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_gradient(x0, x1, x2, x3, x4, x5, x6, x7, x8); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_gradient _cffi_d_ncplane_gradient
#endif

static int _cffi_d_ncplane_highgradient(struct ncplane * x0, uint32_t x1, uint32_t x2, uint32_t x3, uint32_t x4, int x5, int x6)
{
  return ncplane_highgradient(x0, x1, x2, x3, x4, x5, x6);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_highgradient(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  uint32_t x1;
  uint32_t x2;
  uint32_t x3;
  uint32_t x4;
  int x5;
  int x6;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;

  if (!PyArg_UnpackTuple(args, "ncplane_highgradient", 7, 7, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint32_t);
  if (x1 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, uint32_t);
  if (x2 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint32_t);
  if (x3 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, uint32_t);
  if (x4 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  x6 = _cffi_to_c_int(arg6, int);
  if (x6 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_highgradient(x0, x1, x2, x3, x4, x5, x6); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_highgradient _cffi_d_ncplane_highgradient
#endif

static int _cffi_d_ncplane_highgradient_sized(struct ncplane * x0, uint32_t x1, uint32_t x2, uint32_t x3, uint32_t x4, int x5, int x6)
{
  return ncplane_highgradient_sized(x0, x1, x2, x3, x4, x5, x6);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_highgradient_sized(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  uint32_t x1;
  uint32_t x2;
  uint32_t x3;
  uint32_t x4;
  int x5;
  int x6;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;

  if (!PyArg_UnpackTuple(args, "ncplane_highgradient_sized", 7, 7, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint32_t);
  if (x1 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, uint32_t);
  if (x2 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint32_t);
  if (x3 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, uint32_t);
  if (x4 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  x6 = _cffi_to_c_int(arg6, int);
  if (x6 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_highgradient_sized(x0, x1, x2, x3, x4, x5, x6); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_highgradient_sized _cffi_d_ncplane_highgradient_sized
#endif

static int _cffi_d_ncplane_hline_interp(struct ncplane * x0, cell const * x1, int x2, uint64_t x3, uint64_t x4)
{
  return ncplane_hline_interp(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_hline_interp(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  cell const * x1;
  int x2;
  uint64_t x3;
  uint64_t x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncplane_hline_interp", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint64_t);
  if (x3 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, uint64_t);
  if (x4 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_hline_interp(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_hline_interp _cffi_d_ncplane_hline_interp
#endif

static int _cffi_d_ncplane_mergedown(struct ncplane const * x0, struct ncplane * x1, int x2, int x3, int x4, int x5, int x6, int x7)
{
  return ncplane_mergedown(x0, x1, x2, x3, x4, x5, x6, x7);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_mergedown(PyObject *self, PyObject *args)
{
  struct ncplane const * x0;
  struct ncplane * x1;
  int x2;
  int x3;
  int x4;
  int x5;
  int x6;
  int x7;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;

  if (!PyArg_UnpackTuple(args, "ncplane_mergedown", 8, 8, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  x6 = _cffi_to_c_int(arg6, int);
  if (x6 == (int)-1 && PyErr_Occurred())
    return NULL;

  x7 = _cffi_to_c_int(arg7, int);
  if (x7 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_mergedown(x0, x1, x2, x3, x4, x5, x6, x7); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_mergedown _cffi_d_ncplane_mergedown
#endif

static int _cffi_d_ncplane_mergedown_simple(struct ncplane const * x0, struct ncplane * x1)
{
  return ncplane_mergedown_simple(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_mergedown_simple(PyObject *self, PyObject *args)
{
  struct ncplane const * x0;
  struct ncplane * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_mergedown_simple", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_mergedown_simple(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_mergedown_simple _cffi_d_ncplane_mergedown_simple
#endif

static int _cffi_d_ncplane_move_above(struct ncplane * x0, struct ncplane * x1)
{
  return ncplane_move_above(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_move_above(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  struct ncplane * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_move_above", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_move_above(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_move_above _cffi_d_ncplane_move_above
#endif

static int _cffi_d_ncplane_move_below(struct ncplane * x0, struct ncplane * x1)
{
  return ncplane_move_below(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_move_below(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  struct ncplane * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_move_below", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_move_below(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_move_below _cffi_d_ncplane_move_below
#endif

static void _cffi_d_ncplane_move_bottom(struct ncplane * x0)
{
  ncplane_move_bottom(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_move_bottom(PyObject *self, PyObject *arg0)
{
  struct ncplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_move_bottom(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_move_bottom _cffi_d_ncplane_move_bottom
#endif

static void _cffi_d_ncplane_move_top(struct ncplane * x0)
{
  ncplane_move_top(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_move_top(PyObject *self, PyObject *arg0)
{
  struct ncplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_move_top(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_move_top _cffi_d_ncplane_move_top
#endif

static int _cffi_d_ncplane_move_yx(struct ncplane * x0, int x1, int x2)
{
  return ncplane_move_yx(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_move_yx(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncplane_move_yx", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_move_yx(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_move_yx _cffi_d_ncplane_move_yx
#endif

static struct ncplane * _cffi_d_ncplane_parent(struct ncplane * x0)
{
  return ncplane_parent(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_parent(PyObject *self, PyObject *arg0)
{
  struct ncplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_parent(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_parent _cffi_d_ncplane_parent
#endif

static struct ncplane const * _cffi_d_ncplane_parent_const(struct ncplane const * x0)
{
  return ncplane_parent_const(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_parent_const(PyObject *self, PyObject *arg0)
{
  struct ncplane const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_parent_const(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(16));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_parent_const _cffi_d_ncplane_parent_const
#endif

static int _cffi_d_ncplane_polyfill_yx(struct ncplane * x0, int x1, int x2, cell const * x3)
{
  return ncplane_polyfill_yx(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_polyfill_yx(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  cell const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_polyfill_yx", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_polyfill_yx(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_polyfill_yx _cffi_d_ncplane_polyfill_yx
#endif

static int _cffi_d_ncplane_pulse(struct ncplane * x0, struct timespec const * x1, int(* x2)(struct notcurses *, struct ncplane *, struct timespec const *, void *), void * x3)
{
  return ncplane_pulse(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_pulse(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  struct timespec const * x1;
  int(* x2)(struct notcurses *, struct ncplane *, struct timespec const *, void *);
  void * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_pulse", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(372), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct timespec const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(372), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = (int(*)(struct notcurses *, struct ncplane *, struct timespec const *, void *))_cffi_to_c_pointer(arg2, _cffi_type(373));
  if (x2 == (int(*)(struct notcurses *, struct ncplane *, struct timespec const *, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(187), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(187), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_pulse(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_pulse _cffi_d_ncplane_pulse
#endif

static int _cffi_d_ncplane_putc_yx(struct ncplane * x0, int x1, int x2, cell const * x3)
{
  return ncplane_putc_yx(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_putc_yx(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  cell const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_putc_yx", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_putc_yx(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_putc_yx _cffi_d_ncplane_putc_yx
#endif

static int _cffi_d_ncplane_putchar_stained(struct ncplane * x0, char x1)
{
  return ncplane_putchar_stained(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_putchar_stained(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  char x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_putchar_stained", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (char)_cffi_to_c_char(arg1);
  if (x1 == (char)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_putchar_stained(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_putchar_stained _cffi_d_ncplane_putchar_stained
#endif

static int _cffi_d_ncplane_putegc_stained(struct ncplane * x0, char const * x1, int * x2)
{
  return ncplane_putegc_stained(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_putegc_stained(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  char const * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncplane_putegc_stained", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_putegc_stained(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_putegc_stained _cffi_d_ncplane_putegc_stained
#endif

static int _cffi_d_ncplane_putegc_yx(struct ncplane * x0, int x1, int x2, char const * x3, int * x4)
{
  return ncplane_putegc_yx(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_putegc_yx(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  char const * x3;
  int * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncplane_putegc_yx", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_putegc_yx(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_putegc_yx _cffi_d_ncplane_putegc_yx
#endif

static int _cffi_d_ncplane_putnstr_aligned(struct ncplane * x0, int x1, ncalign_e x2, size_t x3, char const * x4)
{
  return ncplane_putnstr_aligned(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_putnstr_aligned(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  ncalign_e x2;
  size_t x3;
  char const * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncplane_putnstr_aligned", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(119), arg2) < 0)
    return NULL;

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_putnstr_aligned(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_putnstr_aligned _cffi_d_ncplane_putnstr_aligned
#endif

static int _cffi_d_ncplane_putnstr_yx(struct ncplane * x0, int x1, int x2, size_t x3, char const * x4)
{
  return ncplane_putnstr_yx(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_putnstr_yx(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  size_t x3;
  char const * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncplane_putnstr_yx", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_putnstr_yx(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_putnstr_yx _cffi_d_ncplane_putnstr_yx
#endif

static int _cffi_d_ncplane_putstr_aligned(struct ncplane * x0, int x1, ncalign_e x2, char const * x3)
{
  return ncplane_putstr_aligned(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_putstr_aligned(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  ncalign_e x2;
  char const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_putstr_aligned", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(119), arg2) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_putstr_aligned(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_putstr_aligned _cffi_d_ncplane_putstr_aligned
#endif

static int _cffi_d_ncplane_putstr_stained(struct ncplane * x0, char const * x1)
{
  return ncplane_putstr_stained(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_putstr_stained(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_putstr_stained", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_putstr_stained(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_putstr_stained _cffi_d_ncplane_putstr_stained
#endif

static int _cffi_d_ncplane_puttext(struct ncplane * x0, int x1, ncalign_e x2, char const * x3, size_t * x4)
{
  return ncplane_puttext(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_puttext(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  ncalign_e x2;
  char const * x3;
  size_t * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncplane_puttext", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(119), arg2) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(349), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(349), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_puttext(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_puttext _cffi_d_ncplane_puttext
#endif

static int _cffi_d_ncplane_putwegc_stained(struct ncplane * x0, wchar_t const * x1, int * x2)
{
  return ncplane_putwegc_stained(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_putwegc_stained(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  wchar_t const * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncplane_putwegc_stained", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(153), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (wchar_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(153), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_putwegc_stained(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_putwegc_stained _cffi_d_ncplane_putwegc_stained
#endif

static int _cffi_d_ncplane_putwstr_stained(struct ncplane * x0, wchar_t const * x1)
{
  return ncplane_putwstr_stained(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_putwstr_stained(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  wchar_t const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_putwstr_stained", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(153), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (wchar_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(153), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_putwstr_stained(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_putwstr_stained _cffi_d_ncplane_putwstr_stained
#endif

static int _cffi_d_ncplane_qrcode(struct ncplane * x0, ncblitter_e x1, int * x2, int * x3, void const * x4, size_t x5)
{
  return ncplane_qrcode(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_qrcode(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncblitter_e x1;
  int * x2;
  int * x3;
  void const * x4;
  size_t x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "ncplane_qrcode", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(64), arg1) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(191), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(191), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = _cffi_to_c_int(arg5, size_t);
  if (x5 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_qrcode(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_qrcode _cffi_d_ncplane_qrcode
#endif

static struct ncplane * _cffi_d_ncplane_reparent(struct ncplane * x0, struct ncplane * x1)
{
  return ncplane_reparent(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_reparent(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  struct ncplane * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_reparent", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_reparent(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_reparent _cffi_d_ncplane_reparent
#endif

static int _cffi_d_ncplane_resize(struct ncplane * x0, int x1, int x2, int x3, int x4, int x5, int x6, int x7, int x8)
{
  return ncplane_resize(x0, x1, x2, x3, x4, x5, x6, x7, x8);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_resize(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  int x3;
  int x4;
  int x5;
  int x6;
  int x7;
  int x8;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;
  PyObject *arg8;

  if (!PyArg_UnpackTuple(args, "ncplane_resize", 9, 9, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  x6 = _cffi_to_c_int(arg6, int);
  if (x6 == (int)-1 && PyErr_Occurred())
    return NULL;

  x7 = _cffi_to_c_int(arg7, int);
  if (x7 == (int)-1 && PyErr_Occurred())
    return NULL;

  x8 = _cffi_to_c_int(arg8, int);
  if (x8 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_resize(x0, x1, x2, x3, x4, x5, x6, x7, x8); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_resize _cffi_d_ncplane_resize
#endif

static uint32_t * _cffi_d_ncplane_rgba(struct ncplane const * x0, ncblitter_e x1, int x2, int x3, int x4, int x5)
{
  return ncplane_rgba(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_rgba(PyObject *self, PyObject *args)
{
  struct ncplane const * x0;
  ncblitter_e x1;
  int x2;
  int x3;
  int x4;
  int x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint32_t * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "ncplane_rgba", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(64), arg1) < 0)
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_rgba(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(485));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_rgba _cffi_d_ncplane_rgba
#endif

static int _cffi_d_ncplane_rotate_ccw(struct ncplane * x0)
{
  return ncplane_rotate_ccw(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_rotate_ccw(PyObject *self, PyObject *arg0)
{
  struct ncplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_rotate_ccw(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_rotate_ccw _cffi_d_ncplane_rotate_ccw
#endif

static int _cffi_d_ncplane_rotate_cw(struct ncplane * x0)
{
  return ncplane_rotate_cw(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_rotate_cw(PyObject *self, PyObject *arg0)
{
  struct ncplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_rotate_cw(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_rotate_cw _cffi_d_ncplane_rotate_cw
#endif

static void _cffi_d_ncplane_set_attr(struct ncplane * x0, unsigned int x1)
{
  ncplane_set_attr(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_attr(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_attr", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_set_attr(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_set_attr _cffi_d_ncplane_set_attr
#endif

static int _cffi_d_ncplane_set_base(struct ncplane * x0, char const * x1, uint32_t x2, uint64_t x3)
{
  return ncplane_set_base(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_base(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  char const * x1;
  uint32_t x2;
  uint64_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_set_base", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, uint32_t);
  if (x2 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint64_t);
  if (x3 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_base(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_base _cffi_d_ncplane_set_base
#endif

static int _cffi_d_ncplane_set_base_cell(struct ncplane * x0, cell const * x1)
{
  return ncplane_set_base_cell(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_base_cell(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  cell const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_base_cell", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_base_cell(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_base_cell _cffi_d_ncplane_set_base_cell
#endif

static uint64_t _cffi_d_ncplane_set_bchannel(struct ncplane * x0, uint32_t x1)
{
  return ncplane_set_bchannel(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_bchannel(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  uint32_t x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint64_t result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_bchannel", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint32_t);
  if (x1 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_bchannel(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, uint64_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_bchannel _cffi_d_ncplane_set_bchannel
#endif

static int _cffi_d_ncplane_set_bg_alpha(struct ncplane * x0, unsigned int x1)
{
  return ncplane_set_bg_alpha(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_bg_alpha(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_bg_alpha", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_bg_alpha(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_bg_alpha _cffi_d_ncplane_set_bg_alpha
#endif

static void _cffi_d_ncplane_set_bg_default(struct ncplane * x0)
{
  ncplane_set_bg_default(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_bg_default(PyObject *self, PyObject *arg0)
{
  struct ncplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_set_bg_default(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_set_bg_default _cffi_d_ncplane_set_bg_default
#endif

static int _cffi_d_ncplane_set_bg_palindex(struct ncplane * x0, int x1)
{
  return ncplane_set_bg_palindex(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_bg_palindex(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_bg_palindex", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_bg_palindex(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_bg_palindex _cffi_d_ncplane_set_bg_palindex
#endif

static int _cffi_d_ncplane_set_bg_rgb(struct ncplane * x0, unsigned int x1)
{
  return ncplane_set_bg_rgb(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_bg_rgb(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_bg_rgb", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_bg_rgb(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_bg_rgb _cffi_d_ncplane_set_bg_rgb
#endif

static int _cffi_d_ncplane_set_bg_rgb8(struct ncplane * x0, int x1, int x2, int x3)
{
  return ncplane_set_bg_rgb8(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_bg_rgb8(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_set_bg_rgb8", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_bg_rgb8(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_bg_rgb8 _cffi_d_ncplane_set_bg_rgb8
#endif

static void _cffi_d_ncplane_set_bg_rgb8_clipped(struct ncplane * x0, int x1, int x2, int x3)
{
  ncplane_set_bg_rgb8_clipped(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_bg_rgb8_clipped(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_set_bg_rgb8_clipped", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_set_bg_rgb8_clipped(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_set_bg_rgb8_clipped _cffi_d_ncplane_set_bg_rgb8_clipped
#endif

static void _cffi_d_ncplane_set_channels(struct ncplane * x0, uint64_t x1)
{
  ncplane_set_channels(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_channels(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  uint64_t x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_channels", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint64_t);
  if (x1 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_set_channels(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_set_channels _cffi_d_ncplane_set_channels
#endif

static uint64_t _cffi_d_ncplane_set_fchannel(struct ncplane * x0, uint32_t x1)
{
  return ncplane_set_fchannel(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_fchannel(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  uint32_t x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint64_t result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_fchannel", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint32_t);
  if (x1 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_fchannel(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, uint64_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_fchannel _cffi_d_ncplane_set_fchannel
#endif

static int _cffi_d_ncplane_set_fg_alpha(struct ncplane * x0, unsigned int x1)
{
  return ncplane_set_fg_alpha(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_fg_alpha(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_fg_alpha", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_fg_alpha(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_fg_alpha _cffi_d_ncplane_set_fg_alpha
#endif

static void _cffi_d_ncplane_set_fg_default(struct ncplane * x0)
{
  ncplane_set_fg_default(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_fg_default(PyObject *self, PyObject *arg0)
{
  struct ncplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_set_fg_default(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_set_fg_default _cffi_d_ncplane_set_fg_default
#endif

static int _cffi_d_ncplane_set_fg_palindex(struct ncplane * x0, int x1)
{
  return ncplane_set_fg_palindex(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_fg_palindex(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_fg_palindex", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_fg_palindex(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_fg_palindex _cffi_d_ncplane_set_fg_palindex
#endif

static int _cffi_d_ncplane_set_fg_rgb(struct ncplane * x0, unsigned int x1)
{
  return ncplane_set_fg_rgb(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_fg_rgb(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_fg_rgb", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_fg_rgb(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_fg_rgb _cffi_d_ncplane_set_fg_rgb
#endif

static int _cffi_d_ncplane_set_fg_rgb8(struct ncplane * x0, int x1, int x2, int x3)
{
  return ncplane_set_fg_rgb8(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_fg_rgb8(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_set_fg_rgb8", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_fg_rgb8(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_fg_rgb8 _cffi_d_ncplane_set_fg_rgb8
#endif

static void _cffi_d_ncplane_set_fg_rgb8_clipped(struct ncplane * x0, int x1, int x2, int x3)
{
  ncplane_set_fg_rgb8_clipped(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_fg_rgb8_clipped(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_set_fg_rgb8_clipped", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_set_fg_rgb8_clipped(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_set_fg_rgb8_clipped _cffi_d_ncplane_set_fg_rgb8_clipped
#endif

static _Bool _cffi_d_ncplane_set_scrolling(struct ncplane * x0, _Bool x1)
{
  return ncplane_set_scrolling(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_scrolling(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  _Bool x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_scrolling", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (_Bool)_cffi_to_c__Bool(arg1);
  if (x1 == (_Bool)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_scrolling(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_scrolling _cffi_d_ncplane_set_scrolling
#endif

static void * _cffi_d_ncplane_set_userptr(struct ncplane * x0, void * x1)
{
  return ncplane_set_userptr(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_set_userptr(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  void * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  void * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_set_userptr", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(187), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(187), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_set_userptr(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(187));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_set_userptr _cffi_d_ncplane_set_userptr
#endif

static int _cffi_d_ncplane_stain(struct ncplane * x0, int x1, int x2, uint64_t x3, uint64_t x4, uint64_t x5, uint64_t x6)
{
  return ncplane_stain(x0, x1, x2, x3, x4, x5, x6);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_stain(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int x1;
  int x2;
  uint64_t x3;
  uint64_t x4;
  uint64_t x5;
  uint64_t x6;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;

  if (!PyArg_UnpackTuple(args, "ncplane_stain", 7, 7, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint64_t);
  if (x3 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, uint64_t);
  if (x4 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, uint64_t);
  if (x5 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x6 = _cffi_to_c_int(arg6, uint64_t);
  if (x6 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_stain(x0, x1, x2, x3, x4, x5, x6); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_stain _cffi_d_ncplane_stain
#endif

static unsigned int _cffi_d_ncplane_styles(struct ncplane const * x0)
{
  return ncplane_styles(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_styles(PyObject *self, PyObject *arg0)
{
  struct ncplane const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_styles(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, unsigned int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_styles _cffi_d_ncplane_styles
#endif

static void _cffi_d_ncplane_styles_off(struct ncplane * x0, unsigned int x1)
{
  ncplane_styles_off(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_styles_off(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_styles_off", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_styles_off(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_styles_off _cffi_d_ncplane_styles_off
#endif

static void _cffi_d_ncplane_styles_on(struct ncplane * x0, unsigned int x1)
{
  ncplane_styles_on(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_styles_on(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_styles_on", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_styles_on(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_styles_on _cffi_d_ncplane_styles_on
#endif

static void _cffi_d_ncplane_styles_set(struct ncplane * x0, unsigned int x1)
{
  ncplane_styles_set(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_styles_set(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncplane_styles_set", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_styles_set(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_styles_set _cffi_d_ncplane_styles_set
#endif

static void _cffi_d_ncplane_translate(struct ncplane const * x0, struct ncplane const * x1, int * x2, int * x3)
{
  ncplane_translate(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_translate(PyObject *self, PyObject *args)
{
  struct ncplane const * x0;
  struct ncplane const * x1;
  int * x2;
  int * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncplane_translate", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_translate(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_translate _cffi_d_ncplane_translate
#endif

static _Bool _cffi_d_ncplane_translate_abs(struct ncplane const * x0, int * x1, int * x2)
{
  return ncplane_translate_abs(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_translate_abs(PyObject *self, PyObject *args)
{
  struct ncplane const * x0;
  int * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncplane_translate_abs", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_translate_abs(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_translate_abs _cffi_d_ncplane_translate_abs
#endif

static void * _cffi_d_ncplane_userptr(struct ncplane * x0)
{
  return ncplane_userptr(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_userptr(PyObject *self, PyObject *arg0)
{
  struct ncplane * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  void * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_userptr(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(187));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_userptr _cffi_d_ncplane_userptr
#endif

static int _cffi_d_ncplane_vline_interp(struct ncplane * x0, cell const * x1, int x2, uint64_t x3, uint64_t x4)
{
  return ncplane_vline_interp(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_vline_interp(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  cell const * x1;
  int x2;
  uint64_t x3;
  uint64_t x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncplane_vline_interp", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(75), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (cell const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(75), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint64_t);
  if (x3 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, uint64_t);
  if (x4 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_vline_interp(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_vline_interp _cffi_d_ncplane_vline_interp
#endif

static int _cffi_d_ncplane_x(struct ncplane const * x0)
{
  return ncplane_x(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_x(PyObject *self, PyObject *arg0)
{
  struct ncplane const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_x(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_x _cffi_d_ncplane_x
#endif

static int _cffi_d_ncplane_y(struct ncplane const * x0)
{
  return ncplane_y(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_y(PyObject *self, PyObject *arg0)
{
  struct ncplane const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncplane_y(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncplane_y _cffi_d_ncplane_y
#endif

static void _cffi_d_ncplane_yx(struct ncplane * x0, int * x1, int * x2)
{
  ncplane_yx(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncplane_yx(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  int * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncplane_yx", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncplane_yx(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncplane_yx _cffi_d_ncplane_yx
#endif

static int _cffi_d_ncreader_clear(struct ncreader * x0)
{
  return ncreader_clear(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreader_clear(PyObject *self, PyObject *arg0)
{
  struct ncreader * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(21), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreader *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(21), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreader_clear(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreader_clear _cffi_d_ncreader_clear
#endif

static char * _cffi_d_ncreader_contents(struct ncreader const * x0)
{
  return ncreader_contents(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreader_contents(PyObject *self, PyObject *arg0)
{
  struct ncreader const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreader const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreader_contents(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(86));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreader_contents _cffi_d_ncreader_contents
#endif

static struct ncreader * _cffi_d_ncreader_create(struct ncplane * x0, ncreader_options const * x1)
{
  return ncreader_create(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreader_create(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncreader_options const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncreader * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncreader_create", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(629), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncreader_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(629), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreader_create(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(21));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreader_create _cffi_d_ncreader_create
#endif

static void _cffi_d_ncreader_destroy(struct ncreader * x0, char * * x1)
{
  ncreader_destroy(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreader_destroy(PyObject *self, PyObject *args)
{
  struct ncreader * x0;
  char * * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncreader_destroy", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(21), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreader *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(21), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(784), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(784), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncreader_destroy(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncreader_destroy _cffi_d_ncreader_destroy
#endif

static int _cffi_d_ncreader_move_down(struct ncreader * x0)
{
  return ncreader_move_down(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreader_move_down(PyObject *self, PyObject *arg0)
{
  struct ncreader * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(21), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreader *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(21), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreader_move_down(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreader_move_down _cffi_d_ncreader_move_down
#endif

static int _cffi_d_ncreader_move_left(struct ncreader * x0)
{
  return ncreader_move_left(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreader_move_left(PyObject *self, PyObject *arg0)
{
  struct ncreader * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(21), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreader *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(21), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreader_move_left(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreader_move_left _cffi_d_ncreader_move_left
#endif

static int _cffi_d_ncreader_move_right(struct ncreader * x0)
{
  return ncreader_move_right(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreader_move_right(PyObject *self, PyObject *arg0)
{
  struct ncreader * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(21), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreader *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(21), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreader_move_right(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreader_move_right _cffi_d_ncreader_move_right
#endif

static int _cffi_d_ncreader_move_up(struct ncreader * x0)
{
  return ncreader_move_up(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreader_move_up(PyObject *self, PyObject *arg0)
{
  struct ncreader * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(21), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreader *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(21), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreader_move_up(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreader_move_up _cffi_d_ncreader_move_up
#endif

static _Bool _cffi_d_ncreader_offer_input(struct ncreader * x0, ncinput const * x1)
{
  return ncreader_offer_input(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreader_offer_input(PyObject *self, PyObject *args)
{
  struct ncreader * x0;
  ncinput const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncreader_offer_input", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(21), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreader *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(21), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncinput const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreader_offer_input(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreader_offer_input _cffi_d_ncreader_offer_input
#endif

static struct ncplane * _cffi_d_ncreader_plane(struct ncreader * x0)
{
  return ncreader_plane(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreader_plane(PyObject *self, PyObject *arg0)
{
  struct ncreader * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(21), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreader *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(21), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreader_plane(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreader_plane _cffi_d_ncreader_plane
#endif

static int _cffi_d_ncreader_write_egc(struct ncreader * x0, char const * x1)
{
  return ncreader_write_egc(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreader_write_egc(PyObject *self, PyObject *args)
{
  struct ncreader * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncreader_write_egc", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(21), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreader *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(21), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreader_write_egc(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreader_write_egc _cffi_d_ncreader_write_egc
#endif

static struct nctablet * _cffi_d_ncreel_add(struct ncreel * x0, struct nctablet * x1, struct nctablet * x2, int(* x3)(struct nctablet *, _Bool), void * x4)
{
  return ncreel_add(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreel_add(PyObject *self, PyObject *args)
{
  struct ncreel * x0;
  struct nctablet * x1;
  struct nctablet * x2;
  int(* x3)(struct nctablet *, _Bool);
  void * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct nctablet * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ncreel_add", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(423), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreel *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(423), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(427), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct nctablet *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(427), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(427), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (struct nctablet *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(427), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = (int(*)(struct nctablet *, _Bool))_cffi_to_c_pointer(arg3, _cffi_type(663));
  if (x3 == (int(*)(struct nctablet *, _Bool))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(187), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(187), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreel_add(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(427));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreel_add _cffi_d_ncreel_add
#endif

static struct ncreel * _cffi_d_ncreel_create(struct ncplane * x0, ncreel_options const * x1)
{
  return ncreel_create(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreel_create(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncreel_options const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncreel * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncreel_create", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(633), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncreel_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(633), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreel_create(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(423));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreel_create _cffi_d_ncreel_create
#endif

static int _cffi_d_ncreel_del(struct ncreel * x0, struct nctablet * x1)
{
  return ncreel_del(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreel_del(PyObject *self, PyObject *args)
{
  struct ncreel * x0;
  struct nctablet * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncreel_del", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(423), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreel *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(423), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(427), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct nctablet *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(427), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreel_del(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreel_del _cffi_d_ncreel_del
#endif

static int _cffi_d_ncreel_destroy(struct ncreel * x0)
{
  return ncreel_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreel_destroy(PyObject *self, PyObject *arg0)
{
  struct ncreel * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(423), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreel *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(423), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreel_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreel_destroy _cffi_d_ncreel_destroy
#endif

static struct nctablet * _cffi_d_ncreel_focused(struct ncreel * x0)
{
  return ncreel_focused(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreel_focused(PyObject *self, PyObject *arg0)
{
  struct ncreel * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct nctablet * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(423), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreel *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(423), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreel_focused(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(427));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreel_focused _cffi_d_ncreel_focused
#endif

static struct nctablet * _cffi_d_ncreel_next(struct ncreel * x0)
{
  return ncreel_next(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreel_next(PyObject *self, PyObject *arg0)
{
  struct ncreel * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct nctablet * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(423), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreel *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(423), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreel_next(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(427));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreel_next _cffi_d_ncreel_next
#endif

static struct ncplane * _cffi_d_ncreel_plane(struct ncreel * x0)
{
  return ncreel_plane(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreel_plane(PyObject *self, PyObject *arg0)
{
  struct ncreel * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(423), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreel *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(423), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreel_plane(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreel_plane _cffi_d_ncreel_plane
#endif

static struct nctablet * _cffi_d_ncreel_prev(struct ncreel * x0)
{
  return ncreel_prev(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreel_prev(PyObject *self, PyObject *arg0)
{
  struct ncreel * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct nctablet * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(423), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreel *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(423), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreel_prev(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(427));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreel_prev _cffi_d_ncreel_prev
#endif

static int _cffi_d_ncreel_redraw(struct ncreel * x0)
{
  return ncreel_redraw(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreel_redraw(PyObject *self, PyObject *arg0)
{
  struct ncreel * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(423), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreel *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(423), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreel_redraw(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreel_redraw _cffi_d_ncreel_redraw
#endif

static int _cffi_d_ncreel_tabletcount(struct ncreel const * x0)
{
  return ncreel_tabletcount(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncreel_tabletcount(PyObject *self, PyObject *arg0)
{
  struct ncreel const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(430), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncreel const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(430), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncreel_tabletcount(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncreel_tabletcount _cffi_d_ncreel_tabletcount
#endif

static int _cffi_d_ncselector_additem(struct ncselector * x0, struct ncselector_item const * x1)
{
  return ncselector_additem(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncselector_additem(PyObject *self, PyObject *args)
{
  struct ncselector * x0;
  struct ncselector_item const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncselector_additem", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncselector *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(438), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct ncselector_item const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(438), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncselector_additem(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncselector_additem _cffi_d_ncselector_additem
#endif

static struct ncselector * _cffi_d_ncselector_create(struct ncplane * x0, ncselector_options const * x1)
{
  return ncselector_create(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncselector_create(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncselector_options const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncselector * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncselector_create", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(637), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncselector_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(637), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncselector_create(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(25));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncselector_create _cffi_d_ncselector_create
#endif

static int _cffi_d_ncselector_delitem(struct ncselector * x0, char const * x1)
{
  return ncselector_delitem(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncselector_delitem(PyObject *self, PyObject *args)
{
  struct ncselector * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncselector_delitem", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncselector *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncselector_delitem(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncselector_delitem _cffi_d_ncselector_delitem
#endif

static void _cffi_d_ncselector_destroy(struct ncselector * x0, char * * x1)
{
  ncselector_destroy(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncselector_destroy(PyObject *self, PyObject *args)
{
  struct ncselector * x0;
  char * * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncselector_destroy", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncselector *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(784), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(784), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncselector_destroy(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncselector_destroy _cffi_d_ncselector_destroy
#endif

static char const * _cffi_d_ncselector_nextitem(struct ncselector * x0)
{
  return ncselector_nextitem(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncselector_nextitem(PyObject *self, PyObject *arg0)
{
  struct ncselector * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncselector *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncselector_nextitem(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(92));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncselector_nextitem _cffi_d_ncselector_nextitem
#endif

static _Bool _cffi_d_ncselector_offer_input(struct ncselector * x0, ncinput const * x1)
{
  return ncselector_offer_input(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncselector_offer_input(PyObject *self, PyObject *args)
{
  struct ncselector * x0;
  ncinput const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncselector_offer_input", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncselector *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncinput const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncselector_offer_input(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncselector_offer_input _cffi_d_ncselector_offer_input
#endif

static struct ncplane * _cffi_d_ncselector_plane(struct ncselector * x0)
{
  return ncselector_plane(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncselector_plane(PyObject *self, PyObject *arg0)
{
  struct ncselector * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncselector *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncselector_plane(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncselector_plane _cffi_d_ncselector_plane
#endif

static char const * _cffi_d_ncselector_previtem(struct ncselector * x0)
{
  return ncselector_previtem(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncselector_previtem(PyObject *self, PyObject *arg0)
{
  struct ncselector * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncselector *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncselector_previtem(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(92));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncselector_previtem _cffi_d_ncselector_previtem
#endif

static char const * _cffi_d_ncselector_selected(struct ncselector const * x0)
{
  return ncselector_selected(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncselector_selected(PyObject *self, PyObject *arg0)
{
  struct ncselector const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(81), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncselector const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(81), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncselector_selected(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(92));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncselector_selected _cffi_d_ncselector_selected
#endif

static int _cffi_d_ncstrwidth(char const * x0)
{
  return ncstrwidth(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncstrwidth(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncstrwidth(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncstrwidth _cffi_d_ncstrwidth
#endif

static struct ncsubproc * _cffi_d_ncsubproc_createv(struct ncplane * x0, ncsubproc_options const * x1, char const * x2, char * const * x3, int(* x4)(struct ncfdplane *, void const *, size_t, void *), int(* x5)(struct ncfdplane *, int, void *))
{
  return ncsubproc_createv(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncsubproc_createv(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncsubproc_options const * x1;
  char const * x2;
  char * const * x3;
  int(* x4)(struct ncfdplane *, void const *, size_t, void *);
  int(* x5)(struct ncfdplane *, int, void *);
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncsubproc * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "ncsubproc_createv", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(641), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncsubproc_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(641), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(643), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (char * const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(643), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = (int(*)(struct ncfdplane *, void const *, size_t, void *))_cffi_to_c_pointer(arg4, _cffi_type(560));
  if (x4 == (int(*)(struct ncfdplane *, void const *, size_t, void *))NULL && PyErr_Occurred())
    return NULL;

  x5 = (int(*)(struct ncfdplane *, int, void *))_cffi_to_c_pointer(arg5, _cffi_type(561));
  if (x5 == (int(*)(struct ncfdplane *, int, void *))NULL && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncsubproc_createv(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(441));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncsubproc_createv _cffi_d_ncsubproc_createv
#endif

static struct ncsubproc * _cffi_d_ncsubproc_createvp(struct ncplane * x0, ncsubproc_options const * x1, char const * x2, char * const * x3, int(* x4)(struct ncfdplane *, void const *, size_t, void *), int(* x5)(struct ncfdplane *, int, void *))
{
  return ncsubproc_createvp(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncsubproc_createvp(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncsubproc_options const * x1;
  char const * x2;
  char * const * x3;
  int(* x4)(struct ncfdplane *, void const *, size_t, void *);
  int(* x5)(struct ncfdplane *, int, void *);
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncsubproc * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "ncsubproc_createvp", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(641), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncsubproc_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(641), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(643), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (char * const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(643), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = (int(*)(struct ncfdplane *, void const *, size_t, void *))_cffi_to_c_pointer(arg4, _cffi_type(560));
  if (x4 == (int(*)(struct ncfdplane *, void const *, size_t, void *))NULL && PyErr_Occurred())
    return NULL;

  x5 = (int(*)(struct ncfdplane *, int, void *))_cffi_to_c_pointer(arg5, _cffi_type(561));
  if (x5 == (int(*)(struct ncfdplane *, int, void *))NULL && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncsubproc_createvp(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(441));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncsubproc_createvp _cffi_d_ncsubproc_createvp
#endif

static struct ncsubproc * _cffi_d_ncsubproc_createvpe(struct ncplane * x0, ncsubproc_options const * x1, char const * x2, char * const * x3, char * const * x4, int(* x5)(struct ncfdplane *, void const *, size_t, void *), int(* x6)(struct ncfdplane *, int, void *))
{
  return ncsubproc_createvpe(x0, x1, x2, x3, x4, x5, x6);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncsubproc_createvpe(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncsubproc_options const * x1;
  char const * x2;
  char * const * x3;
  char * const * x4;
  int(* x5)(struct ncfdplane *, void const *, size_t, void *);
  int(* x6)(struct ncfdplane *, int, void *);
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncsubproc * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;

  if (!PyArg_UnpackTuple(args, "ncsubproc_createvpe", 7, 7, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(641), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncsubproc_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(641), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(643), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (char * const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(643), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(643), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (char * const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(643), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (int(*)(struct ncfdplane *, void const *, size_t, void *))_cffi_to_c_pointer(arg5, _cffi_type(560));
  if (x5 == (int(*)(struct ncfdplane *, void const *, size_t, void *))NULL && PyErr_Occurred())
    return NULL;

  x6 = (int(*)(struct ncfdplane *, int, void *))_cffi_to_c_pointer(arg6, _cffi_type(561));
  if (x6 == (int(*)(struct ncfdplane *, int, void *))NULL && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncsubproc_createvpe(x0, x1, x2, x3, x4, x5, x6); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(441));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncsubproc_createvpe _cffi_d_ncsubproc_createvpe
#endif

static int _cffi_d_ncsubproc_destroy(struct ncsubproc * x0)
{
  return ncsubproc_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncsubproc_destroy(PyObject *self, PyObject *arg0)
{
  struct ncsubproc * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(441), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncsubproc *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(441), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncsubproc_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncsubproc_destroy _cffi_d_ncsubproc_destroy
#endif

static struct ncplane * _cffi_d_ncsubproc_plane(struct ncsubproc * x0)
{
  return ncsubproc_plane(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncsubproc_plane(PyObject *self, PyObject *arg0)
{
  struct ncsubproc * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(441), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncsubproc *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(441), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncsubproc_plane(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncsubproc_plane _cffi_d_ncsubproc_plane
#endif

static struct ncplane * _cffi_d_nctablet_ncplane(struct nctablet * x0)
{
  return nctablet_ncplane(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_nctablet_ncplane(PyObject *self, PyObject *arg0)
{
  struct nctablet * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(427), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct nctablet *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(427), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = nctablet_ncplane(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_nctablet_ncplane _cffi_d_nctablet_ncplane
#endif

static void * _cffi_d_nctablet_userptr(struct nctablet * x0)
{
  return nctablet_userptr(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_nctablet_userptr(PyObject *self, PyObject *arg0)
{
  struct nctablet * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  void * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(427), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct nctablet *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(427), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = nctablet_userptr(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(187));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_nctablet_userptr _cffi_d_nctablet_userptr
#endif

static int _cffi_d_ncuplot_add_sample(struct ncuplot * x0, uint64_t x1, uint64_t x2)
{
  return ncuplot_add_sample(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncuplot_add_sample(PyObject *self, PyObject *args)
{
  struct ncuplot * x0;
  uint64_t x1;
  uint64_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncuplot_add_sample", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(448), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncuplot *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(448), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint64_t);
  if (x1 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, uint64_t);
  if (x2 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncuplot_add_sample(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncuplot_add_sample _cffi_d_ncuplot_add_sample
#endif

static struct ncuplot * _cffi_d_ncuplot_create(struct ncplane * x0, ncplot_options const * x1, uint64_t x2, uint64_t x3)
{
  return ncuplot_create(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncuplot_create(PyObject *self, PyObject *args)
{
  struct ncplane * x0;
  ncplot_options const * x1;
  uint64_t x2;
  uint64_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncuplot * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncuplot_create", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(12), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(12), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(552), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncplot_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(552), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, uint64_t);
  if (x2 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint64_t);
  if (x3 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncuplot_create(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(448));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncuplot_create _cffi_d_ncuplot_create
#endif

static void _cffi_d_ncuplot_destroy(struct ncuplot * x0)
{
  ncuplot_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncuplot_destroy(PyObject *self, PyObject *arg0)
{
  struct ncuplot * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(448), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncuplot *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(448), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncuplot_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncuplot_destroy _cffi_d_ncuplot_destroy
#endif

static struct ncplane * _cffi_d_ncuplot_plane(struct ncuplot * x0)
{
  return ncuplot_plane(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncuplot_plane(PyObject *self, PyObject *arg0)
{
  struct ncuplot * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(448), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncuplot *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(448), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncuplot_plane(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncuplot_plane _cffi_d_ncuplot_plane
#endif

static int _cffi_d_ncuplot_sample(struct ncuplot const * x0, uint64_t x1, uint64_t * x2)
{
  return ncuplot_sample(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncuplot_sample(PyObject *self, PyObject *args)
{
  struct ncuplot const * x0;
  uint64_t x1;
  uint64_t * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncuplot_sample", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(453), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncuplot const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(453), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint64_t);
  if (x1 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(34), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (uint64_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(34), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncuplot_sample(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncuplot_sample _cffi_d_ncuplot_sample
#endif

static int _cffi_d_ncuplot_set_sample(struct ncuplot * x0, uint64_t x1, uint64_t x2)
{
  return ncuplot_set_sample(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncuplot_set_sample(PyObject *self, PyObject *args)
{
  struct ncuplot * x0;
  uint64_t x1;
  uint64_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncuplot_set_sample", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(448), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncuplot *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(448), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint64_t);
  if (x1 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, uint64_t);
  if (x2 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncuplot_set_sample(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncuplot_set_sample _cffi_d_ncuplot_set_sample
#endif

static int _cffi_d_ncvisual_at_yx(struct ncvisual const * x0, int x1, int x2, uint32_t * x3)
{
  return ncvisual_at_yx(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_at_yx(PyObject *self, PyObject *args)
{
  struct ncvisual const * x0;
  int x1;
  int x2;
  uint32_t * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncvisual_at_yx", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(54), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncvisual const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(54), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(485), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (uint32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(485), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_at_yx(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_at_yx _cffi_d_ncvisual_at_yx
#endif

static int _cffi_d_ncvisual_decode(struct ncvisual * x0)
{
  return ncvisual_decode(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_decode(PyObject *self, PyObject *arg0)
{
  struct ncvisual * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(458), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncvisual *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(458), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_decode(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_decode _cffi_d_ncvisual_decode
#endif

static void _cffi_d_ncvisual_destroy(struct ncvisual * x0)
{
  ncvisual_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_destroy(PyObject *self, PyObject *arg0)
{
  struct ncvisual * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(458), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncvisual *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(458), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ncvisual_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ncvisual_destroy _cffi_d_ncvisual_destroy
#endif

static struct ncvisual * _cffi_d_ncvisual_from_bgra(void const * x0, int x1, int x2, int x3)
{
  return ncvisual_from_bgra(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_from_bgra(PyObject *self, PyObject *args)
{
  void const * x0;
  int x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncvisual * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncvisual_from_bgra", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(191), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(191), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_from_bgra(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(458));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_from_bgra _cffi_d_ncvisual_from_bgra
#endif

static struct ncvisual * _cffi_d_ncvisual_from_file(char const * x0)
{
  return ncvisual_from_file(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_from_file(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncvisual * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_from_file(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(458));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_from_file _cffi_d_ncvisual_from_file
#endif

static struct ncvisual * _cffi_d_ncvisual_from_plane(struct ncplane const * x0, ncblitter_e x1, int x2, int x3, int x4, int x5)
{
  return ncvisual_from_plane(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_from_plane(PyObject *self, PyObject *args)
{
  struct ncplane const * x0;
  ncblitter_e x1;
  int x2;
  int x3;
  int x4;
  int x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncvisual * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "ncvisual_from_plane", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncplane const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(64), arg1) < 0)
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_from_plane(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(458));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_from_plane _cffi_d_ncvisual_from_plane
#endif

static struct ncvisual * _cffi_d_ncvisual_from_rgba(void const * x0, int x1, int x2, int x3)
{
  return ncvisual_from_rgba(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_from_rgba(PyObject *self, PyObject *args)
{
  void const * x0;
  int x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncvisual * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncvisual_from_rgba", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(191), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(191), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_from_rgba(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(458));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_from_rgba _cffi_d_ncvisual_from_rgba
#endif

static int _cffi_d_ncvisual_geom(struct notcurses const * x0, struct ncvisual const * x1, struct ncvisual_options const * x2, int * x3, int * x4, int * x5, int * x6)
{
  return ncvisual_geom(x0, x1, x2, x3, x4, x5, x6);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_geom(PyObject *self, PyObject *args)
{
  struct notcurses const * x0;
  struct ncvisual const * x1;
  struct ncvisual_options const * x2;
  int * x3;
  int * x4;
  int * x5;
  int * x6;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;

  if (!PyArg_UnpackTuple(args, "ncvisual_geom", 7, 7, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(29), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(29), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(54), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct ncvisual const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(54), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(525), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (struct ncvisual_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(525), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg6, (char **)&x6);
  if (datasize != 0) {
    x6 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg6, (char **)&x6,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_geom(x0, x1, x2, x3, x4, x5, x6); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_geom _cffi_d_ncvisual_geom
#endif

static int _cffi_d_ncvisual_polyfill_yx(struct ncvisual * x0, int x1, int x2, uint32_t x3)
{
  return ncvisual_polyfill_yx(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_polyfill_yx(PyObject *self, PyObject *args)
{
  struct ncvisual * x0;
  int x1;
  int x2;
  uint32_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncvisual_polyfill_yx", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(458), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncvisual *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(458), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint32_t);
  if (x3 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_polyfill_yx(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_polyfill_yx _cffi_d_ncvisual_polyfill_yx
#endif

static struct ncplane * _cffi_d_ncvisual_render(struct notcurses * x0, struct ncvisual * x1, struct ncvisual_options const * x2)
{
  return ncvisual_render(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_render(PyObject *self, PyObject *args)
{
  struct notcurses * x0;
  struct ncvisual * x1;
  struct ncvisual_options const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncvisual_render", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(458), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct ncvisual *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(458), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(525), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (struct ncvisual_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(525), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_render(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_render _cffi_d_ncvisual_render
#endif

static int _cffi_d_ncvisual_resize(struct ncvisual * x0, int x1, int x2)
{
  return ncvisual_resize(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_resize(PyObject *self, PyObject *args)
{
  struct ncvisual * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ncvisual_resize", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(458), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncvisual *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(458), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_resize(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_resize _cffi_d_ncvisual_resize
#endif

static int _cffi_d_ncvisual_rotate(struct ncvisual * x0, double x1)
{
  return ncvisual_rotate(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_rotate(PyObject *self, PyObject *args)
{
  struct ncvisual * x0;
  double x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ncvisual_rotate", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(458), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncvisual *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(458), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (double)_cffi_to_c_double(arg1);
  if (x1 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_rotate(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_rotate _cffi_d_ncvisual_rotate
#endif

static int _cffi_d_ncvisual_set_yx(struct ncvisual const * x0, int x1, int x2, uint32_t x3)
{
  return ncvisual_set_yx(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_set_yx(PyObject *self, PyObject *args)
{
  struct ncvisual const * x0;
  int x1;
  int x2;
  uint32_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ncvisual_set_yx", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(54), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncvisual const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(54), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint32_t);
  if (x3 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_set_yx(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_set_yx _cffi_d_ncvisual_set_yx
#endif

static int _cffi_d_ncvisual_stream(struct notcurses * x0, struct ncvisual * x1, float x2, int(* x3)(struct ncvisual *, struct ncvisual_options *, struct timespec const *, void *), struct ncvisual_options const * x4, void * x5)
{
  return ncvisual_stream(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_stream(PyObject *self, PyObject *args)
{
  struct notcurses * x0;
  struct ncvisual * x1;
  float x2;
  int(* x3)(struct ncvisual *, struct ncvisual_options *, struct timespec const *, void *);
  struct ncvisual_options const * x4;
  void * x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "ncvisual_stream", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(458), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct ncvisual *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(458), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (int(*)(struct ncvisual *, struct ncvisual_options *, struct timespec const *, void *))_cffi_to_c_pointer(arg3, _cffi_type(524));
  if (x3 == (int(*)(struct ncvisual *, struct ncvisual_options *, struct timespec const *, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(525), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (struct ncvisual_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(525), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(187), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(187), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_stream(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_stream _cffi_d_ncvisual_stream
#endif

static char * _cffi_d_ncvisual_subtitle(struct ncvisual const * x0)
{
  return ncvisual_subtitle(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ncvisual_subtitle(PyObject *self, PyObject *arg0)
{
  struct ncvisual const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(54), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ncvisual const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(54), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ncvisual_subtitle(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(86));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ncvisual_subtitle _cffi_d_ncvisual_subtitle
#endif

static char * _cffi_d_notcurses_at_yx(struct notcurses * x0, int x1, int x2, uint16_t * x3, uint64_t * x4)
{
  return notcurses_at_yx(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_at_yx(PyObject *self, PyObject *args)
{
  struct notcurses * x0;
  int x1;
  int x2;
  uint16_t * x3;
  uint64_t * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "notcurses_at_yx", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(33), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (uint16_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(33), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(34), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (uint64_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(34), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_at_yx(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(86));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_at_yx _cffi_d_notcurses_at_yx
#endif

static struct ncplane * _cffi_d_notcurses_bottom(struct notcurses * x0)
{
  return notcurses_bottom(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_bottom(PyObject *self, PyObject *arg0)
{
  struct notcurses * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_bottom(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_bottom _cffi_d_notcurses_bottom
#endif

static _Bool _cffi_d_notcurses_canchangecolor(struct notcurses const * x0)
{
  return notcurses_canchangecolor(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_canchangecolor(PyObject *self, PyObject *arg0)
{
  struct notcurses const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(29), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(29), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_canchangecolor(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_canchangecolor _cffi_d_notcurses_canchangecolor
#endif

static _Bool _cffi_d_notcurses_canfade(struct notcurses const * x0)
{
  return notcurses_canfade(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_canfade(PyObject *self, PyObject *arg0)
{
  struct notcurses const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(29), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(29), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_canfade(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_canfade _cffi_d_notcurses_canfade
#endif

static _Bool _cffi_d_notcurses_canopen_images(struct notcurses const * x0)
{
  return notcurses_canopen_images(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_canopen_images(PyObject *self, PyObject *arg0)
{
  struct notcurses const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(29), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(29), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_canopen_images(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_canopen_images _cffi_d_notcurses_canopen_images
#endif

static _Bool _cffi_d_notcurses_canopen_videos(struct notcurses const * x0)
{
  return notcurses_canopen_videos(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_canopen_videos(PyObject *self, PyObject *arg0)
{
  struct notcurses const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(29), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(29), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_canopen_videos(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_canopen_videos _cffi_d_notcurses_canopen_videos
#endif

static _Bool _cffi_d_notcurses_cansixel(struct notcurses const * x0)
{
  return notcurses_cansixel(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_cansixel(PyObject *self, PyObject *arg0)
{
  struct notcurses const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(29), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(29), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_cansixel(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_cansixel _cffi_d_notcurses_cansixel
#endif

static _Bool _cffi_d_notcurses_cantruecolor(struct notcurses const * x0)
{
  return notcurses_cantruecolor(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_cantruecolor(PyObject *self, PyObject *arg0)
{
  struct notcurses const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(29), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(29), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_cantruecolor(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_cantruecolor _cffi_d_notcurses_cantruecolor
#endif

static _Bool _cffi_d_notcurses_canutf8(struct notcurses const * x0)
{
  return notcurses_canutf8(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_canutf8(PyObject *self, PyObject *arg0)
{
  struct notcurses const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(29), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(29), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_canutf8(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_canutf8 _cffi_d_notcurses_canutf8
#endif

static int _cffi_d_notcurses_cursor_disable(struct notcurses * x0)
{
  return notcurses_cursor_disable(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_cursor_disable(PyObject *self, PyObject *arg0)
{
  struct notcurses * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_cursor_disable(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_cursor_disable _cffi_d_notcurses_cursor_disable
#endif

static int _cffi_d_notcurses_cursor_enable(struct notcurses * x0, int x1, int x2)
{
  return notcurses_cursor_enable(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_cursor_enable(PyObject *self, PyObject *args)
{
  struct notcurses * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "notcurses_cursor_enable", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_cursor_enable(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_cursor_enable _cffi_d_notcurses_cursor_enable
#endif

static void _cffi_d_notcurses_drop_planes(struct notcurses * x0)
{
  notcurses_drop_planes(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_drop_planes(PyObject *self, PyObject *arg0)
{
  struct notcurses * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { notcurses_drop_planes(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_notcurses_drop_planes _cffi_d_notcurses_drop_planes
#endif

static struct notcurses * _cffi_d_notcurses_init(notcurses_options const * x0, FILE * x1)
{
  return notcurses_init(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_init(PyObject *self, PyObject *args)
{
  notcurses_options const * x0;
  FILE * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct notcurses * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "notcurses_init", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(690), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (notcurses_options const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(690), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(498), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (FILE *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(498), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_init(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(57));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_init _cffi_d_notcurses_init
#endif

static int _cffi_d_notcurses_inputready_fd(struct notcurses * x0)
{
  return notcurses_inputready_fd(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_inputready_fd(PyObject *self, PyObject *arg0)
{
  struct notcurses * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_inputready_fd(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_inputready_fd _cffi_d_notcurses_inputready_fd
#endif

static int _cffi_d_notcurses_lex_blitter(char const * x0, ncblitter_e * x1)
{
  return notcurses_lex_blitter(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_lex_blitter(PyObject *self, PyObject *args)
{
  char const * x0;
  ncblitter_e * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "notcurses_lex_blitter", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(96), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncblitter_e *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(96), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_lex_blitter(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_lex_blitter _cffi_d_notcurses_lex_blitter
#endif

static int _cffi_d_notcurses_lex_margins(char const * x0, notcurses_options * x1)
{
  return notcurses_lex_margins(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_lex_margins(PyObject *self, PyObject *args)
{
  char const * x0;
  notcurses_options * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "notcurses_lex_margins", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(104), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (notcurses_options *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(104), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_lex_margins(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_lex_margins _cffi_d_notcurses_lex_margins
#endif

static int _cffi_d_notcurses_lex_scalemode(char const * x0, ncscale_e * x1)
{
  return notcurses_lex_scalemode(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_lex_scalemode(PyObject *self, PyObject *args)
{
  char const * x0;
  ncscale_e * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "notcurses_lex_scalemode", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(100), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncscale_e *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(100), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_lex_scalemode(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_lex_scalemode _cffi_d_notcurses_lex_scalemode
#endif

static int _cffi_d_notcurses_mouse_disable(struct notcurses * x0)
{
  return notcurses_mouse_disable(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_mouse_disable(PyObject *self, PyObject *arg0)
{
  struct notcurses * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_mouse_disable(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_mouse_disable _cffi_d_notcurses_mouse_disable
#endif

static int _cffi_d_notcurses_mouse_enable(struct notcurses * x0)
{
  return notcurses_mouse_enable(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_mouse_enable(PyObject *self, PyObject *arg0)
{
  struct notcurses * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_mouse_enable(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_mouse_enable _cffi_d_notcurses_mouse_enable
#endif

static unsigned int _cffi_d_notcurses_palette_size(struct notcurses const * x0)
{
  return notcurses_palette_size(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_palette_size(PyObject *self, PyObject *arg0)
{
  struct notcurses const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(29), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(29), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_palette_size(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, unsigned int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_palette_size _cffi_d_notcurses_palette_size
#endif

static int _cffi_d_notcurses_refresh(struct notcurses * x0, int * x1, int * x2)
{
  return notcurses_refresh(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_refresh(PyObject *self, PyObject *args)
{
  struct notcurses * x0;
  int * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "notcurses_refresh", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_refresh(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_refresh _cffi_d_notcurses_refresh
#endif

static int _cffi_d_notcurses_render(struct notcurses * x0)
{
  return notcurses_render(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_render(PyObject *self, PyObject *arg0)
{
  struct notcurses * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_render(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_render _cffi_d_notcurses_render
#endif

static int _cffi_d_notcurses_render_to_file(struct notcurses * x0, FILE * x1)
{
  return notcurses_render_to_file(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_render_to_file(PyObject *self, PyObject *args)
{
  struct notcurses * x0;
  FILE * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "notcurses_render_to_file", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(498), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (FILE *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(498), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_render_to_file(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_render_to_file _cffi_d_notcurses_render_to_file
#endif

static void _cffi_d_notcurses_reset_stats(struct notcurses * x0, ncstats * x1)
{
  notcurses_reset_stats(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_reset_stats(PyObject *self, PyObject *args)
{
  struct notcurses * x0;
  ncstats * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "notcurses_reset_stats", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(801), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncstats *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(801), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { notcurses_reset_stats(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_notcurses_reset_stats _cffi_d_notcurses_reset_stats
#endif

static void _cffi_d_notcurses_stats(struct notcurses * x0, ncstats * x1)
{
  notcurses_stats(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_stats(PyObject *self, PyObject *args)
{
  struct notcurses * x0;
  ncstats * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "notcurses_stats", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(801), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ncstats *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(801), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { notcurses_stats(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_notcurses_stats _cffi_d_notcurses_stats
#endif

static struct ncplane * _cffi_d_notcurses_stdplane(struct notcurses * x0)
{
  return notcurses_stdplane(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_stdplane(PyObject *self, PyObject *arg0)
{
  struct notcurses * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_stdplane(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_stdplane _cffi_d_notcurses_stdplane
#endif

static struct ncplane const * _cffi_d_notcurses_stdplane_const(struct notcurses const * x0)
{
  return notcurses_stdplane_const(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_stdplane_const(PyObject *self, PyObject *arg0)
{
  struct notcurses const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(29), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(29), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_stdplane_const(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(16));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_stdplane_const _cffi_d_notcurses_stdplane_const
#endif

static int _cffi_d_notcurses_stop(struct notcurses * x0)
{
  return notcurses_stop(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_stop(PyObject *self, PyObject *arg0)
{
  struct notcurses * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_stop(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_stop _cffi_d_notcurses_stop
#endif

static char const * _cffi_d_notcurses_str_blitter(ncblitter_e x0)
{
  return notcurses_str_blitter(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_str_blitter(PyObject *self, PyObject *arg0)
{
  ncblitter_e x0;
  char const * result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(64), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_str_blitter(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(92));
  return pyresult;
}
#else
#  define _cffi_f_notcurses_str_blitter _cffi_d_notcurses_str_blitter
#endif

static char const * _cffi_d_notcurses_str_scalemode(ncscale_e x0)
{
  return notcurses_str_scalemode(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_str_scalemode(PyObject *self, PyObject *arg0)
{
  ncscale_e x0;
  char const * result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(67), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_str_scalemode(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(92));
  return pyresult;
}
#else
#  define _cffi_f_notcurses_str_scalemode _cffi_d_notcurses_str_scalemode
#endif

static unsigned int _cffi_d_notcurses_supported_styles(struct notcurses const * x0)
{
  return notcurses_supported_styles(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_supported_styles(PyObject *self, PyObject *arg0)
{
  struct notcurses const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(29), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(29), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_supported_styles(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, unsigned int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_supported_styles _cffi_d_notcurses_supported_styles
#endif

static struct ncplane * _cffi_d_notcurses_top(struct notcurses * x0)
{
  return notcurses_top(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_top(PyObject *self, PyObject *arg0)
{
  struct notcurses * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct ncplane * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = notcurses_top(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(12));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_notcurses_top _cffi_d_notcurses_top
#endif

static void _cffi_d_notcurses_version_components(int * x0, int * x1, int * x2, int * x3)
{
  notcurses_version_components(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_notcurses_version_components(PyObject *self, PyObject *args)
{
  int * x0;
  int * x1;
  int * x2;
  int * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "notcurses_version_components", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(17), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(17), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { notcurses_version_components(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_notcurses_version_components _cffi_d_notcurses_version_components
#endif

static void _cffi_d_palette256_free(palette256 * x0)
{
  palette256_free(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_palette256_free(PyObject *self, PyObject *arg0)
{
  palette256 * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(737), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (palette256 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(737), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { palette256_free(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_palette256_free _cffi_d_palette256_free
#endif

static palette256 * _cffi_d_palette256_new(struct notcurses * x0)
{
  return palette256_new(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_palette256_new(PyObject *self, PyObject *arg0)
{
  struct notcurses * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  palette256 * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = palette256_new(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(737));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_palette256_new _cffi_d_palette256_new
#endif

static int _cffi_d_palette256_use(struct notcurses * x0, palette256 const * x1)
{
  return palette256_use(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_palette256_use(PyObject *self, PyObject *args)
{
  struct notcurses * x0;
  palette256 const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "palette256_use", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(57), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct notcurses *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(57), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(512), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (palette256 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(512), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = palette256_use(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_palette256_use _cffi_d_palette256_use
#endif

_CFFI_UNUSED_FN
static void _cffi_checkfld__cell(cell *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->gcluster) | 0);  /* check that 'cell.gcluster' is an integer */
  (void)((p->gcluster_backstop) | 0);  /* check that 'cell.gcluster_backstop' is an integer */
  (void)((p->reserved) | 0);  /* check that 'cell.reserved' is an integer */
  (void)((p->stylemask) | 0);  /* check that 'cell.stylemask' is an integer */
  (void)((p->channels) | 0);  /* check that 'cell.channels' is an integer */
}
struct _cffi_align__cell { char x; cell y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__ncfdplane_options(ncfdplane_options *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { void * *tmp = &p->curry; (void)tmp; }
  (void)((p->follow) | 0);  /* check that 'ncfdplane_options.follow' is an integer */
  (void)((p->flags) | 0);  /* check that 'ncfdplane_options.flags' is an integer */
}
struct _cffi_align__ncfdplane_options { char x; ncfdplane_options y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__ncinput(ncinput *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char32_t *tmp = &p->id; (void)tmp; }
  (void)((p->y) | 0);  /* check that 'ncinput.y' is an integer */
  (void)((p->x) | 0);  /* check that 'ncinput.x' is an integer */
  (void)((p->alt) | 0);  /* check that 'ncinput.alt' is an integer */
  (void)((p->shift) | 0);  /* check that 'ncinput.shift' is an integer */
  (void)((p->ctrl) | 0);  /* check that 'ncinput.ctrl' is an integer */
  (void)((p->seqnum) | 0);  /* check that 'ncinput.seqnum' is an integer */
}
struct _cffi_align__ncinput { char x; ncinput y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_struct_ncmenu_item(struct ncmenu_item *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char * *tmp = &p->desc; (void)tmp; }
  { ncinput *tmp = &p->shortcut; (void)tmp; }
}
struct _cffi_align_struct_ncmenu_item { char x; struct ncmenu_item y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__ncmenu_options(ncmenu_options *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { struct ncmenu_section * *tmp = &p->sections; (void)tmp; }
  (void)((p->sectioncount) | 0);  /* check that 'ncmenu_options.sectioncount' is an integer */
  (void)((p->headerchannels) | 0);  /* check that 'ncmenu_options.headerchannels' is an integer */
  (void)((p->sectionchannels) | 0);  /* check that 'ncmenu_options.sectionchannels' is an integer */
  (void)((p->flags) | 0);  /* check that 'ncmenu_options.flags' is an integer */
}
struct _cffi_align__ncmenu_options { char x; ncmenu_options y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_struct_ncmenu_section(struct ncmenu_section *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char * *tmp = &p->name; (void)tmp; }
  (void)((p->itemcount) | 0);  /* check that 'struct ncmenu_section.itemcount' is an integer */
  { struct ncmenu_item * *tmp = &p->items; (void)tmp; }
  { ncinput *tmp = &p->shortcut; (void)tmp; }
}
struct _cffi_align_struct_ncmenu_section { char x; struct ncmenu_section y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_struct_ncmselector_item(struct ncmselector_item *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char * *tmp = &p->option; (void)tmp; }
  { char * *tmp = &p->desc; (void)tmp; }
  (void)((p->selected) | 0);  /* check that 'struct ncmselector_item.selected' is an integer */
}
struct _cffi_align_struct_ncmselector_item { char x; struct ncmselector_item y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__ncmultiselector_options(ncmultiselector_options *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char * *tmp = &p->title; (void)tmp; }
  { char * *tmp = &p->secondary; (void)tmp; }
  { char * *tmp = &p->footer; (void)tmp; }
  { struct ncmselector_item * *tmp = &p->items; (void)tmp; }
  (void)((p->maxdisplay) | 0);  /* check that 'ncmultiselector_options.maxdisplay' is an integer */
  (void)((p->opchannels) | 0);  /* check that 'ncmultiselector_options.opchannels' is an integer */
  (void)((p->descchannels) | 0);  /* check that 'ncmultiselector_options.descchannels' is an integer */
  (void)((p->titlechannels) | 0);  /* check that 'ncmultiselector_options.titlechannels' is an integer */
  (void)((p->footchannels) | 0);  /* check that 'ncmultiselector_options.footchannels' is an integer */
  (void)((p->boxchannels) | 0);  /* check that 'ncmultiselector_options.boxchannels' is an integer */
  (void)((p->flags) | 0);  /* check that 'ncmultiselector_options.flags' is an integer */
}
struct _cffi_align__ncmultiselector_options { char x; ncmultiselector_options y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__ncplane_options(ncplane_options *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->y) | 0);  /* check that 'ncplane_options.y' is an integer */
  /* cannot generate 'union $1' in field 'horiz': unknown type name */
  (void)((p->rows) | 0);  /* check that 'ncplane_options.rows' is an integer */
  (void)((p->cols) | 0);  /* check that 'ncplane_options.cols' is an integer */
  { void * *tmp = &p->userptr; (void)tmp; }
  { char const * *tmp = &p->name; (void)tmp; }
  { int(* *tmp)(struct ncplane *) = &p->resizecb; (void)tmp; }
  (void)((p->flags) | 0);  /* check that 'ncplane_options.flags' is an integer */
}
struct _cffi_align__ncplane_options { char x; ncplane_options y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__ncplot_options(ncplot_options *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->maxchannels) | 0);  /* check that 'ncplot_options.maxchannels' is an integer */
  (void)((p->minchannels) | 0);  /* check that 'ncplot_options.minchannels' is an integer */
  (void)((p->legendstyle) | 0);  /* check that 'ncplot_options.legendstyle' is an integer */
  { ncblitter_e *tmp = &p->gridtype; (void)tmp; }
  (void)((p->rangex) | 0);  /* check that 'ncplot_options.rangex' is an integer */
  (void)((p->flags) | 0);  /* check that 'ncplot_options.flags' is an integer */
  { char const * *tmp = &p->title; (void)tmp; }
  (void)((p->flags) | 0);  /* check that 'ncplot_options.flags' is an integer */
}
struct _cffi_align__ncplot_options { char x; ncplot_options y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__ncreader_options(ncreader_options *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->tchannels) | 0);  /* check that 'ncreader_options.tchannels' is an integer */
  (void)((p->tattrword) | 0);  /* check that 'ncreader_options.tattrword' is an integer */
  (void)((p->flags) | 0);  /* check that 'ncreader_options.flags' is an integer */
}
struct _cffi_align__ncreader_options { char x; ncreader_options y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__ncreel_options(ncreel_options *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->bordermask) | 0);  /* check that 'ncreel_options.bordermask' is an integer */
  (void)((p->borderchan) | 0);  /* check that 'ncreel_options.borderchan' is an integer */
  (void)((p->tabletmask) | 0);  /* check that 'ncreel_options.tabletmask' is an integer */
  (void)((p->tabletchan) | 0);  /* check that 'ncreel_options.tabletchan' is an integer */
  (void)((p->focusedchan) | 0);  /* check that 'ncreel_options.focusedchan' is an integer */
  (void)((p->flags) | 0);  /* check that 'ncreel_options.flags' is an integer */
}
struct _cffi_align__ncreel_options { char x; ncreel_options y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_struct_ncselector_item(struct ncselector_item *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char * *tmp = &p->option; (void)tmp; }
  { char * *tmp = &p->desc; (void)tmp; }
  (void)((p->opcolumns) | 0);  /* check that 'struct ncselector_item.opcolumns' is an integer */
  (void)((p->desccolumns) | 0);  /* check that 'struct ncselector_item.desccolumns' is an integer */
}
struct _cffi_align_struct_ncselector_item { char x; struct ncselector_item y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__ncselector_options(ncselector_options *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char * *tmp = &p->title; (void)tmp; }
  { char * *tmp = &p->secondary; (void)tmp; }
  { char * *tmp = &p->footer; (void)tmp; }
  { struct ncselector_item * *tmp = &p->items; (void)tmp; }
  (void)((p->defidx) | 0);  /* check that 'ncselector_options.defidx' is an integer */
  (void)((p->maxdisplay) | 0);  /* check that 'ncselector_options.maxdisplay' is an integer */
  (void)((p->opchannels) | 0);  /* check that 'ncselector_options.opchannels' is an integer */
  (void)((p->descchannels) | 0);  /* check that 'ncselector_options.descchannels' is an integer */
  (void)((p->titlechannels) | 0);  /* check that 'ncselector_options.titlechannels' is an integer */
  (void)((p->footchannels) | 0);  /* check that 'ncselector_options.footchannels' is an integer */
  (void)((p->boxchannels) | 0);  /* check that 'ncselector_options.boxchannels' is an integer */
  (void)((p->flags) | 0);  /* check that 'ncselector_options.flags' is an integer */
}
struct _cffi_align__ncselector_options { char x; ncselector_options y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__ncstats(ncstats *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->renders) | 0);  /* check that 'ncstats.renders' is an integer */
  (void)((p->failed_renders) | 0);  /* check that 'ncstats.failed_renders' is an integer */
  (void)((p->render_bytes) | 0);  /* check that 'ncstats.render_bytes' is an integer */
  (void)((p->render_max_bytes) | 0);  /* check that 'ncstats.render_max_bytes' is an integer */
  (void)((p->render_min_bytes) | 0);  /* check that 'ncstats.render_min_bytes' is an integer */
  (void)((p->render_ns) | 0);  /* check that 'ncstats.render_ns' is an integer */
  (void)((p->render_max_ns) | 0);  /* check that 'ncstats.render_max_ns' is an integer */
  (void)((p->render_min_ns) | 0);  /* check that 'ncstats.render_min_ns' is an integer */
  (void)((p->cellelisions) | 0);  /* check that 'ncstats.cellelisions' is an integer */
  (void)((p->cellemissions) | 0);  /* check that 'ncstats.cellemissions' is an integer */
  (void)((p->fbbytes) | 0);  /* check that 'ncstats.fbbytes' is an integer */
  (void)((p->fgelisions) | 0);  /* check that 'ncstats.fgelisions' is an integer */
  (void)((p->fgemissions) | 0);  /* check that 'ncstats.fgemissions' is an integer */
  (void)((p->bgelisions) | 0);  /* check that 'ncstats.bgelisions' is an integer */
  (void)((p->bgemissions) | 0);  /* check that 'ncstats.bgemissions' is an integer */
  (void)((p->defaultelisions) | 0);  /* check that 'ncstats.defaultelisions' is an integer */
  (void)((p->defaultemissions) | 0);  /* check that 'ncstats.defaultemissions' is an integer */
}
struct _cffi_align__ncstats { char x; ncstats y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__ncsubproc_options(ncsubproc_options *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { void * *tmp = &p->curry; (void)tmp; }
  (void)((p->restart_period) | 0);  /* check that 'ncsubproc_options.restart_period' is an integer */
  (void)((p->flags) | 0);  /* check that 'ncsubproc_options.flags' is an integer */
}
struct _cffi_align__ncsubproc_options { char x; ncsubproc_options y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_struct_ncvisual_options(struct ncvisual_options *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { struct ncplane * *tmp = &p->n; (void)tmp; }
  { ncscale_e *tmp = &p->scaling; (void)tmp; }
  (void)((p->y) | 0);  /* check that 'struct ncvisual_options.y' is an integer */
  (void)((p->x) | 0);  /* check that 'struct ncvisual_options.x' is an integer */
  (void)((p->begy) | 0);  /* check that 'struct ncvisual_options.begy' is an integer */
  (void)((p->begx) | 0);  /* check that 'struct ncvisual_options.begx' is an integer */
  (void)((p->leny) | 0);  /* check that 'struct ncvisual_options.leny' is an integer */
  (void)((p->lenx) | 0);  /* check that 'struct ncvisual_options.lenx' is an integer */
  { ncblitter_e *tmp = &p->blitter; (void)tmp; }
  (void)((p->flags) | 0);  /* check that 'struct ncvisual_options.flags' is an integer */
}
struct _cffi_align_struct_ncvisual_options { char x; struct ncvisual_options y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__notcurses_options(notcurses_options *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char const * *tmp = &p->termtype; (void)tmp; }
  { FILE * *tmp = &p->renderfp; (void)tmp; }
  { ncloglevel_e *tmp = &p->loglevel; (void)tmp; }
  (void)((p->margin_t) | 0);  /* check that 'notcurses_options.margin_t' is an integer */
  (void)((p->margin_r) | 0);  /* check that 'notcurses_options.margin_r' is an integer */
  (void)((p->margin_b) | 0);  /* check that 'notcurses_options.margin_b' is an integer */
  (void)((p->margin_l) | 0);  /* check that 'notcurses_options.margin_l' is an integer */
  (void)((p->flags) | 0);  /* check that 'notcurses_options.flags' is an integer */
}
struct _cffi_align__notcurses_options { char x; notcurses_options y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__palette256(palette256 *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { uint32_t(*tmp)[256] = &p->chans; (void)tmp; }
}
struct _cffi_align__palette256 { char x; palette256 y; };

static const struct _cffi_global_s _cffi_globals[] = {
  { "NCALIGN_CENTER", (void *)_cffi_const_NCALIGN_CENTER, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCALIGN_LEFT", (void *)_cffi_const_NCALIGN_LEFT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCALIGN_RIGHT", (void *)_cffi_const_NCALIGN_RIGHT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCBLIT_1x1", (void *)_cffi_const_NCBLIT_1x1, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCBLIT_1x1x4", (void *)_cffi_const_NCBLIT_1x1x4, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCBLIT_2x1", (void *)_cffi_const_NCBLIT_2x1, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCBLIT_2x2", (void *)_cffi_const_NCBLIT_2x2, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCBLIT_4x1", (void *)_cffi_const_NCBLIT_4x1, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCBLIT_8x1", (void *)_cffi_const_NCBLIT_8x1, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCBLIT_BRAILLE", (void *)_cffi_const_NCBLIT_BRAILLE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCBLIT_SIXEL", (void *)_cffi_const_NCBLIT_SIXEL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCLOGLEVEL_DEBUG", (void *)_cffi_const_NCLOGLEVEL_DEBUG, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCLOGLEVEL_ERROR", (void *)_cffi_const_NCLOGLEVEL_ERROR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCLOGLEVEL_FATAL", (void *)_cffi_const_NCLOGLEVEL_FATAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCLOGLEVEL_INFO", (void *)_cffi_const_NCLOGLEVEL_INFO, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCLOGLEVEL_PANIC", (void *)_cffi_const_NCLOGLEVEL_PANIC, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCLOGLEVEL_SILENT", (void *)_cffi_const_NCLOGLEVEL_SILENT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCLOGLEVEL_TRACE", (void *)_cffi_const_NCLOGLEVEL_TRACE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCLOGLEVEL_VERBOSE", (void *)_cffi_const_NCLOGLEVEL_VERBOSE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCLOGLEVEL_WARNING", (void *)_cffi_const_NCLOGLEVEL_WARNING, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCSCALE_NONE", (void *)_cffi_const_NCSCALE_NONE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCSCALE_SCALE", (void *)_cffi_const_NCSCALE_SCALE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NCSCALE_STRETCH", (void *)_cffi_const_NCSCALE_STRETCH, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "cell_duplicate", (void *)_cffi_f_cell_duplicate, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 214), (void *)_cffi_d_cell_duplicate },
  { "cell_extended_gcluster", (void *)_cffi_f_cell_extended_gcluster, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 73), (void *)_cffi_d_cell_extended_gcluster },
  { "cell_load", (void *)_cffi_f_cell_load, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 219), (void *)_cffi_d_cell_load },
  { "cell_release", (void *)_cffi_f_cell_release, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 748), (void *)_cffi_d_cell_release },
  { "ncblit_bgrx", (void *)_cffi_f_ncblit_bgrx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 537), (void *)_cffi_d_ncblit_bgrx },
  { "ncblit_rgba", (void *)_cffi_f_ncblit_rgba, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 537), (void *)_cffi_d_ncblit_rgba },
  { "ncdirect_bg_rgb", (void *)_cffi_f_ncdirect_bg_rgb, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 158), (void *)_cffi_d_ncdirect_bg_rgb },
  { "ncdirect_bg_rgb8", (void *)_cffi_f_ncdirect_bg_rgb8, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 162), (void *)_cffi_d_ncdirect_bg_rgb8 },
  { "ncdirect_box", (void *)_cffi_f_ncdirect_box, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 147), (void *)_cffi_d_ncdirect_box },
  { "ncdirect_canopen_images", (void *)_cffi_f_ncdirect_canopen_images, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_ncdirect_canopen_images },
  { "ncdirect_canutf8", (void *)_cffi_f_ncdirect_canutf8, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_ncdirect_canutf8 },
  { "ncdirect_clear", (void *)_cffi_f_ncdirect_clear, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 106), (void *)_cffi_d_ncdirect_clear },
  { "ncdirect_cursor_disable", (void *)_cffi_f_ncdirect_cursor_disable, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 106), (void *)_cffi_d_ncdirect_cursor_disable },
  { "ncdirect_cursor_down", (void *)_cffi_f_ncdirect_cursor_down, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 123), (void *)_cffi_d_ncdirect_cursor_down },
  { "ncdirect_cursor_enable", (void *)_cffi_f_ncdirect_cursor_enable, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 106), (void *)_cffi_d_ncdirect_cursor_enable },
  { "ncdirect_cursor_left", (void *)_cffi_f_ncdirect_cursor_left, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 123), (void *)_cffi_d_ncdirect_cursor_left },
  { "ncdirect_cursor_move_yx", (void *)_cffi_f_ncdirect_cursor_move_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 127), (void *)_cffi_d_ncdirect_cursor_move_yx },
  { "ncdirect_cursor_right", (void *)_cffi_f_ncdirect_cursor_right, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 123), (void *)_cffi_d_ncdirect_cursor_right },
  { "ncdirect_cursor_up", (void *)_cffi_f_ncdirect_cursor_up, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 123), (void *)_cffi_d_ncdirect_cursor_up },
  { "ncdirect_dim_x", (void *)_cffi_f_ncdirect_dim_x, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 168), (void *)_cffi_d_ncdirect_dim_x },
  { "ncdirect_dim_y", (void *)_cffi_f_ncdirect_dim_y, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 168), (void *)_cffi_d_ncdirect_dim_y },
  { "ncdirect_double_box", (void *)_cffi_f_ncdirect_double_box, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 137), (void *)_cffi_d_ncdirect_double_box },
  { "ncdirect_fg_rgb", (void *)_cffi_f_ncdirect_fg_rgb, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 158), (void *)_cffi_d_ncdirect_fg_rgb },
  { "ncdirect_fg_rgb8", (void *)_cffi_f_ncdirect_fg_rgb8, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 162), (void *)_cffi_d_ncdirect_fg_rgb8 },
  { "ncdirect_flush", (void *)_cffi_f_ncdirect_flush, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 168), (void *)_cffi_d_ncdirect_flush },
  { "ncdirect_hline_interp", (void *)_cffi_f_ncdirect_hline_interp, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 109), (void *)_cffi_d_ncdirect_hline_interp },
  { "ncdirect_init", (void *)_cffi_f_ncdirect_init, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 545), (void *)_cffi_d_ncdirect_init },
  { "ncdirect_palette_size", (void *)_cffi_f_ncdirect_palette_size, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 711), (void *)_cffi_d_ncdirect_palette_size },
  { "ncdirect_putstr", (void *)_cffi_f_ncdirect_putstr, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 132), (void *)_cffi_d_ncdirect_putstr },
  { "ncdirect_render_image", (void *)_cffi_f_ncdirect_render_image, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 116), (void *)_cffi_d_ncdirect_render_image },
  { "ncdirect_rounded_box", (void *)_cffi_f_ncdirect_rounded_box, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 137), (void *)_cffi_d_ncdirect_rounded_box },
  { "ncdirect_stop", (void *)_cffi_f_ncdirect_stop, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 106), (void *)_cffi_d_ncdirect_stop },
  { "ncdirect_styles_off", (void *)_cffi_f_ncdirect_styles_off, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 158), (void *)_cffi_d_ncdirect_styles_off },
  { "ncdirect_styles_on", (void *)_cffi_f_ncdirect_styles_on, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 158), (void *)_cffi_d_ncdirect_styles_on },
  { "ncdirect_styles_set", (void *)_cffi_f_ncdirect_styles_set, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 158), (void *)_cffi_d_ncdirect_styles_set },
  { "ncdirect_vline_interp", (void *)_cffi_f_ncdirect_vline_interp, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 109), (void *)_cffi_d_ncdirect_vline_interp },
  { "ncdplot_add_sample", (void *)_cffi_f_ncdplot_add_sample, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 171), (void *)_cffi_d_ncdplot_add_sample },
  { "ncdplot_create", (void *)_cffi_f_ncdplot_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 550), (void *)_cffi_d_ncdplot_create },
  { "ncdplot_destroy", (void *)_cffi_f_ncdplot_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 739), (void *)_cffi_d_ncdplot_destroy },
  { "ncdplot_plane", (void *)_cffi_f_ncdplot_plane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 571), (void *)_cffi_d_ncdplot_plane },
  { "ncdplot_sample", (void *)_cffi_f_ncdplot_sample, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 176), (void *)_cffi_d_ncdplot_sample },
  { "ncdplot_set_sample", (void *)_cffi_f_ncdplot_set_sample, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 171), (void *)_cffi_d_ncdplot_set_sample },
  { "ncfdplane_create", (void *)_cffi_f_ncfdplane_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 556), (void *)_cffi_d_ncfdplane_create },
  { "ncfdplane_destroy", (void *)_cffi_f_ncfdplane_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 181), (void *)_cffi_d_ncfdplane_destroy },
  { "ncfdplane_plane", (void *)_cffi_f_ncfdplane_plane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 574), (void *)_cffi_d_ncfdplane_plane },
  { "ncmenu_create", (void *)_cffi_f_ncmenu_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 563), (void *)_cffi_d_ncmenu_create },
  { "ncmenu_destroy", (void *)_cffi_f_ncmenu_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 195), (void *)_cffi_d_ncmenu_destroy },
  { "ncmenu_nextitem", (void *)_cffi_f_ncmenu_nextitem, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 195), (void *)_cffi_d_ncmenu_nextitem },
  { "ncmenu_nextsection", (void *)_cffi_f_ncmenu_nextsection, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 195), (void *)_cffi_d_ncmenu_nextsection },
  { "ncmenu_offer_input", (void *)_cffi_f_ncmenu_offer_input, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 3), (void *)_cffi_d_ncmenu_offer_input },
  { "ncmenu_previtem", (void *)_cffi_f_ncmenu_previtem, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 195), (void *)_cffi_d_ncmenu_previtem },
  { "ncmenu_prevsection", (void *)_cffi_f_ncmenu_prevsection, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 195), (void *)_cffi_d_ncmenu_prevsection },
  { "ncmenu_rollup", (void *)_cffi_f_ncmenu_rollup, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 195), (void *)_cffi_d_ncmenu_rollup },
  { "ncmenu_selected", (void *)_cffi_f_ncmenu_selected, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 69), (void *)_cffi_d_ncmenu_selected },
  { "ncmenu_unroll", (void *)_cffi_f_ncmenu_unroll, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 198), (void *)_cffi_d_ncmenu_unroll },
  { "ncmetric", (void *)_cffi_f_ncmetric, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 83), (void *)_cffi_d_ncmetric },
  { "ncmultiselector_create", (void *)_cffi_f_ncmultiselector_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 567), (void *)_cffi_d_ncmultiselector_create },
  { "ncmultiselector_destroy", (void *)_cffi_f_ncmultiselector_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 742), (void *)_cffi_d_ncmultiselector_destroy },
  { "ncmultiselector_offer_input", (void *)_cffi_f_ncmultiselector_offer_input, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 7), (void *)_cffi_d_ncmultiselector_offer_input },
  { "ncmultiselector_plane", (void *)_cffi_f_ncmultiselector_plane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 577), (void *)_cffi_d_ncmultiselector_plane },
  { "ncmultiselector_selected", (void *)_cffi_f_ncmultiselector_selected, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 202), (void *)_cffi_d_ncmultiselector_selected },
  { "ncplane_above", (void *)_cffi_f_ncplane_above, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 580), (void *)_cffi_d_ncplane_above },
  { "ncplane_at_cursor", (void *)_cffi_f_ncplane_at_cursor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 31), (void *)_cffi_d_ncplane_at_cursor },
  { "ncplane_at_yx", (void *)_cffi_f_ncplane_at_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 43), (void *)_cffi_d_ncplane_at_yx },
  { "ncplane_attr", (void *)_cffi_f_ncplane_attr, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 693), (void *)_cffi_d_ncplane_attr },
  { "ncplane_base", (void *)_cffi_f_ncplane_base, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 210), (void *)_cffi_d_ncplane_base },
  { "ncplane_below", (void *)_cffi_f_ncplane_below, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 580), (void *)_cffi_d_ncplane_below },
  { "ncplane_box", (void *)_cffi_f_ncplane_box, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 228), (void *)_cffi_d_ncplane_box },
  { "ncplane_channels", (void *)_cffi_f_ncplane_channels, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 708), (void *)_cffi_d_ncplane_channels },
  { "ncplane_contents", (void *)_cffi_f_ncplane_contents, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 36), (void *)_cffi_d_ncplane_contents },
  { "ncplane_create", (void *)_cffi_f_ncplane_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 583), (void *)_cffi_d_ncplane_create },
  { "ncplane_cursor_move_yx", (void *)_cffi_f_ncplane_cursor_move_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 281), (void *)_cffi_d_ncplane_cursor_move_yx },
  { "ncplane_cursor_yx", (void *)_cffi_f_ncplane_cursor_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 752), (void *)_cffi_d_ncplane_cursor_yx },
  { "ncplane_destroy", (void *)_cffi_f_ncplane_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 207), (void *)_cffi_d_ncplane_destroy },
  { "ncplane_dim_yx", (void *)_cffi_f_ncplane_dim_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 771), (void *)_cffi_d_ncplane_dim_yx },
  { "ncplane_dup", (void *)_cffi_f_ncplane_dup, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 591), (void *)_cffi_d_ncplane_dup },
  { "ncplane_erase", (void *)_cffi_f_ncplane_erase, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 745), (void *)_cffi_d_ncplane_erase },
  { "ncplane_fadein", (void *)_cffi_f_ncplane_fadein, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 370), (void *)_cffi_d_ncplane_fadein },
  { "ncplane_fadeout", (void *)_cffi_f_ncplane_fadeout, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 370), (void *)_cffi_d_ncplane_fadeout },
  { "ncplane_format", (void *)_cffi_f_ncplane_format, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 323), (void *)_cffi_d_ncplane_format },
  { "ncplane_gradient", (void *)_cffi_f_ncplane_gradient, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 262), (void *)_cffi_d_ncplane_gradient },
  { "ncplane_highgradient", (void *)_cffi_f_ncplane_highgradient, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 376), (void *)_cffi_d_ncplane_highgradient },
  { "ncplane_highgradient_sized", (void *)_cffi_f_ncplane_highgradient_sized, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 376), (void *)_cffi_d_ncplane_highgradient_sized },
  { "ncplane_hline_interp", (void *)_cffi_f_ncplane_hline_interp, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 240), (void *)_cffi_d_ncplane_hline_interp },
  { "ncplane_mergedown", (void *)_cffi_f_ncplane_mergedown, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 405), (void *)_cffi_d_ncplane_mergedown },
  { "ncplane_mergedown_simple", (void *)_cffi_f_ncplane_mergedown_simple, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 401), (void *)_cffi_d_ncplane_mergedown_simple },
  { "ncplane_move_above", (void *)_cffi_f_ncplane_move_above, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 366), (void *)_cffi_d_ncplane_move_above },
  { "ncplane_move_below", (void *)_cffi_f_ncplane_move_below, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 366), (void *)_cffi_d_ncplane_move_below },
  { "ncplane_move_bottom", (void *)_cffi_f_ncplane_move_bottom, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 745), (void *)_cffi_d_ncplane_move_bottom },
  { "ncplane_move_top", (void *)_cffi_f_ncplane_move_top, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 745), (void *)_cffi_d_ncplane_move_top },
  { "ncplane_move_yx", (void *)_cffi_f_ncplane_move_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 281), (void *)_cffi_d_ncplane_move_yx },
  { "ncplane_parent", (void *)_cffi_f_ncplane_parent, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 580), (void *)_cffi_d_ncplane_parent },
  { "ncplane_parent_const", (void *)_cffi_f_ncplane_parent_const, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 621), (void *)_cffi_d_ncplane_parent_const },
  { "ncplane_polyfill_yx", (void *)_cffi_f_ncplane_polyfill_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 286), (void *)_cffi_d_ncplane_polyfill_yx },
  { "ncplane_pulse", (void *)_cffi_f_ncplane_pulse, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 370), (void *)_cffi_d_ncplane_pulse },
  { "ncplane_putc_yx", (void *)_cffi_f_ncplane_putc_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 286), (void *)_cffi_d_ncplane_putc_yx },
  { "ncplane_putchar_stained", (void *)_cffi_f_ncplane_putchar_stained, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 273), (void *)_cffi_d_ncplane_putchar_stained },
  { "ncplane_putegc_stained", (void *)_cffi_f_ncplane_putegc_stained, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 251), (void *)_cffi_d_ncplane_putegc_stained },
  { "ncplane_putegc_yx", (void *)_cffi_f_ncplane_putegc_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 292), (void *)_cffi_d_ncplane_putegc_yx },
  { "ncplane_putnstr_aligned", (void *)_cffi_f_ncplane_putnstr_aligned, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 351), (void *)_cffi_d_ncplane_putnstr_aligned },
  { "ncplane_putnstr_yx", (void *)_cffi_f_ncplane_putnstr_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 316), (void *)_cffi_d_ncplane_putnstr_yx },
  { "ncplane_putstr_aligned", (void *)_cffi_f_ncplane_putstr_aligned, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 338), (void *)_cffi_d_ncplane_putstr_aligned },
  { "ncplane_putstr_stained", (void *)_cffi_f_ncplane_putstr_stained, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 247), (void *)_cffi_d_ncplane_putstr_stained },
  { "ncplane_puttext", (void *)_cffi_f_ncplane_puttext, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 344), (void *)_cffi_d_ncplane_puttext },
  { "ncplane_putwegc_stained", (void *)_cffi_f_ncplane_putwegc_stained, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 393), (void *)_cffi_d_ncplane_putwegc_stained },
  { "ncplane_putwstr_stained", (void *)_cffi_f_ncplane_putwstr_stained, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 389), (void *)_cffi_d_ncplane_putwstr_stained },
  { "ncplane_qrcode", (void *)_cffi_f_ncplane_qrcode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 358), (void *)_cffi_d_ncplane_qrcode },
  { "ncplane_reparent", (void *)_cffi_f_ncplane_reparent, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 587), (void *)_cffi_d_ncplane_reparent },
  { "ncplane_resize", (void *)_cffi_f_ncplane_resize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 305), (void *)_cffi_d_ncplane_resize },
  { "ncplane_rgba", (void *)_cffi_f_ncplane_rgba, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 696), (void *)_cffi_d_ncplane_rgba },
  { "ncplane_rotate_ccw", (void *)_cffi_f_ncplane_rotate_ccw, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 207), (void *)_cffi_d_ncplane_rotate_ccw },
  { "ncplane_rotate_cw", (void *)_cffi_f_ncplane_rotate_cw, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 207), (void *)_cffi_d_ncplane_rotate_cw },
  { "ncplane_set_attr", (void *)_cffi_f_ncplane_set_attr, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 767), (void *)_cffi_d_ncplane_set_attr },
  { "ncplane_set_base", (void *)_cffi_f_ncplane_set_base, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 256), (void *)_cffi_d_ncplane_set_base },
  { "ncplane_set_base_cell", (void *)_cffi_f_ncplane_set_base_cell, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 224), (void *)_cffi_d_ncplane_set_base_cell },
  { "ncplane_set_bchannel", (void *)_cffi_f_ncplane_set_bchannel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 704), (void *)_cffi_d_ncplane_set_bchannel },
  { "ncplane_set_bg_alpha", (void *)_cffi_f_ncplane_set_bg_alpha, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 385), (void *)_cffi_d_ncplane_set_bg_alpha },
  { "ncplane_set_bg_default", (void *)_cffi_f_ncplane_set_bg_default, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 745), (void *)_cffi_d_ncplane_set_bg_default },
  { "ncplane_set_bg_palindex", (void *)_cffi_f_ncplane_set_bg_palindex, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 277), (void *)_cffi_d_ncplane_set_bg_palindex },
  { "ncplane_set_bg_rgb", (void *)_cffi_f_ncplane_set_bg_rgb, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 385), (void *)_cffi_d_ncplane_set_bg_rgb },
  { "ncplane_set_bg_rgb8", (void *)_cffi_f_ncplane_set_bg_rgb8, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 299), (void *)_cffi_d_ncplane_set_bg_rgb8 },
  { "ncplane_set_bg_rgb8_clipped", (void *)_cffi_f_ncplane_set_bg_rgb8_clipped, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 757), (void *)_cffi_d_ncplane_set_bg_rgb8_clipped },
  { "ncplane_set_channels", (void *)_cffi_f_ncplane_set_channels, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 763), (void *)_cffi_d_ncplane_set_channels },
  { "ncplane_set_fchannel", (void *)_cffi_f_ncplane_set_fchannel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 704), (void *)_cffi_d_ncplane_set_fchannel },
  { "ncplane_set_fg_alpha", (void *)_cffi_f_ncplane_set_fg_alpha, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 385), (void *)_cffi_d_ncplane_set_fg_alpha },
  { "ncplane_set_fg_default", (void *)_cffi_f_ncplane_set_fg_default, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 745), (void *)_cffi_d_ncplane_set_fg_default },
  { "ncplane_set_fg_palindex", (void *)_cffi_f_ncplane_set_fg_palindex, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 277), (void *)_cffi_d_ncplane_set_fg_palindex },
  { "ncplane_set_fg_rgb", (void *)_cffi_f_ncplane_set_fg_rgb, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 385), (void *)_cffi_d_ncplane_set_fg_rgb },
  { "ncplane_set_fg_rgb8", (void *)_cffi_f_ncplane_set_fg_rgb8, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 299), (void *)_cffi_d_ncplane_set_fg_rgb8 },
  { "ncplane_set_fg_rgb8_clipped", (void *)_cffi_f_ncplane_set_fg_rgb8_clipped, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 757), (void *)_cffi_d_ncplane_set_fg_rgb8_clipped },
  { "ncplane_set_scrolling", (void *)_cffi_f_ncplane_set_scrolling, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 11), (void *)_cffi_d_ncplane_set_scrolling },
  { "ncplane_set_userptr", (void *)_cffi_f_ncplane_set_userptr, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 723), (void *)_cffi_d_ncplane_set_userptr },
  { "ncplane_stain", (void *)_cffi_f_ncplane_stain, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 329), (void *)_cffi_d_ncplane_stain },
  { "ncplane_styles", (void *)_cffi_f_ncplane_styles, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 714), (void *)_cffi_d_ncplane_styles },
  { "ncplane_styles_off", (void *)_cffi_f_ncplane_styles_off, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 767), (void *)_cffi_d_ncplane_styles_off },
  { "ncplane_styles_on", (void *)_cffi_f_ncplane_styles_on, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 767), (void *)_cffi_d_ncplane_styles_on },
  { "ncplane_styles_set", (void *)_cffi_f_ncplane_styles_set, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 767), (void *)_cffi_d_ncplane_styles_set },
  { "ncplane_translate", (void *)_cffi_f_ncplane_translate, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 776), (void *)_cffi_d_ncplane_translate },
  { "ncplane_translate_abs", (void *)_cffi_f_ncplane_translate_abs, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 15), (void *)_cffi_d_ncplane_translate_abs },
  { "ncplane_userptr", (void *)_cffi_f_ncplane_userptr, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 720), (void *)_cffi_d_ncplane_userptr },
  { "ncplane_vline_interp", (void *)_cffi_f_ncplane_vline_interp, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 240), (void *)_cffi_d_ncplane_vline_interp },
  { "ncplane_x", (void *)_cffi_f_ncplane_x, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 398), (void *)_cffi_d_ncplane_x },
  { "ncplane_y", (void *)_cffi_f_ncplane_y, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 398), (void *)_cffi_d_ncplane_y },
  { "ncplane_yx", (void *)_cffi_f_ncplane_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 752), (void *)_cffi_d_ncplane_yx },
  { "ncreader_clear", (void *)_cffi_f_ncreader_clear, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 415), (void *)_cffi_d_ncreader_clear },
  { "ncreader_contents", (void *)_cffi_f_ncreader_contents, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 50), (void *)_cffi_d_ncreader_contents },
  { "ncreader_create", (void *)_cffi_f_ncreader_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 627), (void *)_cffi_d_ncreader_create },
  { "ncreader_destroy", (void *)_cffi_f_ncreader_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 782), (void *)_cffi_d_ncreader_destroy },
  { "ncreader_move_down", (void *)_cffi_f_ncreader_move_down, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 415), (void *)_cffi_d_ncreader_move_down },
  { "ncreader_move_left", (void *)_cffi_f_ncreader_move_left, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 415), (void *)_cffi_d_ncreader_move_left },
  { "ncreader_move_right", (void *)_cffi_f_ncreader_move_right, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 415), (void *)_cffi_d_ncreader_move_right },
  { "ncreader_move_up", (void *)_cffi_f_ncreader_move_up, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 415), (void *)_cffi_d_ncreader_move_up },
  { "ncreader_offer_input", (void *)_cffi_f_ncreader_offer_input, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 20), (void *)_cffi_d_ncreader_offer_input },
  { "ncreader_plane", (void *)_cffi_f_ncreader_plane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 595), (void *)_cffi_d_ncreader_plane },
  { "ncreader_write_egc", (void *)_cffi_f_ncreader_write_egc, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 418), (void *)_cffi_d_ncreader_write_egc },
  { "ncreel_add", (void *)_cffi_f_ncreel_add, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 659), (void *)_cffi_d_ncreel_add },
  { "ncreel_create", (void *)_cffi_f_ncreel_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 631), (void *)_cffi_d_ncreel_create },
  { "ncreel_del", (void *)_cffi_f_ncreel_del, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 425), (void *)_cffi_d_ncreel_del },
  { "ncreel_destroy", (void *)_cffi_f_ncreel_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 422), (void *)_cffi_d_ncreel_destroy },
  { "ncreel_focused", (void *)_cffi_f_ncreel_focused, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 656), (void *)_cffi_d_ncreel_focused },
  { "ncreel_next", (void *)_cffi_f_ncreel_next, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 656), (void *)_cffi_d_ncreel_next },
  { "ncreel_plane", (void *)_cffi_f_ncreel_plane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 598), (void *)_cffi_d_ncreel_plane },
  { "ncreel_prev", (void *)_cffi_f_ncreel_prev, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 656), (void *)_cffi_d_ncreel_prev },
  { "ncreel_redraw", (void *)_cffi_f_ncreel_redraw, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 422), (void *)_cffi_d_ncreel_redraw },
  { "ncreel_tabletcount", (void *)_cffi_f_ncreel_tabletcount, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 429), (void *)_cffi_d_ncreel_tabletcount },
  { "ncselector_additem", (void *)_cffi_f_ncselector_additem, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 436), (void *)_cffi_d_ncselector_additem },
  { "ncselector_create", (void *)_cffi_f_ncselector_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 635), (void *)_cffi_d_ncselector_create },
  { "ncselector_delitem", (void *)_cffi_f_ncselector_delitem, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 432), (void *)_cffi_d_ncselector_delitem },
  { "ncselector_destroy", (void *)_cffi_f_ncselector_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 786), (void *)_cffi_d_ncselector_destroy },
  { "ncselector_nextitem", (void *)_cffi_f_ncselector_nextitem, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 77), (void *)_cffi_d_ncselector_nextitem },
  { "ncselector_offer_input", (void *)_cffi_f_ncselector_offer_input, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 24), (void *)_cffi_d_ncselector_offer_input },
  { "ncselector_plane", (void *)_cffi_f_ncselector_plane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 601), (void *)_cffi_d_ncselector_plane },
  { "ncselector_previtem", (void *)_cffi_f_ncselector_previtem, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 77), (void *)_cffi_d_ncselector_previtem },
  { "ncselector_selected", (void *)_cffi_f_ncselector_selected, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 80), (void *)_cffi_d_ncselector_selected },
  { "ncstrwidth", (void *)_cffi_f_ncstrwidth, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 91), (void *)_cffi_d_ncstrwidth },
  { "ncsubproc_createv", (void *)_cffi_f_ncsubproc_createv, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 648), (void *)_cffi_d_ncsubproc_createv },
  { "ncsubproc_createvp", (void *)_cffi_f_ncsubproc_createvp, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 648), (void *)_cffi_d_ncsubproc_createvp },
  { "ncsubproc_createvpe", (void *)_cffi_f_ncsubproc_createvpe, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 639), (void *)_cffi_d_ncsubproc_createvpe },
  { "ncsubproc_destroy", (void *)_cffi_f_ncsubproc_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 440), (void *)_cffi_d_ncsubproc_destroy },
  { "ncsubproc_plane", (void *)_cffi_f_ncsubproc_plane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 604), (void *)_cffi_d_ncsubproc_plane },
  { "nctablet_ncplane", (void *)_cffi_f_nctablet_ncplane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 607), (void *)_cffi_d_nctablet_ncplane },
  { "nctablet_userptr", (void *)_cffi_f_nctablet_userptr, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 727), (void *)_cffi_d_nctablet_userptr },
  { "ncuplot_add_sample", (void *)_cffi_f_ncuplot_add_sample, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 447), (void *)_cffi_d_ncuplot_add_sample },
  { "ncuplot_create", (void *)_cffi_f_ncuplot_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 666), (void *)_cffi_d_ncuplot_create },
  { "ncuplot_destroy", (void *)_cffi_f_ncuplot_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 790), (void *)_cffi_d_ncuplot_destroy },
  { "ncuplot_plane", (void *)_cffi_f_ncuplot_plane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 610), (void *)_cffi_d_ncuplot_plane },
  { "ncuplot_sample", (void *)_cffi_f_ncuplot_sample, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 452), (void *)_cffi_d_ncuplot_sample },
  { "ncuplot_set_sample", (void *)_cffi_f_ncuplot_set_sample, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 447), (void *)_cffi_d_ncuplot_set_sample },
  { "ncvisual_at_yx", (void *)_cffi_f_ncvisual_at_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 481), (void *)_cffi_d_ncvisual_at_yx },
  { "ncvisual_decode", (void *)_cffi_f_ncvisual_decode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 457), (void *)_cffi_d_ncvisual_decode },
  { "ncvisual_destroy", (void *)_cffi_f_ncvisual_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 793), (void *)_cffi_d_ncvisual_destroy },
  { "ncvisual_from_bgra", (void *)_cffi_f_ncvisual_from_bgra, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 683), (void *)_cffi_d_ncvisual_from_bgra },
  { "ncvisual_from_file", (void *)_cffi_f_ncvisual_from_file, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 672), (void *)_cffi_d_ncvisual_from_file },
  { "ncvisual_from_plane", (void *)_cffi_f_ncvisual_from_plane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 675), (void *)_cffi_d_ncvisual_from_plane },
  { "ncvisual_from_rgba", (void *)_cffi_f_ncvisual_from_rgba, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 683), (void *)_cffi_d_ncvisual_from_rgba },
  { "ncvisual_geom", (void *)_cffi_f_ncvisual_geom, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 528), (void *)_cffi_d_ncvisual_geom },
  { "ncvisual_polyfill_yx", (void *)_cffi_f_ncvisual_polyfill_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 469), (void *)_cffi_d_ncvisual_polyfill_yx },
  { "ncvisual_render", (void *)_cffi_f_ncvisual_render, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 616), (void *)_cffi_d_ncvisual_render },
  { "ncvisual_resize", (void *)_cffi_f_ncvisual_resize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 464), (void *)_cffi_d_ncvisual_resize },
  { "ncvisual_rotate", (void *)_cffi_f_ncvisual_rotate, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 460), (void *)_cffi_d_ncvisual_rotate },
  { "ncvisual_set_yx", (void *)_cffi_f_ncvisual_set_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 487), (void *)_cffi_d_ncvisual_set_yx },
  { "ncvisual_stream", (void *)_cffi_f_ncvisual_stream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 520), (void *)_cffi_d_ncvisual_stream },
  { "ncvisual_subtitle", (void *)_cffi_f_ncvisual_subtitle, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 53), (void *)_cffi_d_ncvisual_subtitle },
  { "notcurses_at_yx", (void *)_cffi_f_notcurses_at_yx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 56), (void *)_cffi_d_notcurses_at_yx },
  { "notcurses_bottom", (void *)_cffi_f_notcurses_bottom, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 613), (void *)_cffi_d_notcurses_bottom },
  { "notcurses_canchangecolor", (void *)_cffi_f_notcurses_canchangecolor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 28), (void *)_cffi_d_notcurses_canchangecolor },
  { "notcurses_canfade", (void *)_cffi_f_notcurses_canfade, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 28), (void *)_cffi_d_notcurses_canfade },
  { "notcurses_canopen_images", (void *)_cffi_f_notcurses_canopen_images, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 28), (void *)_cffi_d_notcurses_canopen_images },
  { "notcurses_canopen_videos", (void *)_cffi_f_notcurses_canopen_videos, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 28), (void *)_cffi_d_notcurses_canopen_videos },
  { "notcurses_cansixel", (void *)_cffi_f_notcurses_cansixel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 28), (void *)_cffi_d_notcurses_cansixel },
  { "notcurses_cantruecolor", (void *)_cffi_f_notcurses_cantruecolor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 28), (void *)_cffi_d_notcurses_cantruecolor },
  { "notcurses_canutf8", (void *)_cffi_f_notcurses_canutf8, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 28), (void *)_cffi_d_notcurses_canutf8 },
  { "notcurses_cursor_disable", (void *)_cffi_f_notcurses_cursor_disable, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 493), (void *)_cffi_d_notcurses_cursor_disable },
  { "notcurses_cursor_enable", (void *)_cffi_f_notcurses_cursor_enable, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 505), (void *)_cffi_d_notcurses_cursor_enable },
  { "notcurses_drop_planes", (void *)_cffi_f_notcurses_drop_planes, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 796), (void *)_cffi_d_notcurses_drop_planes },
  { "notcurses_init", (void *)_cffi_f_notcurses_init, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 689), (void *)_cffi_d_notcurses_init },
  { "notcurses_inputready_fd", (void *)_cffi_f_notcurses_inputready_fd, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 493), (void *)_cffi_d_notcurses_inputready_fd },
  { "notcurses_lex_blitter", (void *)_cffi_f_notcurses_lex_blitter, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 94), (void *)_cffi_d_notcurses_lex_blitter },
  { "notcurses_lex_margins", (void *)_cffi_f_notcurses_lex_margins, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 102), (void *)_cffi_d_notcurses_lex_margins },
  { "notcurses_lex_scalemode", (void *)_cffi_f_notcurses_lex_scalemode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 98), (void *)_cffi_d_notcurses_lex_scalemode },
  { "notcurses_mouse_disable", (void *)_cffi_f_notcurses_mouse_disable, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 493), (void *)_cffi_d_notcurses_mouse_disable },
  { "notcurses_mouse_enable", (void *)_cffi_f_notcurses_mouse_enable, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 493), (void *)_cffi_d_notcurses_mouse_enable },
  { "notcurses_palette_size", (void *)_cffi_f_notcurses_palette_size, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 717), (void *)_cffi_d_notcurses_palette_size },
  { "notcurses_refresh", (void *)_cffi_f_notcurses_refresh, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 500), (void *)_cffi_d_notcurses_refresh },
  { "notcurses_render", (void *)_cffi_f_notcurses_render, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 493), (void *)_cffi_d_notcurses_render },
  { "notcurses_render_to_file", (void *)_cffi_f_notcurses_render_to_file, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 496), (void *)_cffi_d_notcurses_render_to_file },
  { "notcurses_reset_stats", (void *)_cffi_f_notcurses_reset_stats, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 799), (void *)_cffi_d_notcurses_reset_stats },
  { "notcurses_stats", (void *)_cffi_f_notcurses_stats, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 799), (void *)_cffi_d_notcurses_stats },
  { "notcurses_stdplane", (void *)_cffi_f_notcurses_stdplane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 613), (void *)_cffi_d_notcurses_stdplane },
  { "notcurses_stdplane_const", (void *)_cffi_f_notcurses_stdplane_const, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 624), (void *)_cffi_d_notcurses_stdplane_const },
  { "notcurses_stop", (void *)_cffi_f_notcurses_stop, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 493), (void *)_cffi_d_notcurses_stop },
  { "notcurses_str_blitter", (void *)_cffi_f_notcurses_str_blitter, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 63), (void *)_cffi_d_notcurses_str_blitter },
  { "notcurses_str_scalemode", (void *)_cffi_f_notcurses_str_scalemode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 66), (void *)_cffi_d_notcurses_str_scalemode },
  { "notcurses_supported_styles", (void *)_cffi_f_notcurses_supported_styles, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 717), (void *)_cffi_d_notcurses_supported_styles },
  { "notcurses_top", (void *)_cffi_f_notcurses_top, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 613), (void *)_cffi_d_notcurses_top },
  { "notcurses_version_components", (void *)_cffi_f_notcurses_version_components, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 730), (void *)_cffi_d_notcurses_version_components },
  { "palette256_free", (void *)_cffi_f_palette256_free, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 736), (void *)_cffi_d_palette256_free },
  { "palette256_new", (void *)_cffi_f_palette256_new, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 542), (void *)_cffi_d_palette256_new },
  { "palette256_use", (void *)_cffi_f_palette256_use, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 510), (void *)_cffi_d_palette256_use },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "gcluster", offsetof(cell, gcluster),
                sizeof(((cell *)0)->gcluster),
                _CFFI_OP(_CFFI_OP_NOOP, 259) },
  { "gcluster_backstop", offsetof(cell, gcluster_backstop),
                         sizeof(((cell *)0)->gcluster_backstop),
                         _CFFI_OP(_CFFI_OP_NOOP, 849) },
  { "reserved", offsetof(cell, reserved),
                sizeof(((cell *)0)->reserved),
                _CFFI_OP(_CFFI_OP_NOOP, 849) },
  { "stylemask", offsetof(cell, stylemask),
                 sizeof(((cell *)0)->stylemask),
                 _CFFI_OP(_CFFI_OP_NOOP, 846) },
  { "channels", offsetof(cell, channels),
                sizeof(((cell *)0)->channels),
                _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "curry", offsetof(ncfdplane_options, curry),
             sizeof(((ncfdplane_options *)0)->curry),
             _CFFI_OP(_CFFI_OP_NOOP, 187) },
  { "follow", offsetof(ncfdplane_options, follow),
              sizeof(((ncfdplane_options *)0)->follow),
              _CFFI_OP(_CFFI_OP_NOOP, 13) },
  { "flags", offsetof(ncfdplane_options, flags),
             sizeof(((ncfdplane_options *)0)->flags),
             _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "id", offsetof(ncinput, id),
          sizeof(((ncinput *)0)->id),
          _CFFI_OP(_CFFI_OP_NOOP, 805) },
  { "y", offsetof(ncinput, y),
         sizeof(((ncinput *)0)->y),
         _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "x", offsetof(ncinput, x),
         sizeof(((ncinput *)0)->x),
         _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "alt", offsetof(ncinput, alt),
           sizeof(((ncinput *)0)->alt),
           _CFFI_OP(_CFFI_OP_NOOP, 13) },
  { "shift", offsetof(ncinput, shift),
             sizeof(((ncinput *)0)->shift),
             _CFFI_OP(_CFFI_OP_NOOP, 13) },
  { "ctrl", offsetof(ncinput, ctrl),
            sizeof(((ncinput *)0)->ctrl),
            _CFFI_OP(_CFFI_OP_NOOP, 13) },
  { "seqnum", offsetof(ncinput, seqnum),
              sizeof(((ncinput *)0)->seqnum),
              _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "desc", offsetof(struct ncmenu_item, desc),
            sizeof(((struct ncmenu_item *)0)->desc),
            _CFFI_OP(_CFFI_OP_NOOP, 86) },
  { "shortcut", offsetof(struct ncmenu_item, shortcut),
                sizeof(((struct ncmenu_item *)0)->shortcut),
                _CFFI_OP(_CFFI_OP_NOOP, 809) },
  { "sections", offsetof(ncmenu_options, sections),
                sizeof(((ncmenu_options *)0)->sections),
                _CFFI_OP(_CFFI_OP_NOOP, 828) },
  { "sectioncount", offsetof(ncmenu_options, sectioncount),
                    sizeof(((ncmenu_options *)0)->sectioncount),
                    _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "headerchannels", offsetof(ncmenu_options, headerchannels),
                      sizeof(((ncmenu_options *)0)->headerchannels),
                      _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "sectionchannels", offsetof(ncmenu_options, sectionchannels),
                       sizeof(((ncmenu_options *)0)->sectionchannels),
                       _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "flags", offsetof(ncmenu_options, flags),
             sizeof(((ncmenu_options *)0)->flags),
             _CFFI_OP(_CFFI_OP_NOOP, 85) },
  { "name", offsetof(struct ncmenu_section, name),
            sizeof(((struct ncmenu_section *)0)->name),
            _CFFI_OP(_CFFI_OP_NOOP, 86) },
  { "itemcount", offsetof(struct ncmenu_section, itemcount),
                 sizeof(((struct ncmenu_section *)0)->itemcount),
                 _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "items", offsetof(struct ncmenu_section, items),
             sizeof(((struct ncmenu_section *)0)->items),
             _CFFI_OP(_CFFI_OP_NOOP, 826) },
  { "shortcut", offsetof(struct ncmenu_section, shortcut),
                sizeof(((struct ncmenu_section *)0)->shortcut),
                _CFFI_OP(_CFFI_OP_NOOP, 809) },
  { "option", offsetof(struct ncmselector_item, option),
              sizeof(((struct ncmselector_item *)0)->option),
              _CFFI_OP(_CFFI_OP_NOOP, 86) },
  { "desc", offsetof(struct ncmselector_item, desc),
            sizeof(((struct ncmselector_item *)0)->desc),
            _CFFI_OP(_CFFI_OP_NOOP, 86) },
  { "selected", offsetof(struct ncmselector_item, selected),
                sizeof(((struct ncmselector_item *)0)->selected),
                _CFFI_OP(_CFFI_OP_NOOP, 13) },
  { "title", offsetof(ncmultiselector_options, title),
             sizeof(((ncmultiselector_options *)0)->title),
             _CFFI_OP(_CFFI_OP_NOOP, 86) },
  { "secondary", offsetof(ncmultiselector_options, secondary),
                 sizeof(((ncmultiselector_options *)0)->secondary),
                 _CFFI_OP(_CFFI_OP_NOOP, 86) },
  { "footer", offsetof(ncmultiselector_options, footer),
              sizeof(((ncmultiselector_options *)0)->footer),
              _CFFI_OP(_CFFI_OP_NOOP, 86) },
  { "items", offsetof(ncmultiselector_options, items),
             sizeof(((ncmultiselector_options *)0)->items),
             _CFFI_OP(_CFFI_OP_NOOP, 830) },
  { "maxdisplay", offsetof(ncmultiselector_options, maxdisplay),
                  sizeof(((ncmultiselector_options *)0)->maxdisplay),
                  _CFFI_OP(_CFFI_OP_NOOP, 85) },
  { "opchannels", offsetof(ncmultiselector_options, opchannels),
                  sizeof(((ncmultiselector_options *)0)->opchannels),
                  _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "descchannels", offsetof(ncmultiselector_options, descchannels),
                    sizeof(((ncmultiselector_options *)0)->descchannels),
                    _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "titlechannels", offsetof(ncmultiselector_options, titlechannels),
                     sizeof(((ncmultiselector_options *)0)->titlechannels),
                     _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "footchannels", offsetof(ncmultiselector_options, footchannels),
                    sizeof(((ncmultiselector_options *)0)->footchannels),
                    _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "boxchannels", offsetof(ncmultiselector_options, boxchannels),
                   sizeof(((ncmultiselector_options *)0)->boxchannels),
                   _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "flags", offsetof(ncmultiselector_options, flags),
             sizeof(((ncmultiselector_options *)0)->flags),
             _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "y", offsetof(ncplane_options, y),
         sizeof(((ncplane_options *)0)->y),
         _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "horiz", offsetof(ncplane_options, horiz),
             sizeof(((ncplane_options *)0)->horiz),
             _CFFI_OP(_CFFI_OP_NOOP, 850) },
  { "rows", offsetof(ncplane_options, rows),
            sizeof(((ncplane_options *)0)->rows),
            _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "cols", offsetof(ncplane_options, cols),
            sizeof(((ncplane_options *)0)->cols),
            _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "userptr", offsetof(ncplane_options, userptr),
               sizeof(((ncplane_options *)0)->userptr),
               _CFFI_OP(_CFFI_OP_NOOP, 187) },
  { "name", offsetof(ncplane_options, name),
            sizeof(((ncplane_options *)0)->name),
            _CFFI_OP(_CFFI_OP_NOOP, 92) },
  { "resizecb", offsetof(ncplane_options, resizecb),
                sizeof(((ncplane_options *)0)->resizecb),
                _CFFI_OP(_CFFI_OP_NOOP, 806) },
  { "flags", offsetof(ncplane_options, flags),
             sizeof(((ncplane_options *)0)->flags),
             _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "maxchannels", offsetof(ncplot_options, maxchannels),
                   sizeof(((ncplot_options *)0)->maxchannels),
                   _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "minchannels", offsetof(ncplot_options, minchannels),
                   sizeof(((ncplot_options *)0)->minchannels),
                   _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "legendstyle", offsetof(ncplot_options, legendstyle),
                   sizeof(((ncplot_options *)0)->legendstyle),
                   _CFFI_OP(_CFFI_OP_NOOP, 846) },
  { "gridtype", offsetof(ncplot_options, gridtype),
                sizeof(((ncplot_options *)0)->gridtype),
                _CFFI_OP(_CFFI_OP_NOOP, 64) },
  { "rangex", offsetof(ncplot_options, rangex),
              sizeof(((ncplot_options *)0)->rangex),
              _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "flags", offsetof(ncplot_options, flags),
             sizeof(((ncplot_options *)0)->flags),
             _CFFI_OP(_CFFI_OP_NOOP, 85) },
  { "title", offsetof(ncplot_options, title),
             sizeof(((ncplot_options *)0)->title),
             _CFFI_OP(_CFFI_OP_NOOP, 92) },
  { "flags", offsetof(ncplot_options, flags),
             sizeof(((ncplot_options *)0)->flags),
             _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "tchannels", offsetof(ncreader_options, tchannels),
                 sizeof(((ncreader_options *)0)->tchannels),
                 _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "tattrword", offsetof(ncreader_options, tattrword),
                 sizeof(((ncreader_options *)0)->tattrword),
                 _CFFI_OP(_CFFI_OP_NOOP, 259) },
  { "flags", offsetof(ncreader_options, flags),
             sizeof(((ncreader_options *)0)->flags),
             _CFFI_OP(_CFFI_OP_NOOP, 85) },
  { "bordermask", offsetof(ncreel_options, bordermask),
                  sizeof(((ncreel_options *)0)->bordermask),
                  _CFFI_OP(_CFFI_OP_NOOP, 85) },
  { "borderchan", offsetof(ncreel_options, borderchan),
                  sizeof(((ncreel_options *)0)->borderchan),
                  _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "tabletmask", offsetof(ncreel_options, tabletmask),
                  sizeof(((ncreel_options *)0)->tabletmask),
                  _CFFI_OP(_CFFI_OP_NOOP, 85) },
  { "tabletchan", offsetof(ncreel_options, tabletchan),
                  sizeof(((ncreel_options *)0)->tabletchan),
                  _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "focusedchan", offsetof(ncreel_options, focusedchan),
                   sizeof(((ncreel_options *)0)->focusedchan),
                   _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "flags", offsetof(ncreel_options, flags),
             sizeof(((ncreel_options *)0)->flags),
             _CFFI_OP(_CFFI_OP_NOOP, 85) },
  { "option", offsetof(struct ncselector_item, option),
              sizeof(((struct ncselector_item *)0)->option),
              _CFFI_OP(_CFFI_OP_NOOP, 86) },
  { "desc", offsetof(struct ncselector_item, desc),
            sizeof(((struct ncselector_item *)0)->desc),
            _CFFI_OP(_CFFI_OP_NOOP, 86) },
  { "opcolumns", offsetof(struct ncselector_item, opcolumns),
                 sizeof(((struct ncselector_item *)0)->opcolumns),
                 _CFFI_OP(_CFFI_OP_NOOP, 192) },
  { "desccolumns", offsetof(struct ncselector_item, desccolumns),
                   sizeof(((struct ncselector_item *)0)->desccolumns),
                   _CFFI_OP(_CFFI_OP_NOOP, 192) },
  { "title", offsetof(ncselector_options, title),
             sizeof(((ncselector_options *)0)->title),
             _CFFI_OP(_CFFI_OP_NOOP, 86) },
  { "secondary", offsetof(ncselector_options, secondary),
                 sizeof(((ncselector_options *)0)->secondary),
                 _CFFI_OP(_CFFI_OP_NOOP, 86) },
  { "footer", offsetof(ncselector_options, footer),
              sizeof(((ncselector_options *)0)->footer),
              _CFFI_OP(_CFFI_OP_NOOP, 86) },
  { "items", offsetof(ncselector_options, items),
             sizeof(((ncselector_options *)0)->items),
             _CFFI_OP(_CFFI_OP_NOOP, 837) },
  { "defidx", offsetof(ncselector_options, defidx),
              sizeof(((ncselector_options *)0)->defidx),
              _CFFI_OP(_CFFI_OP_NOOP, 85) },
  { "maxdisplay", offsetof(ncselector_options, maxdisplay),
                  sizeof(((ncselector_options *)0)->maxdisplay),
                  _CFFI_OP(_CFFI_OP_NOOP, 85) },
  { "opchannels", offsetof(ncselector_options, opchannels),
                  sizeof(((ncselector_options *)0)->opchannels),
                  _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "descchannels", offsetof(ncselector_options, descchannels),
                    sizeof(((ncselector_options *)0)->descchannels),
                    _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "titlechannels", offsetof(ncselector_options, titlechannels),
                     sizeof(((ncselector_options *)0)->titlechannels),
                     _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "footchannels", offsetof(ncselector_options, footchannels),
                    sizeof(((ncselector_options *)0)->footchannels),
                    _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "boxchannels", offsetof(ncselector_options, boxchannels),
                   sizeof(((ncselector_options *)0)->boxchannels),
                   _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "flags", offsetof(ncselector_options, flags),
             sizeof(((ncselector_options *)0)->flags),
             _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "renders", offsetof(ncstats, renders),
               sizeof(((ncstats *)0)->renders),
               _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "failed_renders", offsetof(ncstats, failed_renders),
                      sizeof(((ncstats *)0)->failed_renders),
                      _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "render_bytes", offsetof(ncstats, render_bytes),
                    sizeof(((ncstats *)0)->render_bytes),
                    _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "render_max_bytes", offsetof(ncstats, render_max_bytes),
                        sizeof(((ncstats *)0)->render_max_bytes),
                        _CFFI_OP(_CFFI_OP_NOOP, 807) },
  { "render_min_bytes", offsetof(ncstats, render_min_bytes),
                        sizeof(((ncstats *)0)->render_min_bytes),
                        _CFFI_OP(_CFFI_OP_NOOP, 807) },
  { "render_ns", offsetof(ncstats, render_ns),
                 sizeof(((ncstats *)0)->render_ns),
                 _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "render_max_ns", offsetof(ncstats, render_max_ns),
                     sizeof(((ncstats *)0)->render_max_ns),
                     _CFFI_OP(_CFFI_OP_NOOP, 807) },
  { "render_min_ns", offsetof(ncstats, render_min_ns),
                     sizeof(((ncstats *)0)->render_min_ns),
                     _CFFI_OP(_CFFI_OP_NOOP, 807) },
  { "cellelisions", offsetof(ncstats, cellelisions),
                    sizeof(((ncstats *)0)->cellelisions),
                    _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "cellemissions", offsetof(ncstats, cellemissions),
                     sizeof(((ncstats *)0)->cellemissions),
                     _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "fbbytes", offsetof(ncstats, fbbytes),
               sizeof(((ncstats *)0)->fbbytes),
               _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "fgelisions", offsetof(ncstats, fgelisions),
                  sizeof(((ncstats *)0)->fgelisions),
                  _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "fgemissions", offsetof(ncstats, fgemissions),
                   sizeof(((ncstats *)0)->fgemissions),
                   _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "bgelisions", offsetof(ncstats, bgelisions),
                  sizeof(((ncstats *)0)->bgelisions),
                  _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "bgemissions", offsetof(ncstats, bgemissions),
                   sizeof(((ncstats *)0)->bgemissions),
                   _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "defaultelisions", offsetof(ncstats, defaultelisions),
                       sizeof(((ncstats *)0)->defaultelisions),
                       _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "defaultemissions", offsetof(ncstats, defaultemissions),
                        sizeof(((ncstats *)0)->defaultemissions),
                        _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "curry", offsetof(ncsubproc_options, curry),
             sizeof(((ncsubproc_options *)0)->curry),
             _CFFI_OP(_CFFI_OP_NOOP, 187) },
  { "restart_period", offsetof(ncsubproc_options, restart_period),
                      sizeof(((ncsubproc_options *)0)->restart_period),
                      _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "flags", offsetof(ncsubproc_options, flags),
             sizeof(((ncsubproc_options *)0)->flags),
             _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "n", offsetof(struct ncvisual_options, n),
         sizeof(((struct ncvisual_options *)0)->n),
         _CFFI_OP(_CFFI_OP_NOOP, 12) },
  { "scaling", offsetof(struct ncvisual_options, scaling),
               sizeof(((struct ncvisual_options *)0)->scaling),
               _CFFI_OP(_CFFI_OP_NOOP, 67) },
  { "y", offsetof(struct ncvisual_options, y),
         sizeof(((struct ncvisual_options *)0)->y),
         _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "x", offsetof(struct ncvisual_options, x),
         sizeof(((struct ncvisual_options *)0)->x),
         _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "begy", offsetof(struct ncvisual_options, begy),
            sizeof(((struct ncvisual_options *)0)->begy),
            _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "begx", offsetof(struct ncvisual_options, begx),
            sizeof(((struct ncvisual_options *)0)->begx),
            _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "leny", offsetof(struct ncvisual_options, leny),
            sizeof(((struct ncvisual_options *)0)->leny),
            _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "lenx", offsetof(struct ncvisual_options, lenx),
            sizeof(((struct ncvisual_options *)0)->lenx),
            _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "blitter", offsetof(struct ncvisual_options, blitter),
               sizeof(((struct ncvisual_options *)0)->blitter),
               _CFFI_OP(_CFFI_OP_NOOP, 64) },
  { "flags", offsetof(struct ncvisual_options, flags),
             sizeof(((struct ncvisual_options *)0)->flags),
             _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "termtype", offsetof(notcurses_options, termtype),
                sizeof(((notcurses_options *)0)->termtype),
                _CFFI_OP(_CFFI_OP_NOOP, 92) },
  { "renderfp", offsetof(notcurses_options, renderfp),
                sizeof(((notcurses_options *)0)->renderfp),
                _CFFI_OP(_CFFI_OP_NOOP, 498) },
  { "loglevel", offsetof(notcurses_options, loglevel),
                sizeof(((notcurses_options *)0)->loglevel),
                _CFFI_OP(_CFFI_OP_NOOP, 810) },
  { "margin_t", offsetof(notcurses_options, margin_t),
                sizeof(((notcurses_options *)0)->margin_t),
                _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "margin_r", offsetof(notcurses_options, margin_r),
                sizeof(((notcurses_options *)0)->margin_r),
                _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "margin_b", offsetof(notcurses_options, margin_b),
                sizeof(((notcurses_options *)0)->margin_b),
                _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "margin_l", offsetof(notcurses_options, margin_l),
                sizeof(((notcurses_options *)0)->margin_l),
                _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "flags", offsetof(notcurses_options, flags),
             sizeof(((notcurses_options *)0)->flags),
             _CFFI_OP(_CFFI_OP_NOOP, 113) },
  { "chans", offsetof(palette256, chans),
             sizeof(((palette256 *)0)->chans),
             _CFFI_OP(_CFFI_OP_NOOP, 847) },
  { "x", (size_t)-1,
         (size_t)-1,
         _CFFI_OP(_CFFI_OP_NOOP, 38) },
  { "align", (size_t)-1,
             (size_t)-1,
             _CFFI_OP(_CFFI_OP_NOOP, 119) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "$1", 850, _CFFI_F_UNION|_CFFI_F_CHECK_FIELDS,
    (size_t)-2, -2, 120, 2 /* unnamed */ },
  { "_IO_FILE", 803, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "cell", 804, _CFFI_F_CHECK_FIELDS,
    sizeof(cell), offsetof(struct _cffi_align__cell, y), 0, 5 },
  { "ncdirect", 822, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncdplot", 823, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncfdplane", 824, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncfdplane_options", 808, _CFFI_F_CHECK_FIELDS,
    sizeof(ncfdplane_options), offsetof(struct _cffi_align__ncfdplane_options, y), 5, 3 },
  { "ncinput", 809, _CFFI_F_CHECK_FIELDS,
    sizeof(ncinput), offsetof(struct _cffi_align__ncinput, y), 8, 7 },
  { "ncmenu", 825, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncmenu_item", 827, _CFFI_F_CHECK_FIELDS,
    sizeof(struct ncmenu_item), offsetof(struct _cffi_align_struct_ncmenu_item, y), 15, 2 },
  { "ncmenu_options", 811, _CFFI_F_CHECK_FIELDS,
    sizeof(ncmenu_options), offsetof(struct _cffi_align__ncmenu_options, y), 17, 5 },
  { "ncmenu_section", 829, _CFFI_F_CHECK_FIELDS,
    sizeof(struct ncmenu_section), offsetof(struct _cffi_align_struct_ncmenu_section, y), 22, 4 },
  { "ncmselector_item", 831, _CFFI_F_CHECK_FIELDS,
    sizeof(struct ncmselector_item), offsetof(struct _cffi_align_struct_ncmselector_item, y), 26, 3 },
  { "ncmultiselector", 832, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncmultiselector_options", 812, _CFFI_F_CHECK_FIELDS,
    sizeof(ncmultiselector_options), offsetof(struct _cffi_align__ncmultiselector_options, y), 29, 11 },
  { "ncplane", 833, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncplane_options", 813, _CFFI_F_CHECK_FIELDS,
    sizeof(ncplane_options), offsetof(struct _cffi_align__ncplane_options, y), 40, 8 },
  { "ncplot_options", 814, _CFFI_F_CHECK_FIELDS,
    sizeof(ncplot_options), offsetof(struct _cffi_align__ncplot_options, y), 48, 8 },
  { "ncreader", 834, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncreader_options", 815, _CFFI_F_CHECK_FIELDS,
    sizeof(ncreader_options), offsetof(struct _cffi_align__ncreader_options, y), 56, 3 },
  { "ncreel", 835, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncreel_options", 816, _CFFI_F_CHECK_FIELDS,
    sizeof(ncreel_options), offsetof(struct _cffi_align__ncreel_options, y), 59, 6 },
  { "ncselector", 836, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncselector_item", 838, _CFFI_F_CHECK_FIELDS,
    sizeof(struct ncselector_item), offsetof(struct _cffi_align_struct_ncselector_item, y), 65, 4 },
  { "ncselector_options", 817, _CFFI_F_CHECK_FIELDS,
    sizeof(ncselector_options), offsetof(struct _cffi_align__ncselector_options, y), 69, 12 },
  { "ncstats", 818, _CFFI_F_CHECK_FIELDS,
    sizeof(ncstats), offsetof(struct _cffi_align__ncstats, y), 81, 17 },
  { "ncsubproc", 839, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncsubproc_options", 819, _CFFI_F_CHECK_FIELDS,
    sizeof(ncsubproc_options), offsetof(struct _cffi_align__ncsubproc_options, y), 98, 3 },
  { "nctablet", 840, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncuplot", 841, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncvisual", 842, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "ncvisual_options", 843, _CFFI_F_CHECK_FIELDS,
    sizeof(struct ncvisual_options), offsetof(struct _cffi_align_struct_ncvisual_options, y), 101, 10 },
  { "notcurses", 844, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "notcurses_options", 820, _CFFI_F_CHECK_FIELDS,
    sizeof(notcurses_options), offsetof(struct _cffi_align__notcurses_options, y), 111, 8 },
  { "palette256", 821, _CFFI_F_CHECK_FIELDS,
    sizeof(palette256), offsetof(struct _cffi_align__palette256, y), 119, 1 },
  { "timespec", 845, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
};

static const struct _cffi_enum_s _cffi_enums[] = {
  { "$ncalign_e", 119, _cffi_prim_int(sizeof(ncalign_e), ((ncalign_e)-1) <= 0),
    "NCALIGN_LEFT,NCALIGN_CENTER,NCALIGN_RIGHT" },
  { "$ncblitter_e", 64, _cffi_prim_int(sizeof(ncblitter_e), ((ncblitter_e)-1) <= 0),
    "NCBLIT_1x1,NCBLIT_2x1,NCBLIT_1x1x4,NCBLIT_2x2,NCBLIT_4x1,NCBLIT_BRAILLE,NCBLIT_8x1,NCBLIT_SIXEL" },
  { "$ncloglevel_e", 810, _cffi_prim_int(sizeof(ncloglevel_e), ((ncloglevel_e)-1) <= 0),
    "NCLOGLEVEL_SILENT,NCLOGLEVEL_PANIC,NCLOGLEVEL_FATAL,NCLOGLEVEL_ERROR,NCLOGLEVEL_WARNING,NCLOGLEVEL_INFO,NCLOGLEVEL_VERBOSE,NCLOGLEVEL_DEBUG,NCLOGLEVEL_TRACE" },
  { "$ncscale_e", 67, _cffi_prim_int(sizeof(ncscale_e), ((ncscale_e)-1) <= 0),
    "NCSCALE_NONE,NCSCALE_SCALE,NCSCALE_STRETCH" },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "FILE", 803 },
  { "cell", 804 },
  { "fadecb", 373 },
  { "ncalign_e", 119 },
  { "ncblitter_e", 64 },
  { "ncfdplane_callback", 189 },
  { "ncfdplane_done_cb", 184 },
  { "ncfdplane_options", 808 },
  { "ncinput", 809 },
  { "ncloglevel_e", 810 },
  { "ncmenu_options", 811 },
  { "ncmultiselector_options", 812 },
  { "ncplane_options", 813 },
  { "ncplot_options", 814 },
  { "ncreader_options", 815 },
  { "ncreel_options", 816 },
  { "ncscale_e", 67 },
  { "ncselector_options", 817 },
  { "ncstats", 818 },
  { "ncsubproc_options", 819 },
  { "notcurses_options", 820 },
  { "palette256", 821 },
  { "streamcb", 524 },
  { "tabletcb", 663 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  _cffi_enums,
  _cffi_typenames,
  262,  /* num_globals */
  36,  /* num_struct_unions */
  4,  /* num_enums */
  24,  /* num_typenames */
  NULL,  /* no includes */
  853,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__notcurses(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__notcurses(void) { return NULL; }
#  else
     init_notcurses(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__notcurses(void)
{
  return _cffi_init("_notcurses", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_notcurses(void)
{
  _cffi_init("_notcurses", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
